/*
 * Portions of this file are from:
 * 
 * ME3D 3-D Modeler Program
 * Copyright (C) 1998 Sam Revitch
 *
 * G3d modeler program
 * Copyright (C) 2000 Blake E Hegerle
 *
 * Both ME3d and G3d are distributed 
 * under the GPL, as is this file.
 *
 */

/*  
 * 
 * glpreview.c
 * 
 * handles the 3d vertex selection, polygon selection,
 * and opengl preview functionality
 * 
 */

#include "tri.h"
#include "globals.h"
#include "glpreview.h"

#include "memwatch.h"

void ed_glconfigure(struct model_view *view);
void ed_glviewport(struct model_view *view);
void ed_render(struct model_view *view);
gboolean ed_button_press(GtkWidget *widget, GdkEventButton *event,
                         struct model_view *view);

void ed_draw_cursor(struct model_view *view, GLdouble x, GLdouble y,
                    GLdouble z, GLdouble a);
gint ed_draw_vertices(struct model_view *view, struct vertex_l *vtx_list);
void ed_draw_girder(struct model_view *view, struct vertex_l *p1,
		   struct vertex_l *p2);

void draw_surfaces(struct model_view *view, struct model_group *group,
		   gint box_it);
void draw_normals(struct model_group *group);
void draw_grid(GLfloat *color, GLfloat *mins, GLfloat *maxes, GLfloat *spacing);
struct vertex_l *ed_test_click_vertex(struct model_view *view,
				      GLfloat *z, guint x, guint y);
struct surface_l *ed_test_click_surface(struct model_view *view, GLfloat *z,
					guint xc, guint yc);


//////////////////////////////////////////////////////////////////////////
//
//
//  OLD STUFF, from g3d
//
//
///////////////////////////////////////////////////////////////////////////



GtkWidget *glarea;
// tris *glpt;
int glx, gly;
double rotx = 0, roty = 0;

GtkWidget *gltable, *ghs, *gvs;

gint vadj_changed( GtkWidget *widget, gpointer data ) {
    rotx = ( double ) GTK_ADJUSTMENT( widget ) ->value;
    gtk_widget_draw( glarea, NULL );

    return TRUE;
}

gint hadj_changed( GtkWidget *widget, gpointer data ) {
    roty = ( double ) GTK_ADJUSTMENT( widget ) ->value;
    gtk_widget_draw( glarea, NULL );

    return TRUE;
}

void texenv( void ) {
    GLfloat env_color[] = {0, 1, 0, 1};
    GLfloat border_color[] = {1, 0, 0, 1};

    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER,
                     GL_NEAREST );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
    glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
    glTexEnvi( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL );
    glTexEnvfv( GL_TEXTURE_ENV, GL_TEXTURE_ENV_COLOR, env_color );
    glTexParameterfv( GL_TEXTURE_2D, GL_TEXTURE_BORDER_COLOR, border_color );
    glEnable( GL_TEXTURE_2D );
}

gint glinit( GtkWidget *widget, gpointer data ) {
    if ( gtk_gl_area_make_current( GTK_GL_AREA( widget ) ) ) {
        texenv();
        glViewport( 0, 0, widget->requisition.width, widget->requisition.height );
        glMatrixMode( GL_PROJECTION );
        glLoadIdentity();
        glFrustum( -100, 100, -100, 100, 100, 300 );
        glMatrixMode( GL_MODELVIEW );
        glLoadIdentity();
        glPolygonMode( GL_FRONT_AND_BACK, GL_FILL );
        glEnable( GL_DEPTH_TEST );
    }
    return 1;
}

gint glreshape( GtkWidget *widget, GdkEventConfigure *event, gpointer data ) {
    if ( gtk_gl_area_make_current( GTK_GL_AREA( widget ) ) )
        glViewport( 0, 0, event->width, event->height );

    return TRUE;
}

void tex_make_current( tex *ct ) {
    glTexImage2D( GL_TEXTURE_2D, 0, ct->depth, ct->width, ct->height, 0,
                  ct->depth == 4 ? GL_RGBA : GL_RGB, GL_UNSIGNED_BYTE,
                  ct->data );
}

gint gldraw( GtkWidget *widget, GdkEventExpose *event, gpointer data ) {
    GList * cl;
    Tri *t;
    static tex *active_texture = NULL;

    if ( event->count > 0 )
        return TRUE;

    if ( GTK_WIDGET_VISIBLE( gltable ) ) {
        if ( gtk_gl_area_make_current( GTK_GL_AREA( widget ) ) ) {
            glClearColor( 0, 0, 0, 0 );
            glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
            glColor3f( 1, 1, 1 );
            glMatrixMode( GL_MODELVIEW );
            glLoadIdentity();

            glTranslated( 0, 0, -190 );

            glRotated( rotx, 1, 0, 0 );
            glRotated( roty, 0, 1, 0 );

            glScaled( 1, -1, 1 );
            glTranslated( -50, -50, -50 );

            for ( cl = this_model->triangles; cl; cl = cl->next ) {
                t = ( Tri * ) cl->data;
                if ( t->texture != NULL ) {
                    if ( active_texture != t->texture ) {
                        active_texture = t->texture;
                        tex_make_current( t->texture );
                    }
                    glEnable( GL_TEXTURE_2D );
                    glBegin( GL_TRIANGLES );
                    glTexCoord2d( t->tv[ 0 ] ->x,
                                  t->tv[ 0 ] ->y );
                    glVertex3d( t->verticies[ 0 ] ->v[ 0 ],
                                t->verticies[ 0 ] ->v[ 1 ],
                                t->verticies[ 0 ] ->v[ 2 ] );
                    glTexCoord2d( t->tv[ 1 ] ->x,
                                  t->tv[ 1 ] ->y );
                    glVertex3d( t->verticies[ 1 ] ->v[ 0 ],
                                t->verticies[ 1 ] ->v[ 1 ],
                                t->verticies[ 1 ] ->v[ 2 ] );
                    glTexCoord2d( t->tv[ 2 ] ->x,
                                  t->tv[ 2 ] ->y );
                    glVertex3d( t->verticies[ 2 ] ->v[ 0 ],
                                t->verticies[ 2 ] ->v[ 1 ],
                                t->verticies[ 2 ] ->v[ 2 ] );
                    glEnd();
                } else {
                    glDisable( GL_TEXTURE_2D );
                    glBegin( GL_TRIANGLES );
                    glVertex3d( t->verticies[ 0 ] ->v[ 0 ],
                                t->verticies[ 0 ] ->v[ 1 ],
                                t->verticies[ 0 ] ->v[ 2 ] );
                    glVertex3d( t->verticies[ 1 ] ->v[ 0 ],
                                t->verticies[ 1 ] ->v[ 1 ],
                                t->verticies[ 1 ] ->v[ 2 ] );
                    glVertex3d( t->verticies[ 2 ] ->v[ 0 ],
                                t->verticies[ 2 ] ->v[ 1 ],
                                t->verticies[ 2 ] ->v[ 2 ] );
                    glEnd();
                }
            }

            gtk_gl_area_swapbuffers( GTK_GL_AREA( widget ) );
        }
    }

    return 1;
}

GtkWidget *make_new_gl_preview( void ) {
    GtkAdjustment * hadjustment, *vadjustment;
    GtkWidget *frame;

    int attrlist[] = {
        GDK_GL_RGBA,
        GDK_GL_DOUBLEBUFFER,
        GDK_GL_NONE
    };

    if ( gdk_gl_query() == FALSE ) {
        g_print( "OpenGL not available\n" );
        return NULL;
    }

    glarea = GTK_WIDGET( gtk_gl_area_new( attrlist ) );
    // gtk_widget_set_usize(glarea, 278, 258);

    gtk_signal_connect( GTK_OBJECT( glarea ), "expose_event",
                        GTK_SIGNAL_FUNC( gldraw ), NULL );
    gtk_signal_connect( GTK_OBJECT( glarea ), "configure_event",
                        GTK_SIGNAL_FUNC( glreshape ), NULL );
    gtk_signal_connect( GTK_OBJECT( glarea ), "realize",
                        GTK_SIGNAL_FUNC( glinit ), NULL );
    gtk_widget_set_events( glarea, GDK_EXPOSURE_MASK );
    frame = gtk_frame_new( NULL );
    gtk_frame_set_shadow_type( GTK_FRAME( frame ), GTK_SHADOW_OUT );
    gtk_container_add( GTK_CONTAINER( frame ), glarea );
    gtk_widget_show( frame );

    gltable = gtk_table_new( 2, 2, FALSE );

    gtk_table_attach( GTK_TABLE( gltable ), frame, 0, 1, 0, 1,
                      GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL, 0, 0 );

    vadjustment = GTK_ADJUSTMENT( gtk_adjustment_new( 0, 0, 420, 5, 30, 60 ) );
    gtk_signal_connect ( GTK_OBJECT ( vadjustment ), "value_changed",
                         GTK_SIGNAL_FUNC ( vadj_changed ), NULL );
    gvs = gtk_vscrollbar_new( vadjustment );
    gtk_widget_show( gvs );
    gtk_table_attach( GTK_TABLE( gltable ), gvs, 1, 2, 0, 1,
                      GTK_SHRINK, GTK_EXPAND | GTK_FILL, 0, 0 );

    hadjustment = GTK_ADJUSTMENT( gtk_adjustment_new( 0, 0, 420, 5, 30, 60 ) );
    gtk_signal_connect ( GTK_OBJECT ( hadjustment ), "value_changed",
                         GTK_SIGNAL_FUNC ( hadj_changed ), NULL );
    ghs = gtk_hscrollbar_new( hadjustment );
    gtk_widget_show( ghs );
    gtk_table_attach( GTK_TABLE( gltable ), ghs, 0, 1, 1, 2,
                      GTK_EXPAND | GTK_FILL, GTK_SHRINK, 0, 0 );

    gtk_widget_show( glarea );
    gtk_widget_show( gltable );

    return gltable;
}









//////////////////////////////////////////////////////////////////////////
//
//
//  NEW STUFF, from me3d
//
//
//////////////////////////////////////////////////////////////////////////



void ed_glconfigure(struct model_view *view)
{
  GLfloat whitelight[] = { 1.0, 1.0, 1.0, 0.5 };
  GLfloat amblight[] = { 0.2, 0.2, 0.1, 1.0 };
  GLfloat lightpos[] = { -5.0, 5.0, -2.0, 1.0 };

//  GdkImlibImage *iimg;

  /* clear frame color */
  glClearColor(view->config.bkgnd_color[0], view->config.bkgnd_color[1],
	       view->config.bkgnd_color[2], 0.0);

  /* These are the same under all circumstances */
  glEnable(GL_DEPTH_TEST);
  glEnable(GL_CULL_FACE);
  glPolygonMode(GL_FRONT, GL_FILL);

  /* lighting is the same (for now) */
  glEnable(GL_LIGHT0);
  glLightfv(GL_LIGHT0, GL_POSITION, lightpos);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, whitelight);
  glLightfv(GL_LIGHT0, GL_AMBIENT, amblight);

  if(view->pDoc->config.clockwise)
    glFrontFace(GL_CW);
  else
    glFrontFace(GL_CCW);

  if(view->pDoc->config.smooth_mode)
    glShadeModel(GL_SMOOTH);
  else
    glShadeModel(GL_FLAT);

  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();

/*
  iimg = gdk_imlib_load_image("backgnd.gif");
  if(iimg != NULL) {
    view->active_tex = doctex_convert_imlib(iimg);

    glTexImage2D(GL_TEXTURE_2D, 0, view->active_tex->ifmt,
	view->active_tex->gl_width, view->active_tex->gl_height,
	0, view->active_tex->fmt, GL_UNSIGNED_BYTE,
	view->active_tex->gl_data);

    gluBuild2DMipmaps(GL_TEXTURE_2D, view->active_tex->ifmt,
                      view->active_tex->gl_width, view->active_tex->gl_height,
		      view->active_tex->fmt, GL_UNSIGNED_BYTE,
                      view->active_tex->gl_data);
    glGenTextures(1, &(view->active_tex->tex_id));
    glBindTexture(GL_TEXTURE_2D, view->active_tex->tex_id);
  }
*/

}


void ed_glviewport(struct model_view *view)
{
  /* Set the camera */
  if(GTK_WIDGET_REALIZED(GTK_WIDGET(view->GLctx)) &&
     (GTK_WIDGET(view->GLctx)->allocation.width > 1) &&
     (GTK_WIDGET(view->GLctx)->allocation.height > 1))
  {
    GLfloat xd, yd, hwr, fov_rad;

    g_return_if_fail(gtk_gl_area_make_current(GTK_GL_AREA(view->GLctx)));

    glViewport(0, 0, GTK_WIDGET(view->GLctx)->allocation.width,
	       GTK_WIDGET(view->GLctx)->allocation.height);
    hwr = (GLfloat) GTK_WIDGET(view->GLctx)->allocation.height /
      GTK_WIDGET(view->GLctx)->allocation.width;

    fov_rad = (view->config.fov * M_PI) / 360;
    xd = view->config.near * tan(fov_rad);
    yd = xd * hwr;
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glFrustum(-xd, xd, -yd, yd, view->config.near, view->config.far);
    glMatrixMode(GL_MODELVIEW);

#ifdef VERBOSE_DEBUG
    g_print("ed_glviewport %ux%u %f->%f (%f) %f %f %f\n",
            GTK_WIDGET(view->GLctx)->allocation.width,
	    GTK_WIDGET(view->GLctx)->allocation.height,
	    view->config.near, view->config.far, view->config.fov,
            xd, yd, hwr);
#endif
  }
}


void ed_render(struct model_view *view)
{
  gint animated = False;
  static GLfloat initializer[4] = { 0.0, 0.0, 0.0, 1.0 };

  g_return_if_fail(view->active_group != NULL);

  glDisable(GL_LIGHTING);
  glColor3f(1.0, 1.0, 1.0);

#if 0
  glEnable(GL_TEXTURE_2D);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
  glTexEnvi(GL_TEXTURE_2D, GL_TEXTURE_ENV_MODE, GL_DECAL);

  glBegin(GL_QUADS);
  glTexCoord2f(1.0, 0.0);  glVertex3f(60.0, 80.0, -75.0);
  glTexCoord2f(0.0, 0.0);  glVertex3f(-60.0, 80.0, -75.0);
  glTexCoord2f(0.0, 1.0);  glVertex3f(-60.0, -80.0, -75.0);
  glTexCoord2f(1.0, 1.0);  glVertex3f(60.0, -80.0, -75.0);
  glEnd();
  glDisable(GL_TEXTURE_2D);
#endif

  glPushMatrix();
  glTranslatef(view->camX, view->camY, view->camZ-(view->dist + view->config.near));
  glPushMatrix();
  glRotatef(-(view->pitch), 1.0, 0.0, 0.0);
  glPushMatrix();
  glRotatef(view->spin, 0.0, 1.0, 0.0);

  if(view->config.grid_visible)
    draw_grid(view->config.grid_color, view->config.grid_mins,
	      view->config.grid_maxes, view->config.grid_spacing);
  if(view->config.normals_visible)
    draw_normals(view->active_group);


  switch(view->config.polygon_mode)
  {
    case MP_SHOW_WIREFRAME:		/* wire frame mode is non-shaded */
      glColor3f(1.0, 1.0, 1.0);		/* unfortunately not much faster */
      glPolygonMode(GL_FRONT, GL_LINE);
      draw_surfaces(view, view->active_group, False);
      glPolygonMode(GL_FRONT, GL_FILL);
      glEnable(GL_LIGHTING);
      break;
    case MP_SHOW_SURFACES:
      glEnable(GL_LIGHTING);
      draw_surfaces(view, view->active_group, False);
      break;
    default:
      break;
  }

  /* necessary for miscellaneous stuff */
  glEnable(GL_COLOR_MATERIAL);
  glColorMaterial(GL_FRONT, GL_AMBIENT_AND_DIFFUSE);
  glMaterialfv(GL_FRONT, GL_SPECULAR, initializer);
  glMaterialfv(GL_FRONT, GL_EMISSION, initializer);
  glMaterialf(GL_FRONT, GL_SHININESS, 0.0);

  if(view->config.vertices_visible)
    animated = ed_draw_vertices(view, view->active_group->vertex_list);
  if(cursor_visible) {
    ed_draw_cursor(view, view->cursX, view->cursY, view->cursZ,
		   view->cursA);
    if(!animated)
      view->cursA += 5;
  }

  if(view->surface_focused != NULL)
    SURFACE_RENDER_EDIT(view, view->surface_focused);

  glPopMatrix();
  glPopMatrix();
  glPopMatrix();

  if((!cursor_visible) && (!animated))
    view->animated = False;
}




gboolean ed_button_press(GtkWidget *widget, GdkEventButton *event,
			 struct model_view *view)
{
  int rerender = False;
  GLenum poly_mode = GL_FALSE;
  gint exp_polygon = False;
  struct vertex_l *vtx;
  struct surface_l *surf;
  GLfloat z;

  if(event->button == 3)
  {
    ed_configure_menu(view);
    docmenu_show(view, event);
    return True;
  }

  switch(op_mode)
  {
    case M_SELECT:
      vtx = ed_test_click_vertex(view, &z, event->x, event->y);
      if(vtx != NULL)
      {
	if(vtx->selected)
	  doc_deselect(view->pDoc, view->active_group, vtx);
	else
	  doc_select(view->pDoc, view->active_group, vtx);
	rerender = True;
      }
      break;

    case M_POLY_SEL:
      surf = ed_test_click_surface(view, &z, event->x, event->y);
      if(surf != NULL)
      {
	if(view->surface_focused != NULL) {
	  doc_surface_commit(view->pDoc, view->active_group, view->surface_focused);
	  view->surface_focused = NULL;
	}
	doc_surface_decommit(view->pDoc, view->active_group, surf);
	view->surface_focused = surf;
	rerender = True;
      }
      break;

    case M_TRIANGLES:
      poly_mode = GL_TRIANGLES;
      exp_polygon = True;
      break;
    case M_QUADS:
      poly_mode = GL_QUADS;
      exp_polygon = True;
      break;
    case M_TRIANGLE_STRIP:
      poly_mode = GL_TRIANGLE_STRIP;
      exp_polygon = True;
      break;
    case M_QUAD_STRIP:
      poly_mode = GL_QUAD_STRIP;
      exp_polygon = True;
      break;
    case M_TRIANGLE_FAN:
      poly_mode = GL_TRIANGLE_FAN;
      exp_polygon = True;
      break;
    case M_POLYGON:
      poly_mode = GL_POLYGON;
      exp_polygon = True;
      break;
    case M_SOR:
      exp_polygon = True;
      break;

    default:
      break;
  }

  if(exp_polygon) {
    vtx = ed_test_click_vertex(view, &z, event->x, event->y);
    if(vtx != NULL) {
      if(view->surface_focused == NULL) {
	if(op_mode == M_POLYGON)
	  view->surface_focused = polygon_tess_new(&(view->pDoc->config.s_prefs));
	else if (op_mode == M_SOR)
	  view->surface_focused = sor_new (&(view->pDoc->config.s_prefs));
	else
	  view->surface_focused =
	    primitive_new(&(view->pDoc->config.s_prefs), poly_mode);

	doc_surface_material_gtpv(view->pDoc, view->surface_focused,
				  GTK_GL_TEX_PREVIEW(cps->sample_area));
      }
      if(op_mode == M_POLYGON)
	rerender = polygon_tess_attach_vertex(view->surface_focused, vtx);
      else
	rerender = primitive_attach_vertex(view->surface_focused, vtx);
    }
  }

  if(rerender)
    view_start_rendercycle_docwide(view->pDoc);

  return True;
}


void ed_draw_cursor(struct model_view *view, GLdouble x, GLdouble y,
		    GLdouble z, GLdouble a)
{
  glPushMatrix();
  glTranslatef(x, y, z);
  glPushMatrix();
  glRotatef(a, 0, 1, 0);

  glInterleavedArrays(GL_C4F_N3F_V3F, 0, view->cursor_glarray);
  glDrawArrays(GL_TRIANGLES, 0, 24);

  glPopMatrix();
  glPopMatrix();
}

gint ed_draw_vertices(struct model_view *view, struct vertex_l *vtx_list)
{
  gint rval = False;
  glInterleavedArrays(GL_N3F_V3F, 0, view->vertex_glarray);

  view->cursA += 5;
  if(view->cursA >= 360)
    view->cursA -= 360;

  while(vtx_list != NULL)
  {
    if(vtx_list->selected) {
      switch(vtx_list->prefs.override_mode) {
        case VSO_DONT:
	  glColor3f(VTX_COLOR_NORMAL_SEL);
	  break;
        case VSO_NONCONNECTIVE:
        case VSO_CONNECTIVE:
	  glColor3f(VTX_COLOR_CONNECTIVE_SEL);
	  break;
        case VSO_ABSOLUTE_FLAT:
        case VSO_ABSOLUTE_SMOOTH:
	  glColor3f(VTX_COLOR_ABSOLUTE_SEL);
	  break;
      }
      glPushMatrix();
      glTranslatef(vtx_list->v[0], vtx_list->v[1], vtx_list->v[2]);
      glPushMatrix();
      glRotatef(view->cursA, 0, 1, 0);
      glDrawArrays(GL_QUADS, 0, 24);
      glPopMatrix();
      glPopMatrix();
      rval = True;
    } else {
      switch(vtx_list->prefs.override_mode) {
        case VSO_DONT:
	  glColor3f(VTX_COLOR_NORMAL);
	  break;
        case VSO_NONCONNECTIVE:
        case VSO_CONNECTIVE:
	  glColor3f(VTX_COLOR_CONNECTIVE);
	  break;
        case VSO_ABSOLUTE_FLAT:
        case VSO_ABSOLUTE_SMOOTH:
	  glColor3f(VTX_COLOR_ABSOLUTE);
	  break;
      }
      glPushMatrix();
      glTranslatef(vtx_list->v[0], vtx_list->v[1], vtx_list->v[2]);
      glDrawArrays(GL_QUADS, 0, 24);
      glPopMatrix();
    }

    vtx_list = vtx_list->pNext;
  }

  return rval;
}

void ed_draw_girder(struct model_view *view, struct vertex_l *p1,
		    struct vertex_l *p2)
{
  GLfloat length, v1[3], n[2], sin_theta, theta;
  struct vertex_l *temp;

  if(p1->v[2] > p2->v[2] ) {	    	/* +/- issue with arcsin */
    temp = p2;  p2 = p1;  p1 = temp;
  }					/* what a pain */

  v1[0] = p2->v[0] - p1->v[0];
  v1[1] = p2->v[1] - p1->v[1];
  v1[2] = p2->v[2] - p1->v[2];

  sin_theta = (v1[0] * v1[0]) + (v1[1] * v1[1]);
  length = sqrt(sin_theta + (v1[2] * v1[2]));
  sin_theta = sqrt(sin_theta);
  if(sin_theta == 0) {
    n[0] = 1;		/* doesn't matter */
    n[1] = 0;
  } else {
    n[0] = -v1[1] / sin_theta;
    n[1] = v1[0] / sin_theta;
  }
  sin_theta = sin_theta / length;
  theta = asin(sin_theta) * ( (GLdouble) 180 / M_PI );
#ifdef VERBOSE
  printf("v = <%f, %f, %f> ", v1[0], v1[1], v1[2]);
  printf("n = <%f, %f, 0> theta = %f\n", n[0], n[1], theta);
#endif

  /* Set the remap matrix appropriately */
  glPushMatrix();
  glTranslated(p1->v[0], p1->v[1], p1->v[2]);
  glPushMatrix();
  glRotatef(theta, n[0], n[1], 0);

  view->girder_glarray[17] = length;		/* Patch the array */
  view->girder_glarray[23] = length;
  view->girder_glarray[41] = length;
  view->girder_glarray[47] = length;
  view->girder_glarray[65] = length;
  view->girder_glarray[71] = length;
  view->girder_glarray[89] = length;
  view->girder_glarray[95] = length;

  glColor3f(GIRDER_COLOR);
  glInterleavedArrays(GL_N3F_V3F, 0, view->girder_glarray);
  glDrawArrays(GL_QUADS, 0, 16);

  glPopMatrix();
  glPopMatrix();
}


struct vertex_l *ed_test_click_vertex(struct model_view *view,
				      GLfloat *z, guint x, guint y)
{
  GLdouble best_z = 9999999.0, click_r, cr1, cr2, cr3;
  GLfloat array[8];
  struct vertex_l *best_vert = NULL, *pV;

  g_return_val_if_fail (view != NULL, NULL);
  g_return_val_if_fail (view->GLctx != NULL, NULL);

  /* fragments for computing the approximate vertex radius in a frustum perspective */
  /* Ze = (near*far) / (far - Zwin*(far-near)) */
  /* but these fragments are to find the size of the pixel square for each vertex */
  /* i.e. winradius / (Ze * tan(fov/2)) */
  if(view->config.enlarge_vertices)
    cr1 = (view->config.vertex_rad[1] * M_SQRT2 * view->GLctx->allocation.width) /
      (2 * view->config.near * view->config.far * tan(view->config.fov / 2));
  else
    cr1 = (view->config.vertex_rad[0] * M_SQRT2 * view->GLctx->allocation.width) /
      (2 * view->config.near * view->config.far * tan(view->config.fov / 2));
  cr2 = (view->config.far);
  cr3 = (view->config.far - view->config.near);
  /* the maximum vertex radius is sqrt(3)*VTX_RAD, the minimum is VTX_RAD */

  g_return_if_fail(gtk_gl_area_make_current(GTK_GL_AREA(view->GLctx)));
  glFeedbackBuffer(8, GL_3D, array);
  glRenderMode(GL_FEEDBACK);

  /* for some reason it seems to flip the y-axis.  WHY?! */
  y = view->GLctx->allocation.height - y;

  glPushMatrix();
  glTranslatef(view->camX, view->camY, view->camZ-(view->dist + view->config.near));
  glPushMatrix();
  glRotatef(-(view->pitch), 1.0, 0.0, 0.0);
  glPushMatrix();
  glRotatef(view->spin, 0.0, 1.0, 0.0);

  pV = view->active_group->vertex_list;
  while(pV != NULL)
  {
    glBegin(GL_POINTS);
    glVertex3dv(&(pV->v[0]));
    glEnd();

    /*    printf("glRenderMode returns %i\n", glRenderMode(GL_FEEDBACK)); */

    if(4 != glRenderMode(GL_FEEDBACK)) {
      pV = pV->pNext;
      continue;			/* Point isn't within clipping volume */
    }

    /* Compute click radius */
    click_r = cr1 * (cr2 - (cr3 * array[3]));	/* assemble fragments */

    array[1] += 0.5;		/* round to nearest integer */
    array[2] += 0.5;		/* normally float->int just */
    click_r += 0.5;		/* chops off the decimal point and beyond */

    /*    d = (view->near * view->far) /
	  (doc->view - (array[3] * (view->far - view->near))); */

    /*    printf("vertex <%f,%f,%f> @ (%8.8f,%8.8f) r=%8.8f\n",
	  pV->v[0], pV->v[1], pV->v[2], array[1], array[2], click_r); */

    if((array[1] + click_r >= x) &&
       (array[1] - click_r <= x) &&
       (array[2] + click_r >= y) &&
       (array[2] - click_r <= y) &&
       (array[3] < best_z))
    {
      best_vert = pV;
      best_z = array[3];
    }

    pV = pV->pNext;
  }

  glPopMatrix();
  glPopMatrix();
  glPopMatrix();
  glRenderMode(GL_RENDER);
  *z = best_z;
  return best_vert;
}

#define ED_ARRAY_SIZE 2048
struct surface_l *ed_test_click_surface(struct model_view *view,
					GLfloat *z, guint xc, guint yc)
{
  GLdouble best_z = 9999999.0, bz_vert, m, b, x, y;
  GLfloat array[ED_ARRAY_SIZE], *v1, *v2;
  struct surface_l *best_surf = NULL, *pS;
  int i, j, len, polylen, intersections;

  x = xc;
  y = yc;

  g_return_val_if_fail (view != NULL, NULL);
  g_return_val_if_fail (view->GLctx != NULL, NULL);

  g_return_if_fail(gtk_gl_area_make_current(GTK_GL_AREA(view->GLctx)));
  glFeedbackBuffer(ED_ARRAY_SIZE, GL_3D, array);
  glRenderMode(GL_FEEDBACK);

  /* for some reason it seems to flip the y-axis.  WHY?! */
  y = view->GLctx->allocation.height - y;

  glPushMatrix();
  glTranslatef(view->camX, view->camY, view->camZ-(view->dist + view->config.near));
  glPushMatrix();
  glRotatef(-(view->pitch), 1.0, 0.0, 0.0);
  glPushMatrix();
  glRotatef(view->spin, 0.0, 1.0, 0.0);

  pS = view->active_group->surface_list;
  while(pS != NULL)
  {
    SURFACE_RENDER(view, pS);

    /* now process the feedback buffer and check for hits */
    len = glRenderMode(GL_FEEDBACK);
    intersections = 0;
    for(i = 0; i < len; )
    {
      g_return_val_if_fail(array[i] == GL_POLYGON_TOKEN, NULL);

      i++;
      polylen = (int)array[i];
      i++;

      bz_vert = 9999999.0;
      v1 = &(array[3*(polylen - 1) + i]);
      for(j = 0; j < polylen; j++)
      {
	v2 = v1;
	v1 = &(array[i]);
	i += 3;

	if(array[2] < bz_vert)
	  bz_vert = array[2];

	/* try to eliminate the line represented by v1 and v2 */

	/* in order to intersect the test line, the v1<->v2 line */
	/* must have a vertex to both sides of the click point */
	if( !((v1[0] < x) && (v2[0] > x)) &&
	    !((v2[0] < x) && (v1[0] > x)) )
	  continue;
	/* note that it's now impossible for v1[0] == v2[0] */

	/* the line can be immediately qualified if both are above */
	/* the click point */
	if( (v1[1] < y) && (v2[1] < y) )
	{
	  intersections++;
	  continue;
	}

	/* if both vertices are below the click point, the line */
	/* can't intersect */
	if( (v1[1] > y) && (v2[1] > y) )
	  continue;

	/* now the situation is sticky, the line formula must be used */ 
	/* to test for intersection */

	m = (v2[1] - v1[1]) / (v2[0] - v1[0]);
	b = v1[1] - (m * v1[0]);

	if(y < ((m * x) + b))
	  continue;

	intersections++;
      }

      intersections &= 0x1;	/* an odd number of intersections means */
                                /* it was clicked on */
      if(intersections && (bz_vert < best_z))
      {
	best_surf = pS;
	best_z = bz_vert;
      }
    }

    pS = pS->pNext;
  }	/* pS != NULL */

  glPopMatrix();
  glPopMatrix();
  glPopMatrix();
  glRenderMode(GL_RENDER);
  *z = best_z;
  return best_surf;
}

void draw_surfaces(struct model_view *view, struct model_group *group,
		   gint box_it)
{
  static struct material_obj default_material = {
    { 0.8, 0.8, 0.8, 1.0 },
    { 0.2, 0.2, 0.2, 1.0 },
    { 0.0, 0.0, 0.0, 1.0 },
    { 0.0, 0.0, 0.0, 1.0 },
    0
  };

  struct surface_l *pS;
  struct model_group *pG;
  struct material_obj *material;

  pS = group->surface_list;
  while(pS != NULL)
  {
    g_return_if_fail(pS->type < SURF_NTYPES);
    g_return_if_fail(surface_funcs[pS->type]->render != NULL);

    if(pS->material != NULL)
      material = pS->material;
    else
      material = &default_material;

    if(pS->use_material_color) {
      glEnable(GL_COLOR_MATERIAL);
      glMaterialfv(GL_FRONT, GL_DIFFUSE, material->diff);
      glMaterialfv(GL_FRONT, GL_AMBIENT, material->amb);
      glMaterialfv(GL_FRONT, GL_SPECULAR, material->spec);
      glMaterialfv(GL_FRONT, GL_EMISSION, material->emis);
      glMaterialf(GL_FRONT, GL_SHININESS, material->shininess);
      glColorMaterial(GL_FRONT, pS->material_color_mode);
    } else {
      glDisable(GL_COLOR_MATERIAL);
      glMaterialfv(GL_FRONT, GL_DIFFUSE, material->diff);
      glMaterialfv(GL_FRONT, GL_AMBIENT, material->amb);
      glMaterialfv(GL_FRONT, GL_SPECULAR, material->spec);
      glMaterialfv(GL_FRONT, GL_EMISSION, material->emis);
      glMaterialf(GL_FRONT, GL_SHININESS, material->shininess);
    }

    SURFACE_RENDER(view, pS);

    pS = pS->pNext;
  }

  pG = group->pDown;
  while(pG != NULL) {
    glPushMatrix();
    glTranslated(pG->trans[0], pG->trans[1], pG->trans[2]);
    draw_surfaces(view, pG, True);
    glPopMatrix();

    pG = pG->pNext;
  }
}

void draw_normals(struct model_group *group)
{
  struct surface_link *pL;
  struct surface_l *pS;

  pS = group->surface_list;

  if(pS == NULL)
    return;

  glColor3f(1.0, 1.0, 1.0);
  glBegin(GL_LINES);

  while(pS != NULL)
  {
    pL = pS->sl_list;
    g_return_if_fail(pL != NULL);
    while(pL != NULL)
    {
      if((pL->normal[0] != 0) || (pL->normal[1] != 0) || (pL->normal[2] != 0))
      {
	glVertex3dv(&(pL->vert->v[0]));
	glVertex3d(pL->vert->v[0] + pL->normal[0], pL->vert->v[1] + pL->normal[1],
		   pL->vert->v[2] + pL->normal[2]);
      }
      pL = pL->s_next;
    }
    pS = pS->pNext;
  }

  glEnd();
}


void draw_grid(GLfloat *color, GLfloat *mins, GLfloat *maxes, GLfloat *spacing)
{
  GLfloat coord;

  glColor3fv(color);
  glBegin(GL_LINES);

  for(coord = mins[0]; coord <= maxes[0]; coord += spacing[0]) {
    glVertex3f(coord, 0, mins[2]);
    glVertex3f(coord, 0, maxes[2]);
    glVertex3f(coord, mins[1], 0);
    glVertex3f(coord, maxes[1], 0);
  }

  for(coord = mins[1]; coord <= maxes[1]; coord += spacing[1]) {
    glVertex3f(0, coord, mins[2]);
    glVertex3f(0, coord, maxes[2]);
    glVertex3f(mins[1], coord, 0);
    glVertex3f(maxes[1], coord, 0);
  }

  for(coord = mins[2]; coord <= maxes[2]; coord += spacing[2]) {
    glVertex3f(0, mins[1], coord);
    glVertex3f(0, maxes[1], coord);
    glVertex3f(mins[0], 0, coord);
    glVertex3f(maxes[0], 0, coord);
  }

  glEnd();
}


///////////////////////////////////////////////////////////////////////////////
//
//
// ONLY DEAD CODE BELOW THIS POINT
//
//
///////////////////////////////////////////////////////////////////////////////


#if 0

extern gboolean ed_key_press(GtkWidget *widget, GdkEventKey *event,
			     struct model_view *view);
extern void ed_set_mode(struct model_view *view, guint op_mode);

void ed_configure_menu(struct model_view *view);
gboolean ed_return_pressed(struct model_view *view);
gboolean ed_delete_pressed(struct model_view *view);
gboolean ed_escape_pressed(struct model_view *view);



gboolean ed_key_press(GtkWidget *widget, GdkEventKey *event,
		      struct model_view *view)
{
  int rerender = False, rerender_view = False, rerender_statusbar = False,
      reconfigure_view = False;
  GLdouble iX = 0, iY = 0, iZ = 0;

  switch(event->keyval)
  {
    /* Cursor translation */
    case 'e':
      iX += 1.0;
      break;
    case 'q':
      iX -= 1.0;
      break;
    case 'E':
      iX += 0.1;
      break;
    case 'Q':
      iX -= 0.1;
      break;
    case 'w':
      iY += 1.0;
      break;
    case 'x':
      iY -= 1.0;
      break;
    case 'W':
      iY += 0.1;
      break;
    case 'X':
      iY -= 0.1;
      break;
    case 'a':
      iZ += 1.0;
      break;
    case 'd':
      iZ -= 1.0;
      break;
    case 'A':
      iZ += 0.1;
      break;
    case 'D':
      iZ -= 0.1;
      break;

    /* Camera translation */
    case 'r':
      view->camX += 1.0;
      rerender_statusbar = rerender_view = True;
      break;
    case 'f':
      view->camX -= 1.0;
      rerender_statusbar = rerender_view = True;
      break;
    case 'R':
      view->camX += 0.1;
      rerender_statusbar = rerender_view = True;
      break;
    case 'F':
      view->camX -= 0.1;
      rerender_statusbar = rerender_view = True;
      break;
    case 't':
      view->camY += 1.0;
      rerender_statusbar = rerender_view = True;
      break;
    case 'g':
      view->camY -= 1.0;
      rerender_statusbar = rerender_view = True;
      break;
    case 'T':
      view->camY += 0.1;
      rerender_statusbar = rerender_view = True;
      break;
    case 'G':
      view->camY -= 0.1;
      rerender_statusbar = rerender_view = True;
      break;
    case 'y':
      view->camZ += 1.0;
      rerender_statusbar = rerender_view = True;
      break;
    case 'h':
      view->camZ -= 1.0;
      rerender_statusbar = rerender_view = True;
      break;
    case 'Y':
      view->camZ += 0.1;
      rerender_statusbar = rerender_view = True;
      break;
    case 'H':
      view->camZ -= 0.1;
      rerender_statusbar = rerender_view = True;
      break;

    case '[':
      /*    case GDK_Page_Down: */
      view->dist += 1;
      rerender_view = True;
      break;
    case '{':
      view->dist += 0.1;
      rerender_view = True;
      break;
    case ']':
      /*    case GDK_End: */
      view->dist -= 1;
      rerender_view = True;
      break;
    case '}':
      view->dist -= 0.1;
      rerender_view = True;
      break;

    /* FOV tweaking */
    case ',':
      view->config.fov += 5;
      reconfigure_view = True;
      break;
    case '<':
      view->config.fov += 1;
      reconfigure_view = True;
      break;
    case '.':
      view->config.fov -= 5;
      reconfigure_view = True;
      break;
    case '>':
      view->config.fov -= 1;
      reconfigure_view = True;
      break;

    /* Near Clipping Plane Adjustment */
    case GDK_End:
      view->config.near -= 1;
      reconfigure_view = True;
      break;
    case GDK_Page_Down:
      view->config.near += 1;
      reconfigure_view = True;
      break;

    /* Far Clipping Plane Adjustment */
    case GDK_Home:
      view->config.far -= 1;
      reconfigure_view = True;
      break;
    case GDK_Page_Up:
      view->config.far += 1;
      reconfigure_view = True;
      break;

    /* Model rotation */
    case GDK_Left:
      view->spin -= 5;
      rerender_view = True;
      break;
    case GDK_Right:
      view->spin += 5;
      rerender_view = True;
      break;
    case GDK_Up:
      view->pitch += 5;
      rerender_view = True;
      break;
    case GDK_Down:
      view->pitch -= 5;
      rerender_view = True;
      break;

      /*
    case GDK_Page_Up:
      view->yaw -= 5;
      rerender_view = True;
      break;
    case GDK_Home:
      view->yaw -= 5;
      rerender_view = True;
      break;
      */

    case GDK_KP_5:
      view->spin = 0;
      view->pitch = 0;
      view->yaw = 0;
      view->config.fov = view->pDoc->config.view_defaults.fov;
      view->config.near = view->pDoc->config.view_defaults.near;
      view->config.far = view->pDoc->config.view_defaults.far;
      view->camX = view->camY = view->camZ = 0;
      reconfigure_view = True;
      break;

    /* Mode specific commands */
    case GDK_Return:
      rerender = ed_return_pressed(view);
      break;
    case GDK_Delete:
    case GDK_BackSpace:		/* in some places, backspace/delete are same */
      rerender = ed_delete_pressed(view);
      break;
    case GDK_Escape:
      rerender = ed_escape_pressed(view);
      break;

    case ' ':
      if(view->config.enlarge_vertices)
	view->config.enlarge_vertices = False;
      else
	view->config.enlarge_vertices = True;
      view_rebuild_glarrays(view);
      rerender_view = True;
      break;
      /*
    case 'z':
      rerender = doc_undo(view->pDoc);
      break;
    case 'x':
      rerender = doc_redo(view->pDoc);
      break;
      */
#ifdef ME_DEBUG
    default:
      printf("don't know key 0x%x\n", event->keyval);
#endif
  }

  if((iX != 0) || (iY != 0) || (iZ != 0))
  {
    switch(op_mode)
    {
      case M_SELECT:
	/*      case M_POLY_SEL: */
	rerender = doc_selection_translate(view->pDoc, view->active_group,
					   iX, iY, iZ);
	break;
      case M_VERTEX_ADD:
	view->cursX += iX;  view->cursY += iY;  view->cursZ += iZ;
	rerender_view = True;
	rerender_statusbar = True;
	break;

      default:
	break;
    }
  }

  if(view->spin >= 360)
    view->spin -= 360;
  if(view->spin < 0)
    view->spin += 360;
  if(view->pitch >= 360)
    view->pitch -= 360;
  if(view->pitch < 0)
    view->pitch += 360;

  if(reconfigure_view) {
    /* AFAIK there isn't anything in this function that changes
       the GL drawing area properties.. */
    view_reconfigure(view, RCV_VIEWPORT);
    rerender_view = True;
  }
  if(rerender)
    view_start_rendercycle_docwide(view->pDoc);
  else {
    if(rerender_view)
      view_start_rendercycle(view);
  }

  if (rerender_statusbar)
    view_set_statusbar(view);

  return True;
}



gboolean ed_return_pressed(struct model_view *view)
{
  switch(op_mode)
  {
    case M_VERTEX_ADD:
      doc_vertex_add(view->pDoc, view->active_group,
		     view->cursX, view->cursY, view->cursZ);
      return True;
      break;

    case M_POLY_SEL:
    case M_TRIANGLES:
    case M_QUADS:
    case M_TRIANGLE_STRIP:
    case M_QUAD_STRIP:
    case M_TRIANGLE_FAN:
    case M_POLYGON:
    case M_SOR:
      if(view->surface_focused != NULL) {
	if(doc_surface_commit(view->pDoc, view->active_group, view->surface_focused)) {
	  view->surface_focused = NULL;
	  return True;
	} else
	  gdk_beep();
      }
      break;

    default:
      break;
  }

  return False;
}

gboolean ed_delete_pressed(struct model_view *view)
{
  switch(op_mode)
  {
    case M_VERTEX_ADD:
      doc_vertex_remove_pos(view->pDoc, view->active_group, view->cursX,
			    view->cursY, view->cursZ);
      break;

    case M_SELECT:
      return doc_selection_remove(view->pDoc, view->active_group);
      break;

    case M_TRIANGLES:
    case M_QUADS:
    case M_TRIANGLE_STRIP:
    case M_QUAD_STRIP:
    case M_TRIANGLE_FAN:
      if(view->surface_focused != NULL)
	return primitive_detach_last_vertex(view->surface_focused);
      break;

    case M_POLYGON:
      if(view->surface_focused != NULL)
	return polygon_tess_detach_last_vertex(view->surface_focused);
      break;

    case M_SOR:
      if (view->surface_focused != NULL)
	return sor_detach_last_vertex (view->surface_focused);
      break;

    case M_POLY_SEL:
      if(view->surface_focused) {
	switch(view->surface_focused->type) {
	  case SURF_PRIMITIVE:
	    return primitive_detach_last_vertex(view->surface_focused);
	    break;
	  case SURF_POLYGON_TESS:
	    return polygon_tess_detach_last_vertex(view->surface_focused);
	    break;

	  default:
	    break;
	}
      }
      break;

    default:
      break;
  }

  return False;
}

gboolean ed_escape_pressed(struct model_view *view)
{
  switch(op_mode)
  {
    case M_SELECT:
      return doc_deselect_all(view->pDoc, view->active_group);
      break;

    case M_POLY_SEL:
    case M_TRIANGLES:
    case M_QUADS:
    case M_TRIANGLE_STRIP:
    case M_QUAD_STRIP:
    case M_TRIANGLE_FAN:
    case M_POLYGON:
    case M_SOR:
      if(view->surface_focused != NULL) {
	doc_surface_destroy(view->pDoc, NULL, view->surface_focused);
	view->surface_focused = NULL;
      }
      return True;
      break;

    default:
      break;
  }

  return False;
}


void ed_configure_menu(struct model_view *view)
{
  if(fmt_load_dialog_available() == True)
    docmenu_set_sensitive("<Model>/File/Open", True);
  else
    docmenu_set_sensitive("<Model>/File/Open", False);

  if(fmt_save_dialog_available() == True) {
    docmenu_set_sensitive("<Model>/File/Save as", True);
    if(view->pDoc->fmt_idx >= 0)
      docmenu_set_sensitive("<Model>/File/Save", True);
    else
      docmenu_set_sensitive("<Model>/File/Save", False);
  } else {
    docmenu_set_sensitive("<Model>/File/Save as", False);
    docmenu_set_sensitive("<Model>/File/Save", False);
  }

  if(view->config.vertices_visible)
    docmenu_set_check("<Model>/View/Show Vertices", True);
  else
    docmenu_set_check("<Model>/View/Show Vertices", False);
  if(view->config.grid_visible)
    docmenu_set_check("<Model>/View/Show Grid", True);
  else
    docmenu_set_check("<Model>/View/Show Grid", False);
  if(view->config.normals_visible)
    docmenu_set_check("<Model>/View/Show Normals", True);
  else
    docmenu_set_check("<Model>/View/Show Normals", False);

  switch(view->config.polygon_mode) {
    case MP_HIDE:
      docmenu_set_check("<Model>/View/Surfaces/Hidden", True);
      docmenu_set_check("<Model>/View/Surfaces/Wireframe", False);
      docmenu_set_check("<Model>/View/Surfaces/Full", False);
      break;
    case MP_SHOW_WIREFRAME:
      docmenu_set_check("<Model>/View/Surfaces/Hidden", False);
      docmenu_set_check("<Model>/View/Surfaces/Wireframe", True);
      docmenu_set_check("<Model>/View/Surfaces/Full", False);
      break;
    case MP_SHOW_SURFACES:
      docmenu_set_check("<Model>/View/Surfaces/Hidden", False);
      docmenu_set_check("<Model>/View/Surfaces/Wireframe", False);
      docmenu_set_check("<Model>/View/Surfaces/Full", True);
      break;
  }

  if(view->pDoc->config.smooth_mode)
    docmenu_set_check("<Model>/Model/Smooth Shading", True);
  else
    docmenu_set_check("<Model>/Model/Smooth Shading", False);
  if(view->pDoc->config.clockwise)
    docmenu_set_check("<Model>/Model/Clockwise Orientation", True);
  else
    docmenu_set_check("<Model>/Model/Clockwise Orientation", False);
}


void ed_set_mode(struct model_view *view, guint op_mode)
{
  if(view->surface_focused != NULL) {
    doc_surface_destroy(view->pDoc, NULL, view->surface_focused);
    view->surface_focused = NULL;
  }
  view_start_rendercycle(view);
}
#endif






