#include <math.h>
#include <stdlib.h>

#include "frame.h"
#include "toolbar.h"
#include "glpreview.h"
#include "fileops.h"
#include "gadj.h"

#include "memwatch.h"


#define WIN_WIDTH widget->allocation.width
#define WIN_HEIGHT widget->allocation.height

mframe *frames[ 3 ];
int grid_spacing = 1, grid_snap = 0;
GtkWidget *table_child[ 4 ];



void clear_pixmap( mframe *fr ) {

    // notes from andy -
    //
    // 1 - unfreed gc?               fixed
    // 2 - unfreed colormap?         fixed
    //
    // 3 - wouldn't it be more efficient to draw lines, instead of hollow 
    // rectangles... y'know, from an 'overdraw' point-of-view


    double a;
    GdkGC *gc;
    GdkColor color;
    GdkColormap *colormap;
    
    Point min, max;
    
    int compareIndexX = get_x_index_used_by_frame( fr->frame_adj->type );
    int compareIndexY = get_y_index_used_by_frame( fr->frame_adj->type );
    double scale = fr->frame_adj->scale;
    Point origin = fr->frame_adj->origin;
    
    
    min.v[0] = ceil( origin.v[compareIndexX] - (( fr->width/scale)/2.0f) );
    min.v[1] = ceil( origin.v[compareIndexY] - ((fr->height/scale)/2.0f) );
    max.v[0] = floor( origin.v[compareIndexX] + (( fr->width/scale)/2.0f) );
    max.v[1] = floor( origin.v[compareIndexY] + ((fr->height/scale)/2.0f) );


//printf( "origin is:   %f   %f   %f\n", origin.v[0], origin.v[1], origin.v[2] );
    
    // draw background

    gc = gdk_gc_new( window->window );
    colormap = gdk_window_get_colormap( window->window );
    color.red = 22938;
    color.green = 22938;
    color.blue = 22938;
    gdk_color_alloc( colormap, &color );
    gdk_gc_set_foreground( gc, &color );

    gdk_draw_rectangle( fr->pixmap,
                        gc,
                        TRUE,
                        0, 0, 
                        fr->width, fr->height );

    // draw grid...

    color.red = 19005;
    color.green = 19005;
    color.blue = 19005;
    gdk_color_alloc( colormap, &color );
    gdk_gc_set_foreground( gc, &color );
    
    for ( a = min.v[0]; a < max.v[0]; a += grid_spacing ) {
        Point current;
        double v[3]; v[0] = v[1] = v[2] = 0.0f;
        v[compareIndexX] = a;
        current = adj_world_to_win( v, fr->frame_adj );
        // vertical line
        gdk_draw_line( fr->pixmap, gc, (int)current.v[0], 0, (int)current.v[0], fr->height );
    }
    
    for ( a = min.v[1]; a < max.v[1]; a += grid_spacing ) {
        Point current;
        double v[3]; v[0] = v[1] = v[2] = 0.0f;
        v[compareIndexY] = a;
        current = adj_world_to_win( v, fr->frame_adj );
        // horizontal line
        gdk_draw_line( fr->pixmap, gc, 0, (int)current.v[1], fr->width, (int)current.v[1] );
    }

    // added by ads, 23/4/01
    gdk_gc_unref( gc );
}



void frame_zoom( double y, mframe *fr ) {

    static double prev_y = 0.0f;
    double delta;
    
    if( prev_y == 0.0f ) prev_y = y;   // first time we've been called.
    // f.p. comparisons don't work on all machines, but this isn't critical...
    
    delta = prev_y - y;
    
    if( delta > 5.0f ) delta = 5.0f;
    if( delta < -5.0f ) delta = -5.0f;

    fr->frame_adj->scale += delta/3.0f;  // 3 seems to work best...
    
    if( fr->frame_adj->scale > 100.0f ) fr->frame_adj->scale = 100.0f;
    if( fr->frame_adj->scale < 1.0f ) fr->frame_adj->scale = 1.0f;
    
    prev_y = y;
    
}



void frame_pan( double x, double y, double prev_x, double prev_y, mframe *fr ) {

    double delta_x, delta_y;
    Point p, p2;
    
    int compareIndexX = get_x_index_used_by_frame( fr->frame_adj->type );
    int compareIndexY = get_y_index_used_by_frame( fr->frame_adj->type );
    
    p = adj_win_to_world( x, y, fr->frame_adj );
    p2 = adj_win_to_world( prev_x, prev_y, fr->frame_adj );
    
    delta_x = p2.v[compareIndexX] - p.v[compareIndexX];
    delta_y = p2.v[compareIndexY] - p.v[compareIndexY];

    
//    if( delta_x > 5.0f )  delta_x = 5.0f;
//    if( delta_x < -5.0f ) delta_x = -5.0f;
//    if( delta_y > 5.0f )  delta_y = 5.0f;
//    if( delta_y < -5.0f ) delta_y = -5.0f;

    fr->frame_adj->origin.v[compareIndexX] += delta_x;
    fr->frame_adj->origin.v[compareIndexY] += delta_y;
    
    
}



int hexpose_event( GtkWidget *widget, GdkEventExpose *event, gpointer data ) {
    mframe * fr = ( mframe * ) data;

    gdk_draw_pixmap( widget->window,
                     widget->style->fg_gc[ GTK_WIDGET_STATE ( widget ) ],
                     fr->pixmap,
                     event->area.x, event->area.y,
                     event->area.x, event->area.y,
                     event->area.width, event->area.height );

    return FALSE;
}

void draw_all_frames( void ) {
    xyz_state = FRAME_XY;
//    paxadj = frames[ FRAME_XY ] ->xadj;
//    payadj = frames[ FRAME_XY ] ->yadj;
    clear_pixmap( frames[ FRAME_XY ] );
    lines_draw( frames[ FRAME_XY ] ->pixmap, frames[ FRAME_XY ]->frame_adj );
    tris_draw( frames[ FRAME_XY ] ->pixmap, frames[ FRAME_XY ]->frame_adj );
    point_draw( frames[ FRAME_XY ] ->pixmap, frames[ FRAME_XY ]->frame_adj );
    gtk_widget_draw ( frames[ FRAME_XY ] ->drawing_area, NULL );

    xyz_state = FRAME_YZ;
//    paxadj = frames[ FRAME_YZ ] ->xadj;
//    payadj = frames[ FRAME_YZ ] ->yadj;
    clear_pixmap( frames[ FRAME_YZ ] );
    lines_draw( frames[ FRAME_YZ ] ->pixmap, frames[ FRAME_YZ ]->frame_adj );
    tris_draw( frames[ FRAME_YZ ] ->pixmap, frames[ FRAME_YZ ]->frame_adj );
    point_draw( frames[ FRAME_YZ ] ->pixmap, frames[ FRAME_YZ ]->frame_adj );
    gtk_widget_draw ( frames[ FRAME_YZ ] ->drawing_area, NULL );

    xyz_state = FRAME_XZ;
//    paxadj = frames[ FRAME_XZ ] ->xadj;
//    payadj = frames[ FRAME_XZ ] ->yadj;
    clear_pixmap( frames[ FRAME_XZ ] );
    lines_draw( frames[ FRAME_XZ ] ->pixmap, frames[ FRAME_XZ ]->frame_adj );
    tris_draw( frames[ FRAME_XZ ] ->pixmap, frames[ FRAME_XZ ]->frame_adj );
    point_draw( frames[ FRAME_XZ ] ->pixmap, frames[ FRAME_XZ ]->frame_adj );
    gtk_widget_draw ( frames[ FRAME_XZ ] ->drawing_area, NULL );

    gtk_widget_draw( glarea, NULL );
}

int hconfigure_event( GtkWidget *widget, GdkEvent *event, gpointer data ) {
    mframe * fr = ( mframe * ) data;

    xyz_state = fr->frame_adj->type;

    if ( fr->pixmap )
        gdk_pixmap_unref( fr->pixmap );

//    gadj_change( fr->xadj, 0.0, ( double ) WIN_WIDTH );
//    gadj_change( fr->yadj, 0.0, ( double ) WIN_HEIGHT );
    
    fr->frame_adj->origin.v[0] = 0.0f;
    fr->frame_adj->origin.v[1] = 0.0f;
    fr->frame_adj->origin.v[2] = 0.0f;
    fr->frame_adj->scale = 60.0f;
    fr->frame_adj->width = WIN_WIDTH;
    fr->frame_adj->height = WIN_HEIGHT;
    fr->width = WIN_WIDTH;
    fr->height = WIN_HEIGHT;

    fr->pixmap = gdk_pixmap_new( widget->window,
                                 WIN_WIDTH,
                                 WIN_HEIGHT,
                                 -1 );

    gdk_draw_rectangle( fr->pixmap,
                        widget->style->white_gc,
                        TRUE,
                        0, 0,
                        WIN_WIDTH,
                        WIN_HEIGHT );

    clear_pixmap( fr );

//    paxadj = fr->xadj;
//    payadj = fr->yadj;
    lines_draw( fr->pixmap, fr->frame_adj );
    tris_draw( fr->pixmap, fr->frame_adj );
    point_draw( fr->pixmap, fr->frame_adj );
    gtk_widget_draw ( fr->drawing_area, NULL );
    return TRUE;
}

void hmotion_notify_event( GtkWidget *widget, GdkEventMotion *event,
                           gpointer data ) {

    static Point * prev_win_coords = NULL;
    static mframe* prev_frame = NULL;
    int x, y;
    double a, b;
    GdkModifierType state;
    mframe *fr = ( mframe * ) data;
    int redraw = 0;
    int compareIndexX = get_x_index_used_by_frame( fr->frame_adj->type );
    int compareIndexY = get_y_index_used_by_frame( fr->frame_adj->type );

    xyz_state = fr->frame_adj->type;


    if ( event->is_hint )
        gdk_window_get_pointer ( event->window, &x, &y, &state );
    else {
        x = event->x;
        y = event->y;
        state = event->state;
    }

    if( prev_win_coords == NULL ) {
        // first time we've been called.
        prev_win_coords = (Point*)malloc( sizeof( Point ) );
        prev_win_coords->v[0] = x;
        prev_win_coords->v[1] = y;
        prev_win_coords->v[2] = 0.0f;
    }
    
    if( prev_frame != fr ) {
        prev_win_coords->v[0] = x;
        prev_win_coords->v[1] = y;
    }

    {
    Point p = adj_win_to_world( x, y, fr->frame_adj );
    a = p.v[compareIndexX];
    b = p.v[compareIndexY];
    }
    
    if ( grid_snap ) {
        a = grid_spacing * ( int ) rint( a / ( ( double ) grid_spacing ) );
        b = grid_spacing * ( int ) rint( b / ( ( double ) grid_spacing ) );
    }

    if ( state & GDK_BUTTON1_MASK && fr->pixmap != NULL ) {
        if ( click_mode[ 1 ] == CM_POINT_SEL )
            point_move_selected( a, b );
        else if ( click_mode[ 1 ] == CM_ZOOM )
            frame_zoom( y, fr );
        else if ( click_mode[ 1 ] == CM_PAN )
            frame_pan( x, y, prev_win_coords->v[0], prev_win_coords->v[1], fr );
        redraw = 1;
    }

    else if ( state & GDK_BUTTON2_MASK && fr->pixmap != NULL ) {
        if ( click_mode[ 2 ] == CM_POINT_SEL )
            point_move_selected( a, b );
        else if ( click_mode[ 2 ] == CM_ZOOM )
            frame_zoom( y, fr );
        else if ( click_mode[ 2 ] == CM_PAN )
            frame_pan( x, y, prev_win_coords->v[0], prev_win_coords->v[1], fr );
        redraw = 1;
    }

    else if ( state & GDK_BUTTON3_MASK && fr->pixmap != NULL ) {
        if ( click_mode[ 3 ] == CM_POINT_SEL )
            point_move_selected( a, b );
        else if ( click_mode[ 3 ] == CM_ZOOM )
            frame_zoom( y, fr );
        else if ( click_mode[ 3 ] == CM_PAN )
            frame_pan( x, y, prev_win_coords->v[0], prev_win_coords->v[1], fr );
        redraw = 1;
    }
    
    prev_win_coords->v[0] = x;
    prev_win_coords->v[1] = y;
    prev_frame = fr;

    if( redraw ) {
        draw_all_frames();
        gtk_widget_draw( widget, NULL );
    }
}

int hbutton_press_event( GtkWidget *widget,
                         GdkEventButton *event,
                         gpointer data ) {
    double x, y;
    static Point *p1 = NULL;  //, *p2 = NULL, *p3 = NULL;
    Point *np;
    mframe *fr = ( mframe * ) data;
    int compareIndexX = get_x_index_used_by_frame( fr->frame_adj->type );
    int compareIndexY = get_y_index_used_by_frame( fr->frame_adj->type );

    xyz_state = fr->frame_adj->type;

    {
    Point p = adj_win_to_world( event->x, event->y, fr->frame_adj );
    x = p.v[compareIndexX];
    y = p.v[compareIndexY];
    }

    if ( grid_snap ) {
        x = grid_spacing * ( int ) rint( x / ( ( double ) grid_spacing ) );
        y = grid_spacing * ( int ) rint( y / ( ( double ) grid_spacing ) );
    }

    if ( fr->pixmap != NULL )
        switch ( click_mode[ event->button ] ) {
        case CM_POINT_ADD:
            np = point_add( x, y );
            break;
        case CM_POINT_SEL:
#if 0
            point_select( x, y );
#endif
            break;
        case CM_POINT_DEL:
            
            p1 = single_point_select( x, y );
            point_del_v( p1 );
            /* delete any lines or triangles that use this point */
            line_del_point( p1 );
            tris_del_point( p1 );
            /* we may have just deleted one of the selected points... */
            point_unselect_all();
            p1 = NULL;
            break;

        default:
            break;
        }

    draw_all_frames();

    return TRUE;
}

gint expand_cb( GtkWidget *widget, gpointer data ) {
    int c;
    static int ms = 1;

    if ( ms ) {
        for ( c = 0; c < 4; c++ )
            if ( table_child[ c ] != ( GtkWidget * ) data ) gtk_widget_hide( table_child[ c ] );
        gtk_table_set_homogeneous( GTK_TABLE( table ), 0 );
    } else {
        for ( c = 0; c < 4; c++ )
            if ( table_child[ c ] != ( GtkWidget * ) data ) gtk_widget_show( table_child[ c ] );
        gtk_table_set_homogeneous( GTK_TABLE( table ), 1 );
    }

    ms = ms ? 0 : 1;

    return FALSE;
}


mframe *init_mframe( int projection_type ) {
    mframe * fr;
    GtkWidget *drawing_area, *label, *hbox, *frame;

    fr = ( mframe * ) malloc( sizeof( mframe ) );
    fr->frame_adj = (gadj *)malloc( sizeof( gadj ) );

    fr->frame_adj->type = projection_type;

    drawing_area = gtk_drawing_area_new();
    gtk_widget_show( drawing_area );

    fr->drawing_area = drawing_area;
    fr->pixmap = NULL;

    gtk_signal_connect ( GTK_OBJECT ( drawing_area ), "expose_event",
                         ( GtkSignalFunc ) hexpose_event, ( gpointer ) fr );
    gtk_signal_connect ( GTK_OBJECT( drawing_area ), "configure_event",
                         ( GtkSignalFunc ) hconfigure_event, ( gpointer ) fr );
    gtk_signal_connect ( GTK_OBJECT ( drawing_area ), "motion_notify_event",
                         ( GtkSignalFunc ) hmotion_notify_event, ( gpointer ) fr );
    gtk_signal_connect ( GTK_OBJECT ( drawing_area ), "button_press_event",
                         ( GtkSignalFunc ) hbutton_press_event, ( gpointer ) fr );

    gtk_widget_set_events ( drawing_area, GDK_EXPOSURE_MASK
                            | GDK_LEAVE_NOTIFY_MASK
                            | GDK_BUTTON_PRESS_MASK
                            | GDK_POINTER_MOTION_MASK
                            | GDK_POINTER_MOTION_HINT_MASK );


    switch( projection_type ) {
        case FRAME_XY:
            label = gtk_button_new_with_label( "XY - Top" );
            break;
        case FRAME_YZ:
            label = gtk_button_new_with_label( "YZ - Side" );
            break;
        case FRAME_XZ:
            label = gtk_button_new_with_label( "XZ - Side" );
            break;
        default:
            label = gtk_button_new_with_label( "Something Bad" );
            break;
    }
    gtk_widget_show( label );

    frame = gtk_frame_new( NULL );
    gtk_widget_show( frame );
    gtk_container_add( GTK_CONTAINER( frame ), drawing_area );
    gtk_frame_set_shadow_type( GTK_FRAME( frame ), GTK_SHADOW_OUT );

    hbox = gtk_vbox_new( FALSE, 0 );
    gtk_box_pack_start( GTK_BOX( hbox ), label, FALSE, TRUE, 0 );
    gtk_box_pack_start( GTK_BOX( hbox ), frame, TRUE, TRUE, 0 );
    gtk_widget_show( hbox );

    gtk_signal_connect( GTK_OBJECT( label ), "clicked",
                        GTK_SIGNAL_FUNC( expand_cb ), ( gpointer ) hbox );

    fr->scw = hbox;

    fr->frame_adj->win_x = get_x_index_used_by_frame( projection_type );
    fr->frame_adj->win_y = get_y_index_used_by_frame( projection_type );


    return fr;
}





int get_x_index_used_by_frame( int type ) {

    int result = 99999999;
    
    switch( type ) {
        case FRAME_XY :
            result = 0;
            break;
        case FRAME_YZ :
            result = 1;
            break;
        case FRAME_XZ :
            result = 0;
            break;
        default:
            // should never happen, but just in case...
            break;
    }

    return result;
}


int get_y_index_used_by_frame( int type ) {

    int result = 99999999;
    
    switch( type ) {
        case FRAME_XY :
            result = 1;
            break;
        case FRAME_YZ :
            result = 2;
            break;
        case FRAME_XZ :
            result = 2;
            break;
        default:
            // should never happen, but just in case...
            break;
    }

    return result;
}















