/* 
 * File:         fileio.c
 * 
 * Description:  handles loading and saving of model data
 * 
 * This source code is part of kludge3d, and is released under the 
 * GNU General Public License.
 * 
 * 
 */
 
/*
 * Portions of this file were ripped from:
 *    - ac3d.c from the "orbit" space combat game
 *        and, consequently, the ac_to_gl program as well
 * 
 */



#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <math.h>

#include "transform.h"
#include "globals.h"
#include "mesh.h"
#include "model.h"
#include "tri.h"
#include "points.h"
#include "texture.h"
#include "fileio.h"
#include "error_queue.h"

#include "memwatch.h"




struct Tag {
    char *token;
    int ( *func ) ( char *s, GNode * node );
};

int Search ( struct Tag *tags, char *s, GNode * );
int DoMaterial ( char *s, GNode * );
int DoObject ( char *s, GNode * );
int DoACName ( char *s, GNode * );
int DoData ( char *s, GNode * );
int DoTexture ( char *s, GNode * );
int DoTexrep ( char *s, GNode * );
int DoRot ( char *s, GNode * );
int DoLoc ( char *s, GNode * );
int DoUrl ( char *s, GNode * );
int DoNumvert ( char *s, GNode * );
int DoNumsurf ( char *s, GNode * );
int DoSurf ( char *s, GNode * );
int DoMat ( char *s, GNode * );
int DoRefs ( char *s, GNode * );
int DoKids ( char *s, GNode * );

int DoObjWorld ( char *s, GNode * );
int DoObjPoly ( char *s, GNode * );
int DoObjGroup ( char *s, GNode * );

void SkipSpaces ( char **s );
void SkipQuotes ( char **s );
void RemoveQuotes ( char *s );

//#define MAX_TEXTURES 1000    /* This *ought* to be enough! */
//char *texture_fnames [ MAX_TEXTURES ];

int acsmooth;			/* Smooth surface? */
int acsurfpass;			/* Pass through surface */
#define PASS_NORMAL (0)		/* Get normals */
#define PASS_RENDER (1)		/* Render */


int num_materials = 0;
int num_textures = 0;
int ntex = 0;

int last_flags = -1;
int last_mat;
int last_num_kids = -1;
int current_flags = -1;
int texture_enabled = 0;
int need_texture = 0;

#define PARSE_CONT   0
#define PARSE_POP    1
//FIXME - should have a PARSE_ERROR so that we can gracefully back out 
// of all these funcs

int matlist[ 1024 ];
int modelnum;
FILE *acfd;

//#define NO_SWAP_YZ


// OUTPUT STUFF ///////////////////////////////////////////////////////////

void write_mesh_ac3d( Mesh *m, FILE *fp ) {

    GList * c;
    Tri *tri;
//    tex *text;
    Point *p;
    int num_points, num_triangles;

GList *debugging = model_textures;

    num_points = g_list_length( m->points );
    num_triangles = g_list_length( m->triangles );
    
    fprintf( fp, "OBJECT poly\n" );

    if( m->name != NULL ) 
        fprintf( fp, "name \"%s\"\n", m->name );

    if( m->texture != NULL ) {
        fprintf( fp, "texture \"%s\"\n", m->texture->filename );
        fprintf( fp, "texrep %f %f\n", m->texrep_x, m->texrep_y );
    }
    
    if( m->location != NULL ) {
        p = m->location;
#ifdef NO_SWAP_YZ
        fprintf( fp, "loc %f %f %f\n", p->v[0], p->v[1], p->v[2] );
#else
        fprintf( fp, "loc %f %f %f\n", p->v[0], p->v[2], -1.0 * p->v[1] );
#endif
    }

    fprintf( fp, "numvert %i\n", num_points );

    for ( c = m->points; c != NULL; c = c->next ) {
        p = ( Point * ) c->data;
#ifdef NO_SWAP_YZ
        fprintf( fp, "%f %f %f\n", p->v[ 0 ], p->v[ 1 ], p->v[ 2 ] );
#else
        fprintf( fp, "%f %f %f\n", p->v[ 0 ], p->v[ 2 ], -1.0 * p->v[ 1 ] );
#endif
    }

    fprintf( fp, "numsurf %i\n", num_triangles );

    for ( c = m->triangles; c != NULL; c = c->next ) {
        int i;
        tri = ( Tri * ) c->data;

        fprintf( fp, "SURF 0x0\n" );
        fprintf( fp, "mat 0\n" );
        fprintf( fp, "refs 3\n" );


        for ( i = 0; i < 3; i++ ) {

            if ( m->texture != NULL )
                fprintf( fp, "%d %f %f\n",
                         g_list_index( m->points, tri->vertices[ i ] ),
                         tri->tv[ i ].x,
                         tri->tv[ i ].y );
            else
                fprintf( fp, "%d %f %f\n",
                         g_list_index( m->points, tri->vertices[ i ] ),
                         0.0f, 0.0f );

        }

    }



}


void write_child_ac3d( GNode *n, FILE *fp ) {


    for( ; n != NULL; n = n->next ) {
        int kids = g_node_n_children( n );
        write_mesh_ac3d( (Mesh*)(n->data), fp );
        fprintf( fp, "kids %i\n", kids );
        if( kids > 0 )
            write_child_ac3d( n->children, fp );
        
        
    }

}


void fileio_save_ac3d( GNode *n, char *filename ) {

    FILE *fp;

    fp = fopen( filename, "w" );
    if ( fp == NULL ) return ;

    fprintf( fp, "AC3Db\n" );
    fprintf( fp, "MATERIAL \"\" rgb 1 1 1  amb 0.2 0.2 0.2  emis 0 0 0  spec 0.5 0.5 0.5  shi 10  trans 0\n" );
    fprintf( fp, "OBJECT world\n" );
    fprintf( fp, "kids %i\n", g_node_n_children( n ) );

    
    write_child_ac3d( n->children, fp );
    
    fflush( fp );
    fclose( fp );
}


/*
void model_load_textures(  ) {
    GList * ch;
    tex *ttex;

    for ( ch = model_textures; ch != NULL; ch = ch->next ) {
        ttex = ( tex * ) ch->data;
        tex_load( ttex );
    }
}
*/




/******************************************************************
 *
 * AC3D
 * LOADING
 * STUFF
 *
 *******************************************************************/

/****************************************
 *
 * TOP-LEVEL FUNCTIONS AND STRUCTS
 *
 ****************************************/


struct Tag top_tags [] = {
	{"MATERIAL", DoMaterial },
	{"OBJECT" , DoObject },
	{NULL, NULL }
};


GNode * fileio_load_ac3d ( char* name ) {

    char buffer[ 1024 ], *s;
    int firsttime;
    GNode *model;

    printf( "LoadAC3D: Loading model %s\n", name );

    model = model_new();

    /* Open it */
    acfd = fopen( name, "rt" );
    if ( acfd == NULL ) {
        printf( "LoadAC3D: Can't open %s, giving up!\n", name );
        return ( NULL );
    }

    firsttime = 1;

    num_materials = 0;
    num_textures = 0;
    last_flags = -1 ;
    last_num_kids = -1 ;
    current_flags = -1 ;
    texture_enabled = 0;
    need_texture = 0;
    last_mat = -1;

    while ( fgets ( buffer, 1024, acfd ) != NULL ) {

        s = buffer ;

        /* Skip leading whitespace */
        SkipSpaces ( &s );

        /* Skip blank lines and comments */
        if ( *s < ' ' && *s != '\t' ) continue;
        if ( *s == '#' || *s == ';' ) continue;

        if ( firsttime ) {
            firsttime = 0;

            if ( strncmp ( s, "AC3D", 4 ) != 0 ) {
                printf( "LoadAC3d: %s is not an AC3D format file\n", name );
                return ( NULL );
            }

        } else {
            Search( top_tags, s, model );
        }
    }

    fclose( acfd );

    return model;
}


void SkipSpaces ( char **s ) {
    while ( **s == ' ' || **s == '\t' ) ( *s ) ++;
}

void SkipQuotes ( char **s ) {
    char * t;

    SkipSpaces ( s );

    if ( **s == '\"' ) {
        ( *s ) ++;
        t = *s;
        while ( *t != '\0' && *t != '\"' ) t++;

        if ( *t != '\"' )
            printf( "SkipQuotes: Mismatched double-quote in '%s'\n", *s );

        *t = '\0';
    } else {
        printf( "SkipQuotes: Expected double-quote in '%s'\n", *s );
    }
}

void RemoveQuotes ( char *s ) {
    /*
     *  Remove quotes from string
     */

    char * t;
    int i, j, len;

    len = strlen ( s );
    t = ( char * ) malloc ( len + 1 );
    strcpy ( t, s );
    j = 0;

    for ( i = 0; i < len; i++ ) {
        if ( t[ i ] != '"' ) {
            s[ j++ ] = t[ i ];
        }
    }
    s[ j ] = 0;

    free ( t );
}

int Search ( struct Tag *tags, char *s, GNode *model ) {
    int i;

    SkipSpaces ( &s );

//printf( "Search has been handed: %s", s );

    for ( i = 0; tags[ i ].token != NULL; i++ ) {
        if ( !strncasecmp ( tags[ i ].token, s, strlen ( tags[ i ].token ) ) ) {
            s += strlen ( tags[ i ].token );
            SkipSpaces ( &s );
            return ( *( tags[ i ].func ) ) ( s, model );
        }
    }

    printf( "Search: Unrecognised token '%s'\n", s );
//    exit( 1 );
// wtf?

    return PARSE_POP;
}


/*****************************************
 *
 * OBJECT-LEVEL FUNCTIONS AND STRUCTS 
 *
 *****************************************/


struct Tag object_tags[] = {
	{"name" , DoACName },
	{"data" , DoData},
	{"texture", DoTexture},
	{"texrep" , DoTexrep},
	{"rot" , DoRot},
	{"loc" , DoLoc},
	{"url" , DoUrl},
	{"numvert", DoNumvert},
	{"numsurf", DoNumsurf},
	{"kids" , DoKids},
	{NULL, NULL }
};

struct Tag surf_tag[] = {
	{"SURF", DoSurf },
	{NULL, NULL}
};

struct Tag surface_tags[] = {
	{"mat" , DoMat },
	{"refs", DoRefs},
	{NULL, NULL}
};

struct Tag obj_type_tags[] = {
	{"world", DoObjWorld },
	{"poly" , DoObjPoly },
	{"group", DoObjGroup},
	{NULL, NULL}
};

#define OBJ_WORLD  0
#define OBJ_POLY   1
#define OBJ_GROUP  2

int DoObjWorld ( char *s, GNode * model ) {
    return OBJ_WORLD;
}

int DoObjPoly ( char *s, GNode * model ) {
    return OBJ_POLY;
}

int DoObjGroup ( char *s, GNode * model ) {
    return OBJ_GROUP;
}

int DoMaterial ( char *s, GNode * model ) {
    char name[ 1024 ], *nm;
    float rgb[ 3 ], amb[ 3 ], emis[ 3 ], spec[ 3 ], trans;
    int shi;

    if ( 15 != sscanf ( s,
                        "%s rgb %f %f %f amb %f %f %f emis %f %f %f spec %f %f %f shi %d trans %f",
                        name, &rgb[ 0 ], &rgb[ 1 ], &rgb[ 2 ], &amb[ 0 ], &amb[ 1 ], &amb[ 2 ],
                        &emis[ 0 ], &emis[ 1 ], &emis[ 2 ], &spec[ 0 ], &spec[ 1 ], &spec[ 2 ], &shi, &trans ) ) {
        printf( "DoMaterial: Can't parse this MATERIAL: %s\n", s );
    } else {
        nm = name;
        SkipQuotes ( &nm );

#if 0
        /* Get a list for this material */
        matlist[ num_materials ] = glGenLists ( 1 );

        glNewList ( matlist[ num_materials ], GL_COMPILE );
        glMaterialfv ( GL_FRONT_AND_BACK, GL_AMBIENT, amb );
        glMaterialfv ( GL_FRONT_AND_BACK, GL_DIFFUSE, rgb );
        glMaterialfv ( GL_FRONT_AND_BACK, GL_SPECULAR, spec );
        glMaterialfv ( GL_FRONT_AND_BACK, GL_EMISSION, emis );
        glMaterialf ( GL_FRONT_AND_BACK, GL_SHININESS, shi );
        glEndList();
#endif
    }
//    num_materials++;

    return PARSE_CONT;
}

int DoObject ( char *s, GNode * model ) {
    int i, obj_type, num_kids;
    char buffer[ 1024 ];


    obj_type = Search ( obj_type_tags, s, model );

    switch ( obj_type ) {
    case OBJ_WORLD:
        if( model->data != NULL ) {
            mesh_set_name( (Mesh*)model->data, "World" );
        }
        break;

    case OBJ_POLY:
        if( model->data != NULL ) {
            if( ((Mesh*)model->data)->name == NULL )
                mesh_set_name( (Mesh*)model->data, "Poly with no name" );
        }
        break;

    case OBJ_GROUP:
        if( model->data != NULL ) {
            mesh_set_name( (Mesh*)model->data, "Group" );
        }
        break;
    }

    need_texture = 0;
    
    /* for each line of input that pertains to this object... */
    while ( NULL != fgets ( buffer, 1024, acfd ) ) {
        if ( Search ( object_tags, buffer, model ) == PARSE_POP ) break;
    }

    num_kids = last_num_kids;

    /* now do each of the object's children */
    for ( i = 0; i < num_kids; i++ ) {
        g_node_append_data( model, mesh_new() );
    
        fgets ( buffer, 1024, acfd );
        Search ( top_tags, buffer, g_node_last_child( model ) );
    }

    return PARSE_CONT;
}

int DoACName ( char *s, GNode * model ) {
    
    SkipQuotes ( &s );
//printf( "DoACName:  obj name %s\n", s );
    
    mesh_set_name( (Mesh*)model->data, s );
    
    return PARSE_CONT;
}

int DoData ( char *s, GNode * model ) {
    int i, len;

    len = strtol ( s, NULL, 0 );
    printf( "DoData: WARNING - data string encountered\n" );

    //                   v---- plus one for the newline character at the end of the next line
    for ( i = 0; i < len +1; i++ )
        fgetc ( acfd );

    return PARSE_CONT;
}


int DoTexture ( char *s, GNode * model ) {

    tex* t = NULL;
    
    SkipQuotes ( &s );
    
    t = model_get_texture_by_filename( s );
    if( t != NULL ) {
        mesh_set_texture( (Mesh*)model->data, t );
    } else {
        t = tex_new();
        t->filename = strdup( s );
        if( tex_load( t ) ) {
            // there was an error loading the texture file
            eq_add_message( "Texture could not be loaded" );
            tex_delete( t );
            t = NULL;
        } else {
            model_add_texture( t );
            mesh_set_texture( (Mesh*)model->data, t );
        }
    }
    
    
    return PARSE_CONT;
}

int DoTexrep ( char *s, GNode * model ) {
    float texrep[ 2 ];

    if ( 2 != sscanf ( s, "%f %f", &texrep[ 0 ], &texrep[ 1 ] ) ) {
        printf( "DoTextrep: Illegal texrep record: %s\n", s );
    } else {
        mesh_set_texrep( (Mesh*)model->data, texrep[0], texrep[1] );
    }

    return PARSE_CONT;
}

int DoRot ( char *s, GNode * model ) {
    float mat[ 4 ][ 4 ];

    mat[ 0 ][ 3 ] = mat[ 1 ][ 3 ] = mat[ 2 ][ 3 ] = mat[ 3 ][ 0 ] = mat[ 3 ][ 1 ] = mat[ 3 ][ 2 ] = 0.0f;
    mat[ 3 ][ 3 ] = 1.0f;

    if ( 9 != sscanf ( s, "%f %f %f %f %f %f %f %f %f",
                       &mat[ 0 ][ 0 ], &mat[ 0 ][ 1 ], &mat[ 0 ][ 2 ],
                       &mat[ 1 ][ 0 ], &mat[ 1 ][ 1 ], &mat[ 1 ][ 2 ],
                       &mat[ 2 ][ 0 ], &mat[ 2 ][ 1 ], &mat[ 2 ][ 2 ] ) ) {
        printf( "DoRot: Illegal rot record: %s\n", s );
    }

    //	glMultMatrixf (mat);

    return PARSE_CONT ;
}

int DoLoc ( char *s, GNode * model ) {
    float loc[ 3 ];
    Point* p;

    if ( 3 != sscanf ( s, "%f %f %f", &loc[ 0 ], &loc[ 1 ], &loc[ 2 ] ) ) {
        printf( "DoLoc: Illegal loc record: %s\n", s );
    }
    
    p = (Point*)malloc( sizeof(Point) );
    
#ifdef NO_SWAP_YZ
    p->v[0] = loc[0];
    p->v[1] = loc[1];
    p->v[2] = loc[2];
#else
    p->v[0] = loc[0];
    p->v[1] = -1.0 * loc[2];
    p->v[2] = loc[1];
#endif
    
    
    mesh_set_location( (Mesh*)model->data, p );

    return PARSE_CONT;
}

int DoUrl ( char *s, GNode * model ) {
    SkipQuotes ( &s );
    return PARSE_CONT;
}

int DoNumvert ( char *s, GNode * model ) {
    char buffer[ 1024 ];
    int i;
    Point * point;
    float thisX;
    float thisY;
    float thisZ;

    int numVertices = strtol ( s, NULL, 0 );
    
//debugging
Mesh* temp;
temp = ((Mesh*)model->data);
//debugging

    for ( i = 0; i < numVertices; i++ ) {
        fgets ( buffer, 1024, acfd );

//printf( "DoNumvert: buffer is: %s", buffer );
        point = point_new();
        
        thisX = thisY = thisZ = 0.0f;
        
        if ( 3 != sscanf( buffer, "%f %f %f", &thisX, &thisY, &thisZ ) ) {
            printf( "DoNumvert: Illegal vertex record: %s\n", buffer );
            free( point );
        }
        else {
//printf( "\tconverted to: %f %f %f\n", thisX, thisY, thisZ );
            // add a new vertex to the model
#ifdef NO_SWAP_YZ
            point->v[ 0 ] = thisX;
            point->v[ 1 ] = thisY;
            point->v[ 2 ] = thisZ;
#else
            point->v[ 0 ] = thisX;
            point->v[ 1 ] = -1.0 * thisZ;
            point->v[ 2 ] = thisY;
#endif
            point_add_v( (Mesh*)model->data, point );
        }

    }

    return PARSE_CONT;
}

int DoNumsurf ( char *s, GNode * model ) {
    int i, ns;
//    int v;
//    long pos, p;
    char buffer[ 1024 ];

    ns = strtol ( s, NULL, 0 );

#if 0
    /* Do first pass to average normals */
    pos = ftell ( acfd );
    acsurfpass = PASS_NORMAL;
    for ( i = 0; i < ns; i++ ) {
        p = ftell ( acfd );
        fgets ( buffer, 1024, acfd );
        Search ( surf_tag, buffer );
    }

    /* Back to beginning of object */
    fseek ( acfd, pos, SEEK_SET, model );

    /* Average normals */
    for ( v = 0; v < nv; v++ ) {
//        vtab[ v ].normal[ 0 ] /= vtab[ v ].n;
//        vtab[ v ].normal[ 1 ] /= vtab[ v ].n;
//        vtab[ v ].normal[ 2 ] /= vtab[ v ].n;
    }

    /* Now render */
    acsurfpass = PASS_RENDER;
    last_mat = ( -1 );
#endif

    for ( i = 0; i < ns; i++ ) {
//        p = ftell ( acfd );
        fgets ( buffer, 1024, acfd );
        Search ( surf_tag, buffer, model );
    }

    return PARSE_CONT;
}

int DoSurf ( char *s, GNode * model ) {
    char buffer[ 1024 ];

    current_flags = strtol ( s, NULL, 0 );

    while ( NULL != fgets ( buffer, 1024, acfd ) ) {
        if ( Search ( surface_tags, buffer, model ) == PARSE_POP ) break;
    }

    return PARSE_CONT ;
}

int DoMat ( char *s, GNode * model ) {
    int mat;

    mat = strtol ( s, NULL, 0 );

//    if ( mat != last_mat ) {
//        if ( acsurfpass == PASS_RENDER ) glCallList ( matlist[ mat ] );
//        last_mat = mat;
//    }

    return PARSE_CONT ;
}


int DoRefs ( char *s, GNode * model ) {
    int i;
    int nrefs;
    char buffer[ 1024 ] ;
    int *vlist;
    float *tlistu, *tlistv;

    int vertNum;
    float texCoordU;
    float texCoordV;
    int error = 0;

    nrefs = strtol ( s, NULL, 0 );

    if ( nrefs == 0 ) return PARSE_POP;

//    if ( need_texture && !texture_enabled ) {
//        if ( acsurfpass == PASS_RENDER ) glEnable ( GL_TEXTURE_2D );
//        texture_enabled = 1;
//    } else if ( !need_texture && texture_enabled ) {
//        if ( acsurfpass == PASS_RENDER ) glDisable ( GL_TEXTURE_2D );
//        texture_enabled = 0;
//    }

    vlist = ( int * ) malloc ( sizeof( int ) * nrefs );
    tlistu = ( float * ) malloc ( sizeof( float ) * nrefs ) ;
    tlistv = ( float * ) malloc ( sizeof( float ) * nrefs ) ;

//    DoBegin ( current_flags );

    for ( i = 0; i < nrefs; i++ ) {
        fgets ( buffer, 1024, acfd );
	

        vlist[ i ] = vertNum = 0;
        tlistu[ i ] = texCoordU = 0.0f;
        tlistv[ i ] = texCoordV = 0.0f;
        
        if( 3 != sscanf( buffer, "%u %f %f", &vertNum, &texCoordU, &texCoordV ) ) {
            printf( "DoRefs: Illegal ref record: %s\n", buffer );
            error = 1;
        }
        
        vlist[ i ] = vertNum;
        tlistu[ i ] = texCoordU;
        tlistv[ i ] = texCoordV;

    }


    if ( nrefs == 3 && !error ) {
        
        Tri * newTriangle;

        newTriangle = tri_new();

//                                v--- fabs? ---------------------------------v
        newTriangle->tv[ 0 ].x = ( tlistu[ 0 ] );  newTriangle->tv[ 0 ].y = ( tlistv[ 0 ] );
        newTriangle->tv[ 1 ].x = ( tlistu[ 1 ] );  newTriangle->tv[ 1 ].y = ( tlistv[ 1 ] );
        newTriangle->tv[ 2 ].x = ( tlistu[ 2 ] );  newTriangle->tv[ 2 ].y = ( tlistv[ 2 ] );
        
        newTriangle->vertices[ 0 ] = g_list_nth_data( ((Mesh*)model->data)->points, vlist[0] );
        newTriangle->vertices[ 1 ] = g_list_nth_data( ((Mesh*)model->data)->points, vlist[1] );
        newTriangle->vertices[ 2 ] = g_list_nth_data( ((Mesh*)model->data)->points, vlist[2] );
        
        if( newTriangle->vertices[ 0 ] && newTriangle->vertices[ 1 ] && newTriangle->vertices[ 2 ] )
            tris_add_v( (Mesh*)model->data, newTriangle );


    } else if ( nrefs == 4 && !error ) {

// for quads.
// FIXME - yuck, what a hack...        
        Tri * newTriangle1;
        Tri * newTriangle2;

        newTriangle1 = tri_new();
        newTriangle2 = tri_new();

        newTriangle1->tv[ 0 ].x = tlistu[ 0 ];  newTriangle1->tv[ 0 ].y = tlistv[ 0 ];
        newTriangle1->tv[ 1 ].x = tlistu[ 1 ];  newTriangle1->tv[ 1 ].y = tlistv[ 1 ];
        newTriangle1->tv[ 2 ].x = tlistu[ 2 ];  newTriangle1->tv[ 2 ].y = tlistv[ 2 ];
        
        newTriangle1->vertices[ 0 ] = g_list_nth_data( ((Mesh*)model->data)->points, vlist[0] );
        newTriangle1->vertices[ 1 ] = g_list_nth_data( ((Mesh*)model->data)->points, vlist[1] );
        newTriangle1->vertices[ 2 ] = g_list_nth_data( ((Mesh*)model->data)->points, vlist[2] );



        newTriangle2->tv[ 0 ].x = tlistu[ 0 ];  newTriangle2->tv[ 0 ].y = tlistv[ 0 ];
        newTriangle2->tv[ 1 ].x = tlistu[ 2 ];  newTriangle2->tv[ 1 ].y = tlistv[ 2 ];
        newTriangle2->tv[ 2 ].x = tlistu[ 3 ];  newTriangle2->tv[ 2 ].y = tlistv[ 3 ];
        
        newTriangle2->vertices[ 0 ] = g_list_nth_data( ((Mesh*)model->data)->points, vlist[0] );
        newTriangle2->vertices[ 1 ] = g_list_nth_data( ((Mesh*)model->data)->points, vlist[2] );
        newTriangle2->vertices[ 2 ] = g_list_nth_data( ((Mesh*)model->data)->points, vlist[3] );
        
        if( newTriangle1->vertices[ 0 ] && newTriangle1->vertices[ 1 ] && newTriangle1->vertices[ 2 ] )
            tris_add_v( (Mesh*)model->data, newTriangle1 );

        if( newTriangle2->vertices[ 0 ] && newTriangle2->vertices[ 1 ] && newTriangle2->vertices[ 2 ] )
            tris_add_v( (Mesh*)model->data, newTriangle2 );


    }

    free ( vlist );
    free ( tlistu );
    free ( tlistv );


    return PARSE_POP;
}

int DoKids ( char *s, GNode * model ) {
    last_num_kids = strtol ( s, NULL, 0 );
    return PARSE_POP;
}


/* **************************************************************************
 *
 * ALL STUFF BELOW IS DEAD CODE, KEPT ONLY FOR REFERENCE
 * 
 * 
 * MOVE ALONG... NOTHING TO SEE HERE...
 * 
 ************************************************************************* */


#if 0

void model_gen_display_list( Model *m ) {
    GList * c;
    Tri *tri;

    if ( m->list )
        glDeleteLists( m->list, 1 );

    m->list = glGenLists( 1 );
    glNewList( m->list, GL_COMPILE );
    glBegin( GL_TRIANGLES );
    for ( c = m->triangles; c; c = g_list_next( c ) ) {
        tri = ( Tri * ) c->data;
        glVertex3d( tri->vertices[ 0 ] ->v[ 0 ],
                    tri->vertices[ 0 ] ->v[ 1 ],
                    tri->vertices[ 0 ] ->v[ 2 ] );
        glVertex3d( tri->vertices[ 1 ] ->v[ 0 ],
                    tri->vertices[ 1 ] ->v[ 1 ],
                    tri->vertices[ 1 ] ->v[ 2 ] );
        glVertex3d( tri->vertices[ 2 ] ->v[ 0 ],
                    tri->vertices[ 2 ] ->v[ 1 ],
                    tri->vertices[ 2 ] ->v[ 2 ] );
    }
    glEnd();
    glEndList();
}


GLuint model_get_display_list( Model *m ) {
    if ( m->dirty )
        model_gen_display_list( m );
    return m->list;
}

void model_draw( Model *m ) {
    if ( m->trans ) {
        glMatrixMode( GL_MODELVIEW );
        glPushMatrix();
        glLoadIdentity();
        glMultMatrixd( m->trans->modelview );
    }
    glCallList( model_get_display_list( m ) );
    if ( m->trans )
        glPopMatrix();
}


Model *model_load_a( char *filename ) {
    FILE * fp;
    char *string, *tmp, *s1, *s2;
    int t, cn, x, y, textured;
    tex_v *tex_vert[ 3 ];
    Point *point, *pv[ 3 ];
    GList *list;
    Model *model;
    tex *newtex, *tmp_tex = NULL;
    Tri *new_tri;
 
    fp = fopen( filename, "r" );
 
    string = get_line_from_file( fp );
 
    if ( strncmp( "b3d somethingorother", string, foooooo ) ) return NULL;
    free( string );
 
    model = model_new();
 
    string = get_line_from_file( fp );
    while ( string[ 0 ] != '\0' ) { // avoid segfaults....please
        if ( strncmp( "MATERIAL ", string, 9 ) ) break;
//        newtex = ( tex * ) malloc( sizeof( tex ) );
//        newtex->name = ( char * ) malloc( strlen( string + 8 ) + 1 );
//        strcpy( newtex->name, string + 8 );
//        free( string );
 
//        string = get_line_from_file( fp );
//        if ( strncmp( string, " FILENAME ", 10 ) ) return NULL;
//        newtex->filename = ( char * ) malloc( strlen( string + 10 ) + 1 );
//        strcpy( newtex->filename, string + 10 );
        free( string );
 
//        model->textures = g_list_append( model->textures, ( gpointer ) newtex );
        string = get_line_from_file( fp );
    }
    model_load_textures( model );
 
    while ( string[ 0 ] != '\0' ) {
        if ( strncmp( string, "POINT ", 6 ) ) break;
        // FIXME: check for ordinality
        free( string );
 
        point = ( Point * ) malloc( sizeof( Point ) );
        for ( cn = 0; cn < 3; cn++ ) {
            string = get_line_from_file( fp );
            point->v[ cn ] = atof( string + 1 );
            free( string );
        }
        model->points = g_list_append( model->points, ( gpointer ) point );
 
        string = get_line_from_file( fp );
    }
 
    while ( string[ 0 ] != '\0' ) {
        if ( strncmp( string, "TRIANGLE ", 9 ) ) break;
        free( string );
 
        string = get_line_from_file( fp );
        if ( strcmp( string, " POINTS" ) ) break;
        free( string );
 
        for ( cn = 0; cn < 3; cn++ ) {
            string = get_line_from_file( fp );
            tmp = string + 2;
            t = atoi( tmp );
            pv[ cn ] = ( Point * ) ( g_list_nth( model->points, t ) ->data );
            pv[ cn ] ->order = t;
            free( string );
        }
 
        string = get_line_from_file( fp );
        textured = 0;
        if ( !strncmp( string, " TEXTURE", 8 ) ) {
            textured = 1;
            for ( list = model->textures; list; list = list->next ) {
                tmp_tex = ( tex * ) list->data;
                if ( !strcmp( tmp_tex->name, string + 9 ) )
                    break;
            }
            free( string );
            string = get_line_from_file( fp );
            for ( y = 0; y < 3; y++ ) {
                s1 = string + 2;
                for ( x = 0; x < strlen( string ); x++ )
                    if ( string[ x ] == ',' ) break;
                string[ x ] = '\0';
                s2 = string + x + 1;
                tex_vert[ y ] = ( tex_v * ) malloc( sizeof( tex_v ) );
                tex_vert[ y ] ->x = atof( s1 );
                tex_vert[ y ] ->y = atof( s2 );
                free( string );
                string = get_line_from_file( fp );
            }
        }
        new_tri = model_add_triangle( model, pv );
        if ( textured ) {
            new_tri->tv[ 0 ] = tex_vert[ 0 ];
            new_tri->tv[ 1 ] = tex_vert[ 1 ];
            new_tri->tv[ 2 ] = tex_vert[ 2 ];
            new_tri->texture = tmp_tex;
        }
    }
 
    if ( strcmp( string, "END B3D" ) )
        printf( "Missing end of file signature.\n" );
 
    free( string );
    return model;
}



void DoBegin ( int flags ) {
    if ( last_flags == -1 ) last_flags = ~flags;

    if ( ( ( flags >> 4 ) & 0x01 ) != ( ( last_flags >> 4 ) & 0x01 ) ) {
        if ( ( flags >> 4 ) & 0x01 ) {
            /* Smooth Shaded */
            acsmooth = 1;
        } else {
            /* Flat Shaded */
            acsmooth = 0;
        }
    }
    /**
    	if (((flags>>4) & 0x02) != ((last_flags>>4) & 0x02))
    	{
    		if ((flags>>4) & 0x02)
    		{
    			if (acsurfpass == PASS_RENDER) glDisable (GL_CULL_FACE);
    		}
    		else
    		{
    			if (acsurfpass == PASS_RENDER) glEnable (GL_CULL_FACE);
    		}
    	}
     
    	if ((last_flags & 0x0F) == 0  && (flags & 0x0F) != 0)
    	{
    		if (acsurfpass == PASS_RENDER) glDisable (GL_LIGHTING);
    	}
    	else if ((last_flags & 0x0F) != 0  && (flags & 0x0F) == 0)
    	{
    		if (acsurfpass == PASS_RENDER) glEnable (GL_LIGHTING);
    	}
    **/
    last_flags = flags;

    switch ( flags & 0x0F ) {
    case 0:
//        if ( acsurfpass == PASS_RENDER ) glBegin ( GL_POLYGON );
        break;

    case 1:
//        if ( acsurfpass == PASS_RENDER ) glBegin ( GL_LINE_LOOP );
        break;

    case 2:
//        if ( acsurfpass == PASS_RENDER ) glBegin ( GL_LINE_STRIP );
        break;

    default:
        printf( "DoBegin: Illegal surface type 0x%02x\n", flags );
        break;
    }
}


ScanForTextures()
/*
 *  Look through the AC3D file for all "texture" lines
 */
{
    char buf[ 1024 ], fn[ 256 ];

    /* Look for "texture" lines */
    while ( NULL != fgets ( buf, 1024, acfd ) ) {
        if ( 1 == sscanf ( buf, "texture %s", fn ) ) {
            RemoveQuotes ( fn );
            LoadTexture ( fn );
        }
    }

    /* Rewind the file */
    fseek ( acfd, 0, SEEK_SET );
}

LoadTexture ( s )
char *s;
/*
 *  Load a texture file
 */
{
    char fn[ 256 ];

    printf( "LoadTexture: Loading texture %s\n", s );

    /* Construct file name */
    sprintf ( fn, "models/%s", s );

    /* Get a texture id */
    glGenTextures ( 1, &tex[ ntex ].id );
    strcpy ( tex[ ntex ].name, s );

    /* Define the texture */
    glBindTexture ( GL_TEXTURE_2D, tex[ ntex ].id );
    glPixelStorei ( GL_UNPACK_ALIGNMENT, 1 );
    load_sgi_texture ( fn );
    glTexParameteri ( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
    glTexParameteri ( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_LINEAR );
    glTexParameteri ( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
    glTexParameteri ( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );

    ntex++;
}

void ACNormalize ( float *v ) {
    float mag;

    mag = sqrt ( v[ 0 ] * v[ 0 ] + v[ 1 ] * v[ 1 ] + v[ 2 ] * v[ 2 ] );

    if ( mag == 0.0f ) {
        v[ 0 ] = v[ 1 ] = 0.0f;
        v[ 2 ] = 1.0f;
        return ;
    }

    v[ 0 ] /= mag;
    v[ 1 ] /= mag;
    v[ 2 ] /= mag;
}

void CrossProduct ( float *dst, float *a, float *b ) {
    dst[ 0 ] = a[ 1 ] * b[ 2 ] - a[ 2 ] * b[ 1 ];
    dst[ 1 ] = a[ 2 ] * b[ 0 ] - a[ 0 ] * b[ 2 ];
    dst[ 2 ] = a[ 0 ] * b[ 1 ] - a[ 1 ] * b[ 0 ];
}

void MakeNormal ( float *dst, float *a, float *b, float *c ) {
    float ab[ 3 ];
    float ac[ 3 ];

    ab[ 0 ] = b[ 0 ] - a[ 0 ];
    ab[ 1 ] = b[ 1 ] - a[ 1 ];
    ab[ 2 ] = b[ 2 ] - a[ 2 ];
    ACNormalize ( ab );

    ac[ 0 ] = c[ 0 ] - a[ 0 ];
    ac[ 1 ] = c[ 1 ] - a[ 1 ];
    ac[ 2 ] = c[ 2 ] - a[ 2 ];
    ACNormalize ( ac );

    CrossProduct ( dst, ab, ac );
    ACNormalize ( dst );
}
#endif




