/* 
 * File:         fileops.c
 * 
 * Description:  load and save dialogs
 * 
 * 
 * This source code is part of kludge3d, and is released under the 
 * GNU General Public License.
 * 
 * 
 */
 
#include <stdio.h>

#include "fileops.h"
#include "points.h"
#include "tri.h"
#include "frame.h"
#include "model.h"
#include "fileio.h"
#include "group_browser.h"

#include "memwatch.h"


char *save_filename = NULL;

//int save_to_b3d( char *filename ) {
//    fileio_save_ac3d( model_root, filename );
//
//    return 0;
//}


gint destroy_dialog( GtkWidget *widget, gpointer data ) {
    gtk_widget_destroy( GTK_WIDGET( data ) );

    return TRUE;
}

gint save_as_ok( GtkWidget *widget, gpointer data ) {
    char * sf_tmp;

    sf_tmp = gtk_file_selection_get_filename( GTK_FILE_SELECTION( data ) );

    if ( save_filename != NULL )
        free( save_filename );
    save_filename = ( char * ) malloc( strlen( sf_tmp ) );
    strcpy( save_filename, sf_tmp );

    fileio_save_ac3d( model_root, save_filename );

    gtk_widget_destroy( GTK_WIDGET( data ) );

    return TRUE;
}

gint show_save_dialog( GtkWidget *widget, gpointer data ) {
    GtkWidget * file_selector;

    file_selector = gtk_file_selection_new( "Save As..." );
    if ( save_filename != NULL )
        gtk_file_selection_set_filename( GTK_FILE_SELECTION( file_selector ),
                                         save_filename );
    gtk_widget_show( file_selector );

    gtk_signal_connect( GTK_OBJECT( GTK_FILE_SELECTION( file_selector ) ->
                                    cancel_button ), "clicked",
                        GTK_SIGNAL_FUNC( destroy_dialog ), file_selector );
    gtk_signal_connect( GTK_OBJECT( GTK_FILE_SELECTION( file_selector ) ->
                                    ok_button ), "clicked",
                        GTK_SIGNAL_FUNC( save_as_ok ), file_selector );

    return TRUE;
}

gint new_dialog_ok( GtkWidget *widget, gpointer data ) {

    model_delete();
    
    model_root = model_new();

    draw_all_frames();
    docgroup_rebuild_tree();

    return FALSE;
}

gint show_new_dialog( GtkWidget *widget, gpointer data ) {
    GtkWidget * dialog, *label, *button;

    dialog = gtk_dialog_new();
    button = gtk_button_new_with_label( "OK" );
    gtk_box_pack_start( GTK_BOX( GTK_DIALOG( dialog ) ->action_area ), button,
                        FALSE, FALSE, 5 );
    gtk_signal_connect( GTK_OBJECT( button ), "clicked",
                        GTK_SIGNAL_FUNC( new_dialog_ok ), NULL );
    gtk_signal_connect_after( GTK_OBJECT( button ), "clicked",
                              GTK_SIGNAL_FUNC( destroy_dialog ), dialog );
    gtk_widget_show( button );

    button = gtk_button_new_with_label( "Cancel" );
    gtk_box_pack_start( GTK_BOX( GTK_DIALOG( dialog ) ->action_area ), button,
                        FALSE, FALSE, 5 );
    gtk_signal_connect( GTK_OBJECT( button ), "clicked",
                        GTK_SIGNAL_FUNC( destroy_dialog ), dialog );
    gtk_widget_show( button );


    label = gtk_label_new( "  All data will be lost.\n"
                           "  Is it OK to proceed?" );
    gtk_label_set_justify( GTK_LABEL( label ), GTK_JUSTIFY_CENTER );
    gtk_widget_show( label );
    gtk_box_pack_start( GTK_BOX( GTK_DIALOG( dialog ) ->vbox ), label,
                        FALSE, FALSE, 5 );

    gtk_widget_show( dialog );

    return TRUE;
}

gint save_menu_cb( GtkWidget *widget, gpointer data ) {
    if ( save_filename == NULL )
        return show_save_dialog( widget, data );
    else
        fileio_save_ac3d( model_root, save_filename );

    return TRUE;
}

gint load_ok( GtkWidget *widget, gpointer data ) {
    char * filename = gtk_file_selection_get_filename( GTK_FILE_SELECTION( data ) );

    if ( filename == NULL ) {
        fprintf( stderr, "Big problem in load_ok.\n" );
        return TRUE;
    }

    model_delete();
    model_root = fileio_load_ac3d( filename );


    draw_all_frames();
    docgroup_rebuild_tree();
    texman_rebuild_list();

    return TRUE;
}

gint show_load_dialog( GtkWidget *widget, gpointer data ) {
    GtkWidget * file_selector;

    file_selector = gtk_file_selection_new( "Load" );
    gtk_widget_show( file_selector );

    gtk_signal_connect( GTK_OBJECT( GTK_FILE_SELECTION( file_selector ) ->
                                    cancel_button ), "clicked",
                        GTK_SIGNAL_FUNC( destroy_dialog ), file_selector );
    gtk_signal_connect( GTK_OBJECT( GTK_FILE_SELECTION( file_selector ) ->
                                    ok_button ), "clicked",
                        GTK_SIGNAL_FUNC( load_ok ), file_selector );
    gtk_signal_connect_after( GTK_OBJECT( GTK_FILE_SELECTION( file_selector ) ->
                                          ok_button ), "clicked",
                              GTK_SIGNAL_FUNC( destroy_dialog ), file_selector );

    return TRUE;
}


