/* 
 * File:         frame.c
 * 
 * Description:  manages the 4 view panes
 * 
 * 
 * This source code is part of kludge3d, and is released under the 
 * GNU General Public License.
 * 
 * 
 */
 

#include "view.h"
#include "frame.h"

#include "memwatch.h"


GtkWidget *table_child[ 4 ];
GtkWidget *table;

int grid_spacing = 0;
int grid_snap = FALSE;

gint expand_cb( GtkWidget *widget, gpointer data ) {
    int c;
    static int ms = 1;

    if ( ms ) {
        for ( c = 0; c < 4; c++ )
            if ( table_child[ c ] != ( GtkWidget * ) data ) gtk_widget_hide( table_child[ c ] );
        gtk_table_set_homogeneous( GTK_TABLE( table ), 0 );
    } else {
        for ( c = 0; c < 4; c++ )
            if ( table_child[ c ] != ( GtkWidget * ) data ) gtk_widget_show( table_child[ c ] );
        gtk_table_set_homogeneous( GTK_TABLE( table ), 1 );
    }

    ms = ms ? 0 : 1;

    return FALSE;
}


GtkWidget *frame_new( GtkWidget * w, int projection_type ) {
    GtkWidget * button, *vbox;

    vbox = gtk_vbox_new( FALSE, 0 );
    switch( projection_type ) {
        case FRAME_XY:
            button = gtk_button_new_with_label( "XY - Top" );
            break;
        case FRAME_YZ:
            button = gtk_button_new_with_label( "YZ - Side" );
            break;
        case FRAME_XZ:
            button = gtk_button_new_with_label( "XZ - Side" );
            break;
        case FRAME_3D:
            button = gtk_button_new_with_label( "3D Preview" );
            break;
        default:
            button = gtk_button_new_with_label( "Something Bad" );
            break;
    }

    gtk_box_pack_start( GTK_BOX( vbox ), button, FALSE, TRUE, 0 );
    gtk_box_pack_start( GTK_BOX( vbox ), w, TRUE, TRUE, 0 );

    gtk_widget_show( vbox );
    gtk_widget_show( button );

    gtk_signal_connect( GTK_OBJECT( button ), "clicked",
                        GTK_SIGNAL_FUNC( expand_cb ), ( gpointer ) vbox );

    return vbox;
}


GtkWidget * create_frames( ) {
    GtkWidget * glp;

    table = gtk_table_new( 2, 2, 1 );
    gtk_widget_show( table );


    glp = frame_new( create_view( FRAME_XY ), FRAME_XY );
    gtk_table_attach_defaults( GTK_TABLE( table ), glp, 0, 1, 0, 1 );
    table_child[ FRAME_XY ] = glp;


    glp = frame_new( create_view( FRAME_YZ ), FRAME_YZ );
    gtk_table_attach_defaults( GTK_TABLE( table ), glp, 1, 2, 0, 1 );
    table_child[ FRAME_YZ ] = glp;


    glp = frame_new( create_view( FRAME_XZ ), FRAME_XZ );
    gtk_table_attach_defaults( GTK_TABLE( table ), glp, 0, 1, 1, 2 );
    table_child[ FRAME_XZ ] = glp;


    glp = frame_new( create_view( FRAME_3D ), FRAME_3D );
    gtk_table_attach_defaults( GTK_TABLE( table ), glp, 1, 2, 1, 2 );
    table_child[ FRAME_3D ] = glp;


    return table;
}

void draw_all_frames( void ) {
    int i;
    
    for( i = 0; i < 4; i++ ) {
        gtk_widget_draw( views[i]->glarea, NULL );
    }


}

void draw_frame_n( int frame_num ) {
    
    if( frame_num >= FRAME_XY && frame_num <= FRAME_3D ) {
        gtk_widget_draw( views[frame_num]->glarea, NULL );
    }


}





