/* 
 * File:         group_browser.c
 * 
 * Description:  creates and manages the "group" notebook tab, also handles
 * 		 group-related functionality
 * 
 * This source code is part of kludge3d, and is released under the 
 * GNU General Public License.
 * 
 * 
 */
 
/* This file has its origins in:
 * ME3D 3-D Modeler Program
 * Copyright (C) 1998-2000 Sam Revitch
 */

/* Group browser code */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>


#include "group_browser.h"
#include "globals.h"
#include "model.h"
#include "frame.h"
#include "mesh.h"
#include "points.h"
#include "tri.h"

#include "memwatch.h"


/* Pixmaps are attached to buttons here */
#include "pixmaps/group_new.xpm"
#include "pixmaps/group_new_is.xpm"
#include "pixmaps/group_focus.xpm"
#include "pixmaps/group_focus_is.xpm"
#include "pixmaps/group_duplicate.xpm"
#include "pixmaps/group_duplicate_is.xpm"
#include "pixmaps/group_ungroup.xpm"
#include "pixmaps/group_ungroup_is.xpm"
#include "pixmaps/group_delete.xpm"
#include "pixmaps/group_delete_is.xpm"

//extern GtkWidget *tool_window; 		/* for pixmap creation reference */

void docgroup_action_button_cb( GtkWidget *widget, gint idx );
void docgroup_button_set_sensitive( gint button_idx, gint sensitive );

void docgroup_new_group_dlg( GNode *parent );
void docgroup_new_group_cb( GtkWidget *widget, gpointer data );
void docgroup_new_group_dlg_close( GtkWidget *widget, gpointer data );

void docgroup_edit_dlg( GNode *group );
void docgroup_edit_cb( GtkWidget *widget, gpointer data );
void docgroup_edit_dlg_close( GtkWidget *widget, gpointer data );

void docgroup_build_tree( GNode *group, GtkWidget *item_tree, gint pos );
void docgroup_remove_subtree( GNode *group );
void docgroup_dehash( GNode *group );
void docgroup_change_selection( GtkWidget *widget, GNode *group );
void docgroup_clear_selection();
gint docgroup_item_click( GtkWidget *widget, GdkEvent *event,
                          GNode *group );
void docgroup_tree_install_dnd( GtkWidget *widget, GNode *group );
void docgroup_tree_drag_handle( GtkWidget *widget,
                                GdkDragContext *context,
                                GtkSelectionData *selection_data,
                                guint info, guint time,
                                GNode *group );
void docgroup_tree_drop_handle( GtkWidget *widget,
                                GdkDragContext *context,
                                gint x, gint y,
                                GtkSelectionData *selection_data,
                                guint info, guint time,
                                GNode *ctx );


gint docgroup_pointers_equal( gconstpointer p1, gconstpointer p2 ) {
    if ( p1 == p2 ) return TRUE;
    return FALSE;
}


struct docgroup_button_data {
    gchar **pix;
    gchar **pix_is;
    gchar *tooltip;
};

enum docgroup_actions {
    DOCGROUP_NEW = 0,
    DOCGROUP_FOCUS,
    DOCGROUP_EDIT,
    DOCGROUP_DUPE,
    DOCGROUP_DELETE,
    DOCGROUP_NBUTTONS
};


struct docgroup_button_data docgroup_bd[] = {
    { group_new_xpm,       group_new_is_xpm,       "Create Empty Group" },
    { group_focus_xpm,     group_focus_is_xpm,     "Select Points" },
    { group_ungroup_xpm,   group_ungroup_is_xpm,   "Edit Group" },
    { group_duplicate_xpm, group_duplicate_is_xpm, "Disabled" },        //"Duplicate Group" },
    { group_delete_xpm,    group_delete_is_xpm,    "Delete Group" }
};


struct docgroup_browser {
    GtkWidget *OptionMenu;
    GtkWidget *Tree; 	/* group tree widget */
    GtkWidget *GrpButtons[ DOCGROUP_NBUTTONS ][ 2 ];
    GdkPixmap *grp_pix[ DOCGROUP_NBUTTONS ][ 2 ];
    GdkBitmap *grp_mask[ DOCGROUP_NBUTTONS ][ 2 ];
    guint group_button_states; 		/* bitfield */


    GNode *selected_grp;

    GHashTable *gs_hash;
}
docgroup_s[] = { { NULL } };


struct docgroup_newgroup {
    GtkWidget *MainWin;
    GtkWidget *n_ent;
    GtkWidget *x_ent, *y_ent, *z_ent;
//    struct model_doc *doc;
    GNode *parent;
}
docgroup_ng[] = { { NULL } };


struct docgroup_rename {
    GtkWidget *MainWin;
    GtkWidget *n_ent;
    GtkWidget *x_ent, *y_ent, *z_ent;
//    struct model_doc *doc;
    GNode *group;
}
docgroup_rg[] = { { NULL } };


//extern struct model_doc *first_doc, *last_doc;


GtkWidget * create_group_browser( void ) {

    GtkWidget *widget, *vbox2, *hbox;
//    GtkStyle *style;
    GtkTooltips *tips;
    int i;

    vbox2 = gtk_vbox_new( FALSE, 4 );
    gtk_container_border_width( GTK_CONTAINER( vbox2 ), 4 );
    gtk_widget_show( vbox2 );

    widget = gtk_scrolled_window_new( NULL, NULL );
    gtk_scrolled_window_set_policy( GTK_SCROLLED_WINDOW( widget ),
                                    GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC );
    gtk_box_pack_start( GTK_BOX( vbox2 ), widget, TRUE, TRUE, 0 );
    gtk_widget_show( widget );

    docgroup_s->Tree = gtk_tree_new();
    gtk_scrolled_window_add_with_viewport( GTK_SCROLLED_WINDOW( widget ),
                                           docgroup_s->Tree );
    gtk_widget_show( docgroup_s->Tree );

    hbox = gtk_hbox_new( FALSE, 0 );
    gtk_box_pack_start( GTK_BOX( vbox2 ), hbox, FALSE, FALSE, 0 );
    gtk_widget_show( hbox );

    /* action buttons for group tab */
//    style = gtk_widget_get_style( tool_window );
    for ( i = 0; i < DOCGROUP_NBUTTONS; i++ ) {
        /* button itself */
        docgroup_s->GrpButtons[ i ][ 0 ] = gtk_button_new();
        gtk_box_pack_start( GTK_BOX( hbox ), docgroup_s->GrpButtons[ i ][ 0 ], TRUE, TRUE, 0 );
        gtk_signal_connect( GTK_OBJECT( docgroup_s->GrpButtons[ i ][ 0 ] ), "clicked",
                            GTK_SIGNAL_FUNC( docgroup_action_button_cb ), ( gpointer ) i );
        gtk_widget_show( docgroup_s->GrpButtons[ i ][ 0 ] );

        /* tooltip */
        tips = gtk_tooltips_new();
        gtk_tooltips_set_tip( tips, docgroup_s->GrpButtons[ i ][ 0 ],
                              docgroup_bd[ i ].tooltip, NULL );
        gtk_tooltips_enable( tips );

        /* sensitive pixmap */
        docgroup_s->grp_pix[ i ][ 0 ] = gdk_pixmap_create_from_xpm_d(
                                            window->window, &( docgroup_s->grp_mask[ i ][ 0 ] ),
                                            &window->style->bg[ GTK_STATE_NORMAL ], 
                                            ( gchar ** ) docgroup_bd[ i ].pix );

        /* insensitive pixmap */
        docgroup_s->grp_pix[ i ][ 1 ] = gdk_pixmap_create_from_xpm_d(
                                            window->window, &( docgroup_s->grp_mask[ i ][ 1 ] ),
                                            &window->style->bg[ GTK_STATE_NORMAL ], 
                                            ( gchar ** ) docgroup_bd[ i ].pix_is );

        /* now set the button to initial pixmap and make insensitive */
        docgroup_s->GrpButtons[ i ][ 1 ] = gtk_pixmap_new( docgroup_s->grp_pix[ i ][ 1 ],
                                           docgroup_s->grp_mask[ i ][ 1 ] );
        gtk_widget_show( docgroup_s->GrpButtons[ i ][ 1 ] );
        gtk_container_add( GTK_CONTAINER( docgroup_s->GrpButtons[ i ][ 0 ] ),
                           docgroup_s->GrpButtons[ i ][ 1 ] );
        gtk_widget_set_sensitive( docgroup_s->GrpButtons[ i ][ 0 ], FALSE );
    }

    docgroup_s->group_button_states = 0;




    docgroup_s->gs_hash = NULL;
//    docgroup_s->last_doc = NULL;

//    docgroup_rebuild_views();
    
    
    docgroup_build_tree( model_root, docgroup_s->Tree, -1 );
    
    return vbox2;
}


void docgroup_action_button_cb( GtkWidget *widget, gint idx ) {

    g_return_if_fail( docgroup_s->selected_grp != NULL );

    switch ( idx ) {
    case DOCGROUP_NEW:
        docgroup_new_group_dlg( docgroup_s->selected_grp );
        break;
    case DOCGROUP_FOCUS:
        // we're selecting all the points in the current mesh
        point_unselect_all();
        if( current_mesh != NULL ) {
            GList *l = current_mesh->points;
            for( ; l; l = l->next ) {
                point_select_v( (Point*)l->data );
            }
        }
        draw_all_frames();
        break;
    case DOCGROUP_EDIT:
        docgroup_edit_dlg( docgroup_s->selected_grp );
        break;
    case DOCGROUP_DUPE:
//        doc_group_duplicate( docgroup_s->last_doc, docgroup_s->selected_grp );
//        view_start_rendercycle_docwide( docgroup_s->last_doc );
        break;
    case DOCGROUP_DELETE:
printf( "current_node: %p  model_root: %p\n", current_node, model_root );
        if( current_node != NULL && current_node != model_root ){
            GNode * temp = current_node;  // clear_selected screws us up, 
                                          // so we save current_node
                                          // before we do anything else
            docgroup_remove_subtree( current_node );
            model_delete_node( temp );
        }
        current_node = NULL;
        docgroup_rebuild_tree(  );
        draw_all_frames();
        break;
    }

//called by above funcs, no need for it here...
//docgroup_rebuild_tree(  );

}

void docgroup_button_set_sensitive( gint button_idx, gint sensitive ) {
    g_return_if_fail( button_idx < DOCGROUP_NBUTTONS );

    if ( sensitive ) {
        if ( docgroup_s->group_button_states & ( 1 << button_idx ) )
            return ;
        docgroup_s->group_button_states |= ( 1 << button_idx );

        gtk_pixmap_set( GTK_PIXMAP( docgroup_s->GrpButtons[ button_idx ][ 1 ] ),
                        docgroup_s->grp_pix[ button_idx ][ 0 ],
                        docgroup_s->grp_mask[ button_idx ][ 0 ] );
        gtk_widget_set_sensitive( docgroup_s->GrpButtons[ button_idx ][ 0 ], TRUE );

    } else {
        if ( !( docgroup_s->group_button_states & ( 1 << button_idx ) ) )
            return ;
        docgroup_s->group_button_states &= ~( 1 << button_idx );

        gtk_pixmap_set( GTK_PIXMAP( docgroup_s->GrpButtons[ button_idx ][ 1 ] ),
                        docgroup_s->grp_pix[ button_idx ][ 1 ],
                        docgroup_s->grp_mask[ button_idx ][ 1 ] );
        gtk_widget_set_sensitive( docgroup_s->GrpButtons[ button_idx ][ 0 ], FALSE );
    }
}



void docgroup_new_group_dlg( GNode *parent ) {

    if ( docgroup_ng->MainWin == NULL ) {
        GtkWidget * widget, *vbox, *hbox, *tab;

        /* initialize the new group dialog */
        docgroup_ng->MainWin = gtk_window_new( GTK_WINDOW_TOPLEVEL );
        gtk_widget_set_usize( docgroup_ng->MainWin, 180, 200 );
        gtk_window_set_title( GTK_WINDOW( docgroup_ng->MainWin ), "New Group Options" );
        gtk_signal_connect( GTK_OBJECT( docgroup_ng->MainWin ), "delete_event",
                            GTK_SIGNAL_FUNC( docgroup_new_group_dlg_close ), NULL );
        gtk_container_border_width( GTK_CONTAINER( docgroup_ng->MainWin ), 4 );

        vbox = gtk_vbox_new( FALSE, 4 );
        gtk_container_add( GTK_CONTAINER( docgroup_ng->MainWin ), vbox );
        gtk_widget_show( vbox );

        hbox = gtk_hbox_new( FALSE, 4 );
        gtk_box_pack_start( GTK_BOX( vbox ), hbox, FALSE, FALSE, 0 );
        gtk_widget_show( hbox );

        widget = gtk_label_new( "Name:" );
        gtk_box_pack_start( GTK_BOX( hbox ), widget, FALSE, FALSE, 0 );
        gtk_widget_show( widget );
        docgroup_ng->n_ent = gtk_entry_new();
        gtk_box_pack_start( GTK_BOX( hbox ), docgroup_ng->n_ent, TRUE, TRUE, 0 );
        gtk_widget_show( docgroup_ng->n_ent );

        widget = gtk_frame_new( "Location" );
        gtk_box_pack_start( GTK_BOX( vbox ), widget, TRUE, TRUE, 0 );
        gtk_widget_show( widget );

        tab = gtk_table_new( 3, 2, FALSE );
        gtk_container_add( GTK_CONTAINER( widget ), tab );
        gtk_widget_show( tab );

        widget = gtk_label_new( "X:" );
        gtk_table_attach_defaults( GTK_TABLE( tab ), widget, 0, 1, 0, 1 );
        gtk_widget_show( widget );
        docgroup_ng->x_ent = gtk_entry_new();
        gtk_table_attach_defaults( GTK_TABLE( tab ), docgroup_ng->x_ent, 1, 2, 0, 1 );
        gtk_widget_show( docgroup_ng->x_ent );

        widget = gtk_label_new( "Y:" );
        gtk_table_attach_defaults( GTK_TABLE( tab ), widget, 0, 1, 1, 2 );
        gtk_widget_show( widget );
        docgroup_ng->y_ent = gtk_entry_new();
        gtk_table_attach_defaults( GTK_TABLE( tab ), docgroup_ng->y_ent, 1, 2, 1, 2 );
        gtk_widget_show( docgroup_ng->y_ent );

        widget = gtk_label_new( "Z:" );
        gtk_table_attach_defaults( GTK_TABLE( tab ), widget, 0, 1, 2, 3 );
        gtk_widget_show( widget );
        docgroup_ng->z_ent = gtk_entry_new();
        gtk_table_attach_defaults( GTK_TABLE( tab ), docgroup_ng->z_ent, 1, 2, 2, 3 );
        gtk_widget_show( docgroup_ng->z_ent );


        widget = gtk_hseparator_new();
        gtk_box_pack_start( GTK_BOX( vbox ), widget, FALSE, FALSE, 0 );
        gtk_widget_show( widget );

        /* OK/Cancel buttons */
        hbox = gtk_hbox_new( FALSE, 4 );
        gtk_box_pack_start( GTK_BOX( vbox ), hbox, FALSE, FALSE, 0 );
        gtk_widget_show( hbox );
        widget = gtk_button_new_with_label( "OK" );
        gtk_box_pack_start( GTK_BOX( hbox ), widget, TRUE, TRUE, 0 );
        gtk_signal_connect( GTK_OBJECT( widget ), "clicked",
                            GTK_SIGNAL_FUNC( docgroup_new_group_cb ), NULL );
        GTK_WIDGET_SET_FLAGS( widget, GTK_CAN_DEFAULT );
        gtk_widget_show( widget );
        gtk_widget_grab_default( widget );

        widget = gtk_button_new_with_label( "Cancel" );
        gtk_box_pack_start( GTK_BOX( hbox ), widget, TRUE, TRUE, 0 );
        gtk_signal_connect( GTK_OBJECT( widget ), "clicked",
                            GTK_SIGNAL_FUNC( docgroup_new_group_dlg_close ), NULL );
        GTK_WIDGET_SET_FLAGS( widget, GTK_CAN_DEFAULT );
        gtk_widget_show( widget );
    }

    gtk_entry_set_text( GTK_ENTRY( docgroup_ng->n_ent ),
                        "New Group" );
    gtk_entry_set_text( GTK_ENTRY( docgroup_ng->x_ent ), "0.0" );
    gtk_entry_set_text( GTK_ENTRY( docgroup_ng->y_ent ), "0.0" );
    gtk_entry_set_text( GTK_ENTRY( docgroup_ng->z_ent ), "0.0" );

//    docgroup_ng->doc = doc;
    docgroup_ng->parent = parent;
//    view_set_sensitive_docwide( doc, FALSE );

    gtk_widget_show( docgroup_ng->MainWin );
}

void docgroup_new_group_cb( GtkWidget *widget, gpointer data ) {
    gchar * text;
    Point* trans = NULL;
    GNode *group;
    
    trans = (Point*)malloc( sizeof(Point) );
    
    gtk_widget_hide( docgroup_ng->MainWin );
//    view_set_sensitive_docwide( docgroup_ng->doc, TRUE );

    text = gtk_entry_get_text( GTK_ENTRY( docgroup_ng->n_ent ) );

    group = model_new( );
    g_node_append( docgroup_ng->parent, group );

    trans->v[ 0 ] = atof( gtk_entry_get_text( GTK_ENTRY( docgroup_ng->x_ent ) ) );
    trans->v[ 1 ] = atof( gtk_entry_get_text( GTK_ENTRY( docgroup_ng->y_ent ) ) );
    trans->v[ 2 ] = atof( gtk_entry_get_text( GTK_ENTRY( docgroup_ng->z_ent ) ) );

    mesh_set_location( (Mesh *)group->data, trans );
    mesh_set_name( (Mesh *)group->data, text );

//    view_start_rendercycle_docwide( docgroup_ng->doc );

    docgroup_rebuild_tree(  );
}

void docgroup_new_group_dlg_close( GtkWidget *widget, gpointer data ) {
    gtk_widget_hide( docgroup_ng->MainWin );
//    view_set_sensitive_docwide( docgroup_ng->doc, TRUE );
}


void docgroup_edit_dlg( GNode *group ) {

    if ( docgroup_rg->MainWin == NULL ) {
        GtkWidget * widget, *vbox, *hbox, *tab;

        docgroup_rg->MainWin = gtk_window_new( GTK_WINDOW_TOPLEVEL );
        gtk_widget_set_usize( docgroup_rg->MainWin, 180, 200 );
        gtk_window_set_title( GTK_WINDOW( docgroup_rg->MainWin ), "Edit Group" );
        gtk_signal_connect( GTK_OBJECT( docgroup_rg->MainWin ), "delete_event",
                            GTK_SIGNAL_FUNC( docgroup_edit_dlg_close ), NULL );
        gtk_container_border_width( GTK_CONTAINER( docgroup_rg->MainWin ), 4 );

        vbox = gtk_vbox_new( FALSE, 4 );
        gtk_container_add( GTK_CONTAINER( docgroup_rg->MainWin ), vbox );
        gtk_widget_show( vbox );

        hbox = gtk_hbox_new( FALSE, 4 );
        gtk_box_pack_start( GTK_BOX( vbox ), hbox, FALSE, FALSE, 0 );
        gtk_widget_show( hbox );

        widget = gtk_label_new( "Name:" );
        gtk_box_pack_start( GTK_BOX( hbox ), widget, FALSE, FALSE, 0 );
        gtk_widget_show( widget );
        docgroup_rg->n_ent = gtk_entry_new();
        gtk_box_pack_start( GTK_BOX( hbox ), docgroup_rg->n_ent, TRUE, TRUE, 0 );
        gtk_widget_show( docgroup_rg->n_ent );


        widget = gtk_frame_new( "Location" );
        gtk_box_pack_start( GTK_BOX( vbox ), widget, TRUE, TRUE, 0 );
        gtk_widget_show( widget );

        tab = gtk_table_new( 3, 2, FALSE );
        gtk_container_add( GTK_CONTAINER( widget ), tab );
        gtk_widget_show( tab );

        widget = gtk_label_new( "X:" );
        gtk_table_attach_defaults( GTK_TABLE( tab ), widget, 0, 1, 0, 1 );
        gtk_widget_show( widget );
        docgroup_rg->x_ent = gtk_entry_new();
        gtk_table_attach_defaults( GTK_TABLE( tab ), docgroup_rg->x_ent, 1, 2, 0, 1 );
        gtk_widget_show( docgroup_rg->x_ent );

        widget = gtk_label_new( "Y:" );
        gtk_table_attach_defaults( GTK_TABLE( tab ), widget, 0, 1, 1, 2 );
        gtk_widget_show( widget );
        docgroup_rg->y_ent = gtk_entry_new();
        gtk_table_attach_defaults( GTK_TABLE( tab ), docgroup_rg->y_ent, 1, 2, 1, 2 );
        gtk_widget_show( docgroup_rg->y_ent );

        widget = gtk_label_new( "Z:" );
        gtk_table_attach_defaults( GTK_TABLE( tab ), widget, 0, 1, 2, 3 );
        gtk_widget_show( widget );
        docgroup_rg->z_ent = gtk_entry_new();
        gtk_table_attach_defaults( GTK_TABLE( tab ), docgroup_rg->z_ent, 1, 2, 2, 3 );
        gtk_widget_show( docgroup_rg->z_ent );


        widget = gtk_hseparator_new();
        gtk_box_pack_start( GTK_BOX( vbox ), widget, FALSE, FALSE, 0 );
        gtk_widget_show( widget );

        /* OK/Cancel buttons */
        hbox = gtk_hbox_new( FALSE, 4 );
        gtk_box_pack_start( GTK_BOX( vbox ), hbox, FALSE, FALSE, 0 );
        gtk_widget_show( hbox );
        widget = gtk_button_new_with_label( "OK" );
        gtk_box_pack_start( GTK_BOX( hbox ), widget, TRUE, TRUE, 0 );
        gtk_signal_connect( GTK_OBJECT( widget ), "clicked",
                            GTK_SIGNAL_FUNC( docgroup_edit_cb ), NULL );
        GTK_WIDGET_SET_FLAGS( widget, GTK_CAN_DEFAULT );
        gtk_widget_show( widget );
        gtk_widget_grab_default( widget );

        widget = gtk_button_new_with_label( "Cancel" );
        gtk_box_pack_start( GTK_BOX( hbox ), widget, TRUE, TRUE, 0 );
        gtk_signal_connect( GTK_OBJECT( widget ), "clicked",
                            GTK_SIGNAL_FUNC( docgroup_edit_dlg_close ), NULL );
        GTK_WIDGET_SET_FLAGS( widget, GTK_CAN_DEFAULT );
        gtk_widget_show( widget );
    }

    
    {
    
    Mesh* m = (Mesh*)group->data;
    if( m != NULL && m->location != NULL ) {
        char s[32]; 
        memset( s, 0, 32);
        
        gtk_entry_set_text( GTK_ENTRY( docgroup_rg->n_ent ), m->name );
        snprintf( s, 31, "%f", m->location->v[0] );
        gtk_entry_set_text( GTK_ENTRY( docgroup_rg->x_ent ), s );
        snprintf( s, 31, "%f", m->location->v[1] );
        gtk_entry_set_text( GTK_ENTRY( docgroup_rg->y_ent ), s );
        snprintf( s, 31, "%f", m->location->v[2] );
        gtk_entry_set_text( GTK_ENTRY( docgroup_rg->z_ent ), s );
    
    } else {
        gtk_entry_set_text( GTK_ENTRY( docgroup_rg->n_ent ), "fixme" );  //group->name );
        gtk_entry_set_text( GTK_ENTRY( docgroup_rg->x_ent ), "0.0" );
        gtk_entry_set_text( GTK_ENTRY( docgroup_rg->y_ent ), "0.0" );
        gtk_entry_set_text( GTK_ENTRY( docgroup_rg->z_ent ), "0.0" );
    }
    
    }

//    docgroup_rg->doc = doc;
    docgroup_rg->group = group;


//    view_set_sensitive_docwide( docgroup_rg->doc, FALSE );
    gtk_widget_show( docgroup_rg->MainWin );
}

void docgroup_edit_cb( GtkWidget *widget, gpointer data ) {
    gchar * text;
    Point* trans = NULL;

    trans = (Point*)malloc( sizeof(Point) );
    
    gtk_widget_hide( docgroup_rg->MainWin );
//    view_set_sensitive_docwide( docgroup_rg->doc, TRUE );

    text = gtk_entry_get_text( GTK_ENTRY( docgroup_rg->n_ent ) );

    trans->v[ 0 ] = atof( gtk_entry_get_text( GTK_ENTRY( docgroup_rg->x_ent ) ) );
    trans->v[ 1 ] = atof( gtk_entry_get_text( GTK_ENTRY( docgroup_rg->y_ent ) ) );
    trans->v[ 2 ] = atof( gtk_entry_get_text( GTK_ENTRY( docgroup_rg->z_ent ) ) );

    mesh_set_location( (Mesh *)docgroup_rg->group->data, trans );
    mesh_set_name( (Mesh *)docgroup_rg->group->data, text );

    docgroup_rebuild_tree(  );
}

void docgroup_edit_dlg_close( GtkWidget *widget, gpointer data ) {
    gtk_widget_hide( docgroup_rg->MainWin );
//    view_set_sensitive_docwide( docgroup_rg->doc, TRUE );
}



void docgroup_insert_subtree( GNode *subtree ) {

    GNode *pG;
    GtkWidget *item_widget, *item_tree;
    int i = 0;

//    g_return_if_fail( doc != NULL );
    g_return_if_fail( subtree != NULL );
    g_return_if_fail( subtree->parent != NULL ); 	/* not the root node, plz */

//    if ( doc != docgroup_s->last_doc )
//        return ;

    item_widget = g_hash_table_lookup( docgroup_s->gs_hash, subtree->parent );
    g_return_if_fail( item_widget != NULL );

    item_tree = GTK_TREE_ITEM( item_widget ) ->subtree;
    if ( item_tree == NULL ) {
        item_tree = gtk_tree_new();
        gtk_tree_item_set_subtree( GTK_TREE_ITEM( item_widget ), item_tree );
        gtk_tree_item_expand( GTK_TREE_ITEM( item_widget ) );
    }

    /* now insert the subtree in order */
    /* first find its position */
    for( pG = subtree->parent->children; 
          ( pG != subtree ) && ( pG != NULL );
          pG = pG->next ) 
    {
        i++;
    }
          
    g_return_if_fail( pG != NULL );

    docgroup_build_tree( subtree, item_tree, i );

}


void docgroup_rebuild_tree( void ) {

    if( docgroup_s->gs_hash != NULL )
        g_hash_table_destroy( docgroup_s->gs_hash );
        
    docgroup_s->gs_hash = NULL;
    gtk_tree_clear_items( GTK_TREE(docgroup_s->Tree), 0, 999 );

    if( model_root != NULL ) {
        docgroup_s->gs_hash = g_hash_table_new( g_direct_hash, docgroup_pointers_equal );
        docgroup_build_tree( model_root, docgroup_s->Tree, -1 );
    }

    docgroup_clear_selection();

}


void docgroup_build_tree( GNode *group, GtkWidget *item_tree, gint pos ) {
    GNode *pG;
    Mesh * m = NULL;
    GtkWidget *item_widget;
    char* name;
    
    
    g_return_if_fail( group != NULL );
    
    m = (Mesh*)group->data;
    if( m != NULL ) {
        if( m->name != NULL )
            name = strdup( m->name );
        else
            name = strdup( "<null mesh->name>" );
    } else {
        name = strdup( "<null mesh>" );
    }
    
    item_widget = gtk_tree_item_new_with_label( name );
    free( name );
    
    g_return_if_fail( item_widget != NULL );

    gtk_signal_connect( GTK_OBJECT( item_widget ), "select",
                        GTK_SIGNAL_FUNC( docgroup_change_selection ),
                        ( gpointer ) group );
    g_hash_table_insert( docgroup_s->gs_hash, group, item_widget );

    gtk_signal_connect_after( GTK_OBJECT( item_widget ), "realize",
                              GTK_SIGNAL_FUNC( docgroup_tree_install_dnd ),
                              ( gpointer ) group );

    gtk_signal_connect( GTK_OBJECT( item_widget ), "button_press_event",
                        GTK_SIGNAL_FUNC( docgroup_item_click ),
                        ( gpointer ) group );

    if ( pos == -1 )
        gtk_tree_append( GTK_TREE( item_tree ), item_widget );
    else
        gtk_tree_insert( GTK_TREE( item_tree ), item_widget, pos );

    gtk_widget_show( item_widget );


    if ( group->children != NULL ) {
        item_tree = gtk_tree_new();
        g_return_if_fail( item_tree != NULL );

        gtk_tree_item_set_subtree( GTK_TREE_ITEM( item_widget ), item_tree );
        gtk_tree_item_expand( GTK_TREE_ITEM( item_widget ) );

        pG = group->children;
        while ( pG != NULL ) {
            docgroup_build_tree( pG, item_tree, -1 );
            pG = pG->next;
        }
    }

}

void docgroup_remove_subtree( GNode *group ) {
    // removes the widgets only; the model itself is unchanged

    GtkWidget *tree_item, *tree;

//    g_return_if_fail( doc != NULL );
    g_return_if_fail( group != NULL );
//    /*  g_return_if_fail(group->pUp != NULL); */

//    if ( doc != docgroup_s->last_doc )
//        return ;

    // prevent removal of the root of the tree
    tree_item = GTK_WIDGET( g_hash_table_lookup( docgroup_s->gs_hash,
                            group->parent ) );
    g_return_if_fail( tree_item != NULL );

    tree = GTK_TREE_ITEM( tree_item ) ->subtree;

    tree_item = GTK_WIDGET( g_hash_table_lookup( docgroup_s->gs_hash, group ) );
    g_return_if_fail( tree_item != NULL );

    gtk_container_remove( GTK_CONTAINER( tree ), tree_item );
    /*  gtk_widget_destroy(tree_item); */

//fixme - does it handle removal of subtrees ?  should it?

    docgroup_dehash( group );
}

void docgroup_dehash( GNode *group ) {
    GNode *pG;
    g_hash_table_remove( docgroup_s->gs_hash, group );

    if ( group == docgroup_s->selected_grp )
        docgroup_clear_selection();

    pG = group->children;
    while ( pG != NULL ) {
        docgroup_dehash( pG );
        pG = pG->next;
    }
}



void docgroup_change_selection( GtkWidget *widget, GNode *group ) {



#ifdef VERBOSE
    printf( "change_selection\n" );
#endif

    docgroup_s->selected_grp = group;
    current_mesh = (Mesh*)group->data;
    current_node = group;

    point_unselect_all();
    tris_unselect_all();
    
    docgroup_button_set_sensitive( DOCGROUP_NEW, TRUE );

    if ( group->parent == NULL ) {
        docgroup_button_set_sensitive( DOCGROUP_FOCUS, FALSE );
        docgroup_button_set_sensitive( DOCGROUP_EDIT, FALSE );
        docgroup_button_set_sensitive( DOCGROUP_DUPE, FALSE );
        docgroup_button_set_sensitive( DOCGROUP_DELETE, FALSE );  /* can't delete top group */
    } else {
        docgroup_button_set_sensitive( DOCGROUP_FOCUS, TRUE );
        docgroup_button_set_sensitive( DOCGROUP_EDIT, TRUE );
        docgroup_button_set_sensitive( DOCGROUP_DUPE, TRUE );
        docgroup_button_set_sensitive( DOCGROUP_DELETE, TRUE );
    }

    draw_all_frames();
}


void docgroup_clear_selection() {

    docgroup_s->selected_grp = NULL;
    current_mesh = NULL;
    current_node = NULL;
    
    docgroup_button_set_sensitive( DOCGROUP_NEW, FALSE );
    docgroup_button_set_sensitive( DOCGROUP_FOCUS, FALSE );
    docgroup_button_set_sensitive( DOCGROUP_EDIT, FALSE );
    docgroup_button_set_sensitive( DOCGROUP_DUPE, FALSE );
    docgroup_button_set_sensitive( DOCGROUP_DELETE, FALSE );
}

gint docgroup_item_click( GtkWidget *widget, GdkEvent *event, GNode *grp ) {

    if ( event->button.button == 1 ) {
        if ( event->button.type == GDK_2BUTTON_PRESS ) {
//            g_return_val_if_fail( docgroup_s->focused_view != NULL, TRUE );
            g_return_val_if_fail( docgroup_s->selected_grp != NULL, TRUE );
//            view_set_group( docgroup_s->focused_view, docgroup_s->selected_grp );
        }
        return FALSE; 		/* left-click selection is cool */
    }

#if 0
    if ( event->button.button == 3 ) {
        /* set up the right click menu */
        if ( grp->pUp != NULL ) {
            docmenu_set_sensitive( "<Group>/Duplicate", TRUE );
            docmenu_set_sensitive( "<Group>/Ungroup", TRUE );
            docmenu_set_sensitive( "<Group>/Delete", TRUE );
        } else {
            docmenu_set_sensitive( "<Group>/Duplicate", FALSE );
            docmenu_set_sensitive( "<Group>/Ungroup", FALSE );
            docmenu_set_sensitive( "<Group>/Delete", FALSE );
        }

        docmenu_group_tearoff( docgroup_s->last_doc, docgroup_s->focused_view,
                               grp, &( event->button ) );

        /* make the widget think it got a left click */
        event->button.button = 1; 		/* cheap hack, highly evil */
        return FALSE;
    }
#endif

    /* prevent the widget from getting the signal */
    gtk_signal_emit_stop_by_name( GTK_OBJECT( widget ), "button_press_event" );
    return TRUE;
}



int docgroup_reparent( GNode *src, GNode *dest ) {
    model_reparent_node( src, dest );
    return TRUE;
}

#define VERBOSE 1

void docgroup_tree_install_dnd( GtkWidget *widget, GNode *grp ) {

    static const GtkTargetEntry type_accept_list[] = {
        { "kludge3d/model-group-ptr", 0 }
    };
    
#ifdef VERBOSE
    // c short-circuits
    if( grp->data != NULL && ((Mesh*)grp->data)->name != NULL )
        printf( "tree_install_dnd group=0x%08x (%s)\n", ( guint ) grp, ((Mesh*)grp->data)->name );
    else
        printf( "tree_install_dnd group=0x%08x ( no data / no name )\n", ( guint ) grp );
#endif

    if ( grp->parent != NULL ) {
        gtk_drag_source_set( widget,
                             GDK_BUTTON1_MASK | GDK_BUTTON3_MASK,
                             type_accept_list, 1,
                             GDK_ACTION_COPY | GDK_ACTION_MOVE );
        gtk_signal_connect( GTK_OBJECT( widget ), "drag_data_get",
                            GTK_SIGNAL_FUNC( docgroup_tree_drag_handle ), grp );
    }

    gtk_drag_dest_set( widget,
                       GTK_DEST_DEFAULT_HIGHLIGHT |
                       GTK_DEST_DEFAULT_MOTION |
                       GTK_DEST_DEFAULT_DROP,
                       type_accept_list, 1,
                       GDK_ACTION_COPY | GDK_ACTION_MOVE );

    gtk_signal_connect( GTK_OBJECT( widget ), "drag_data_received",
                        GTK_SIGNAL_FUNC( docgroup_tree_drop_handle ), grp );
}

void docgroup_tree_drag_handle( GtkWidget *widget,
                                GdkDragContext *context,
                                GtkSelectionData *selection_data,
                                guint info, guint time,
                                GNode *grp ) {
#ifdef VERBOSE
    // c short-circuits
    if( grp->data != NULL && ((Mesh*)grp->data)->name != NULL )
        printf( "tree_drag_handle group=0x%08x (%s)\n", ( guint ) grp, ((Mesh*)grp->data)->name );
    else
        printf( "tree_drag_handle group=0x%08x ( no data / no name )\n", ( guint ) grp );
#endif

    /* TODO: group serialization and _real_ drag-n-drop support */

    gtk_selection_data_set( selection_data,
                            gdk_atom_intern ( "kludge3d/model-group-ptr", FALSE ),
                            32, ( guchar * ) &grp, sizeof( grp ) );
}

void docgroup_tree_drop_handle( GtkWidget *widget,
                                GdkDragContext *context,
                                gint x, gint y,
                                GtkSelectionData *selection_data,
                                guint info, guint time,
                                GNode *grp ) {
    GNode *src;
    GNode *dest;

    src = *( GNode ** ) selection_data->data;
    dest = grp;
#ifdef VERBOSE
    printf( "reparent_group src=0x%08x dest=0x%08x\n", ( guint ) src, ( guint ) dest );
//jeez...    printf( "(%s) ==> (%s)\n", ((Mesh*)src->data)->name, ((Mesh*)dest->data)->name );
#endif

    if ( docgroup_reparent( src, dest ) == FALSE )
        gdk_beep();
    else
        docgroup_rebuild_tree(  );
}





//////////////////////////////////////////////////////////////////////////
//
//  DEAD CODE
//
//////////////////////////////////////////////////////////////////////////


#if 0

void docgroup_rebuild_tree( GNode *topgrp ) {
    if ( doc != docgroup_s->last_doc )
        return ;

    if ( docgroup_s->gs_hash != NULL )
        g_hash_table_destroy( docgroup_s->gs_hash );
    docgroup_s->gs_hash = NULL;
    gtk_tree_clear_items( GTK_TREE( docgroup_s->Tree ), 0, 999 );

    if ( topgrp != NULL ) {
        docgroup_s->gs_hash = g_hash_table_new( g_direct_hash, docgroup_pointers_equal );
        docgroup_build_tree( topgrp, docgroup_s->Tree, -1 );
    }

    docgroup_clear_selection();
}

void docgroup_change_view( GtkWidget *widget, struct model_view *view ) {
    docgroup_s->focused_view = view;
    if ( view != NULL ) {
        gtk_option_menu_set_history( GTK_OPTION_MENU( docgroup_s->OptionMenu ),
                                     view->group_menu_index );

        if ( view->pDoc != docgroup_s->last_doc ) {
            docgroup_s->last_doc = view->pDoc;
            docgroup_rebuild_tree( view->pDoc, view->pDoc->top_group );
        }
    } else {
        docgroup_s->last_doc = NULL;
        docgroup_rebuild_tree( NULL, NULL );
    }
}


void docgroup_set_sensitive( gint sensitive ) {
    gtk_widget_set_sensitive( docgroup_s->MainWin, sensitive );
}

void docgroup_show() {
    gtk_widget_show( docgroup_s->MainWin );
}

void docgroup_hide() {
    gtk_widget_hide( docgroup_s->MainWin );
}


void docgroup_rebuild_views() {
    GtkWidget *view_list;
    GtkWidget *menu;
    struct model_doc *doc;
    struct model_view *view, *dfl_view = NULL;
    gint hist_pos = 0;
    gchar buf[ 512 ];

    gtk_option_menu_remove_menu( GTK_OPTION_MENU( docgroup_s->OptionMenu ) );

    view_list = gtk_menu_new();

    doc = first_doc;
    if ( doc == NULL ) {
        menu = gtk_menu_item_new_with_label( "None" );
        gtk_menu_append( GTK_MENU( view_list ), menu );
        gtk_widget_show( menu );

        docgroup_clear_selection();
    } else {
        while ( doc != NULL ) {
            view = doc->view_list;
            while ( view != NULL ) {
                view->group_menu_index = hist_pos;
                if ( hist_pos == 0 )
                    dfl_view = view;
                hist_pos++;

                if ( view->view_index == 1 )
                    strcpy( buf, doc->name );
                else
                    sprintf( buf, "%s.%u", doc->name, view->view_index );

                menu = gtk_menu_item_new_with_label( buf );
                gtk_signal_connect( GTK_OBJECT( menu ), "activate",
                                    GTK_SIGNAL_FUNC( docgroup_change_view ), ( gpointer ) view );
                gtk_menu_append( GTK_MENU( view_list ), menu );
                gtk_widget_show( menu );

                view = view->pNext;
            }
            doc = doc->pNext;
        }
    }

    /* set the option menu */
    gtk_option_menu_set_menu( GTK_OPTION_MENU( docgroup_s->OptionMenu ), view_list );
    if ( ( docgroup_s->focused_view == NULL ) || ( docgroup_s->focused_view->pDoc == NULL ) )
        docgroup_change_view( NULL, dfl_view );
    else
        docgroup_change_view( NULL, docgroup_s->focused_view );
}



#endif






