/* 
 * File:         main.c
 * 
 * Description:  main function for kludge3d, creates main window and 
 * 		 populates it
 * 
 * This source code is part of kludge3d, and is released under the 
 * GNU General Public License.
 * 
 * 
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <gtk/gtk.h>


#include "globals.h"
#include "toolbar.h"
#include "frame.h"
#include "menu.h"
#include "notebook.h"
#include "mesh.h"
#include "model.h"
#include "view.h"

#include "memwatch.h"


#define WIN_WIDTH widget->allocation.width
#define WIN_HEIGHT widget->allocation.height

GtkWidget *window; /* a global */

GtkWidget *label;             // message box at bottom of window
//GdkPixmap *pixmap = NULL;


void quit( GtkWidget *widget, gpointer data ) {

    gtk_exit( 0 );
}

int main( int argc, char **argv ) {
    GtkWidget * vbox, *frame, *menu;
    GtkWidget * hbox;

    click_mode[ 1 ] = CM_POINT_ADD;
    click_mode[ 2 ] = CM_ZOOM;
    click_mode[ 3 ] = CM_PAN;

    gtk_init( &argc, &argv );

//    gdk_rgb_set_verbose( TRUE );

    gdk_rgb_init();
    gtk_widget_set_default_colormap (gdk_rgb_get_cmap());
    gtk_widget_set_default_visual (gdk_rgb_get_visual());
    
    model_root = model_new();
    current_mesh = (Mesh*)model_root->data;
    mesh_set_name( current_mesh, "World" );
    current_node = model_root;

    window = gtk_window_new( GTK_WINDOW_TOPLEVEL );
    gtk_widget_set_name( window, "kludge3d" );
    gtk_widget_set_usize( window, 800, 600 );

    vbox = gtk_vbox_new( FALSE, 0 );
    hbox = gtk_hbox_new( FALSE, 0 );

    gtk_container_add( GTK_CONTAINER( window ), vbox );
    gtk_widget_show( vbox );

    gtk_signal_connect( GTK_OBJECT( window ), "destroy",
                        GTK_SIGNAL_FUNC( quit ), NULL );

    gtk_widget_realize( window );


    gtk_box_pack_start( GTK_BOX( hbox ), create_frames(), TRUE, TRUE, 0 );
    gtk_box_pack_start( GTK_BOX( hbox ), create_notebook(), FALSE, FALSE, 0 );
    gtk_widget_show( hbox );

    menu = get_menu_bar( window );
    gtk_menu_bar_set_shadow_type( GTK_MENU_BAR( menu ), GTK_SHADOW_OUT );
    gtk_box_pack_start( GTK_BOX( vbox ), menu, FALSE, FALSE, 0 );
    gtk_box_pack_start( GTK_BOX( vbox ), create_toolbar(), FALSE, FALSE, 0 );

    gtk_box_pack_start( GTK_BOX( vbox ), hbox, TRUE, TRUE, 0 );

    frame = gtk_frame_new( NULL );
    gtk_frame_set_shadow_type( GTK_FRAME( frame ), GTK_SHADOW_IN );
    label = gtk_label_new( "..." );
    gtk_label_set_justify( GTK_LABEL( label ), GTK_JUSTIFY_LEFT );
    gtk_container_add( GTK_CONTAINER( frame ), label );
    gtk_box_pack_start( GTK_BOX( vbox ), frame, FALSE, FALSE, 0 );
    gtk_widget_show( label );
    gtk_widget_show( frame );

    gtk_widget_show( window );

    gtk_main();

    model_delete();

    return 0;
}
