/* 
 * File:         menu.c
 * 
 * Description:  creates and manages the menu bar at the top of the main window
 * 
 * 
 * This source code is part of kludge3d, and is released under the 
 * GNU General Public License.
 * 
 * 
 */
 


#include "menu.h"
#include "fileops.h"
#include "points.h"

#include "memwatch.h"


void show_about( GtkWidget *widget, gpointer data );
//void toggle_switch ( GtkWidget *widget, gpointer data );

void edit_cut_cb( GtkWidget *widget, gpointer data );
void edit_copy_cb( GtkWidget *widget, gpointer data );
void edit_paste_cb( GtkWidget *widget, gpointer data );

static GtkItemFactoryEntry menu_items[] = {
    {"/_File", NULL, NULL, 0, "<Branch>" },
    {"/File/_New", "<control>N", GTK_SIGNAL_FUNC( show_new_dialog ), 0, NULL},
    {"/File/_Open", "<control>O", GTK_SIGNAL_FUNC( show_load_dialog ), 0, NULL},
    {"/File/_Save", "<control>S", GTK_SIGNAL_FUNC( save_menu_cb ), 0, NULL},
    {"/File/Save As", NULL, GTK_SIGNAL_FUNC( show_save_dialog ), 0, NULL},
    {"/File/sep1", NULL, NULL, 0, "<Separator>"},
    {"/File/_Quit", "<control>Q", gtk_main_quit, 0, NULL},
    {"/_Edit", NULL, NULL, 0, "<Branch>"},
    {"/Edit/Cut", "<control>X", edit_cut_cb, 0, NULL},
    {"/Edit/_Copy", "<control>C", edit_copy_cb, 0, NULL},
    {"/Edit/Paste", "<control>V", edit_paste_cb, 0, NULL},
//    {"/_Preview Mode", NULL, NULL, 0, "<Branch>"},
//    {"/Preview Mode/Wire Frame", NULL, NULL, 0, "<RadioItem>"},
//    {"/Preview Mode/Flat", NULL, NULL, 0, "<RadioItem>"},
//    {"/Preview Mode/Smooth", NULL, NULL, 0, "<RadioItem>"},
//    {"/_Texturing" , NULL, NULL, 0, "<Branch>"},
//    {"/_Texturing/Texture Manager...", "<control>T", GTK_SIGNAL_FUNC( show_tex_man_dialog ), 0, NULL},
    {"/_Help", NULL, NULL, 0, "<LastBranch>"},
    {"/_Help/About", "<control>A", show_about, 0, NULL},
};

void edit_cut_cb( GtkWidget *widget, gpointer data ) {
    points_cut();
    draw_all_frames();
}
void edit_copy_cb( GtkWidget *widget, gpointer data ) {
    points_copy();
    draw_all_frames();
}
void edit_paste_cb( GtkWidget *widget, gpointer data ) {
    points_paste();
    draw_all_frames();
}


//void toggle_switch ( GtkWidget *widget, gpointer data ) {
//    grid_snap = grid_snap ? 0 : 1;
//}

void ab_click_button( GtkWidget *widget, gpointer data ) {
    gtk_widget_destroy( GTK_WIDGET( data ) );
}


void show_about( GtkWidget *widget, gpointer data ) {
    GtkWidget * label, *button, *frame, *dialog;

    dialog = gtk_dialog_new();
    gtk_widget_set_name( dialog, "About" );
    gtk_container_set_border_width( GTK_CONTAINER( GTK_DIALOG( dialog )->vbox ), 5 );

    label = gtk_label_new( " This is a simple 3D modeller based on\n"
                           " several other GPL'ed programs, including:\n"
                           "   - G3D by Blake Hegerle \n"
                           "        http://condor.stcloudstate.edu/~hebl9901/g3d \n"
                           "        laertes@stab.gemma.com\n"
                           "   - ME3D by Sam Revitch \n"
                           "        http://www.cs.washington.edu/homes/samr7/me3d \n"
                           "        samr7@cs.washington.edu\n"
                           " The frankenstein-esque result was produced by \n"
                           " Andrew Sampson (ads4260@rit.edu)" );
    gtk_label_set_justify( GTK_LABEL( label ), GTK_JUSTIFY_LEFT );
    gtk_widget_show( label );

    frame = gtk_frame_new( "Creds:" );
    gtk_container_set_border_width( GTK_CONTAINER( frame ), 3 );
    gtk_frame_set_shadow_type( GTK_FRAME( frame ), GTK_SHADOW_OUT );
    gtk_container_add( GTK_CONTAINER( frame ), label );
    gtk_widget_show( frame );

    gtk_box_pack_start( GTK_BOX( GTK_DIALOG( dialog ) ->vbox ),
                        frame, FALSE, FALSE, 5 );

    button = gtk_button_new_with_label( "OK" );
    gtk_widget_show( button );
    gtk_signal_connect( GTK_OBJECT( button ), "clicked",
                        GTK_SIGNAL_FUNC( ab_click_button ), dialog );
    gtk_box_pack_start( GTK_BOX( GTK_DIALOG( dialog ) ->action_area ),
                        button, FALSE, FALSE, 0 );

    gtk_widget_show( dialog );
}

GtkWidget *get_menu_bar( GtkWidget *window ) {
    GtkWidget * menubar;
    int nmenu_items = sizeof( menu_items ) / sizeof( menu_items[ 0 ] );
    GtkItemFactory *item_factory;
    GtkAccelGroup *accel_group;

    accel_group = gtk_accel_group_new();

    item_factory = gtk_item_factory_new( GTK_TYPE_MENU_BAR, "<main>",
                                         accel_group );
    gtk_item_factory_create_items( item_factory, nmenu_items, menu_items, NULL );
    gtk_accel_group_attach ( accel_group, GTK_OBJECT ( window ) );

    menubar = gtk_item_factory_get_widget( item_factory, "<main>" );
    gtk_widget_show( menubar );

//    gtk_widget_set_sensitive( gtk_item_factory_get_widget( item_factory,
//                              "/Preview Mode/Wire Frame" ), FALSE );
//    gtk_widget_set_sensitive( gtk_item_factory_get_widget( item_factory,
//                              "/Preview Mode/Flat" ), FALSE );
//    gtk_widget_set_sensitive( gtk_item_factory_get_widget( item_factory,
//                              "/Preview Mode/Smooth" ), FALSE );

    return menubar;
}
