/* 
 * File:         mesh.c
 * 
 * Description:  funcs for mesh manipulation
 * 
 * 
 * This source code is part of kludge3d, and is released under the 
 * GNU General Public License.
 * 
 * 
 */
 




#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "mesh.h"
#include "points.h"
#include "tri.h"

#include "memwatch.h"


// GLOBAL ////////////////

Mesh *current_mesh = NULL;



Mesh *mesh_new( void ) {
    Mesh * m;
    m = ( Mesh * ) malloc( sizeof( Mesh ) );
    
    m->trans = NULL;
    m->location = NULL;
    m->points = NULL;
    m->triangles = NULL;
    m->texture = NULL;
    m->name = NULL;
    
    m->texrep_x = 1.0;
    m->texrep_y = 1.0;

    m->list = 0;
    m->dirty = 1;

    mesh_set_name( m, "NewMesh" );

    return m;
}


void mesh_delete( Mesh * m ) {

    if( m == NULL ) return;
    
    if( m->trans != NULL ) {
        free( m->trans );
        m->trans = NULL;
    }
    if( m->location != NULL ) {
        free( m->location );
        m->location = NULL;
    }

    if( m->triangles != NULL ) {
        GList * l = m->triangles;
    
        for( ; l != NULL; l = m->triangles ) {
            tris_del_v( m, (Tri*)l->data );
        }

        g_list_free( m->triangles );
        m->triangles = NULL;
    }
    if( m->points != NULL ) {
        GList * l = m->points;
    
        for( ; l != NULL; l = m->points ) {
            point_del_v( m, (Point*)l->data );
        }

        g_list_free( m->points );
        m->points = NULL;
    }

    if( m->name != NULL ) {
        free( m->name );
        m->name = NULL;
    }

    m->texture = NULL;
    m->list = 0;
    m->dirty = 1;


    free( m );
}


void mesh_set_transform( Mesh *m, Trans *new_trans ) {
    if( m != NULL && new_trans != NULL )
        m->trans = new_trans;
}

void mesh_set_location( Mesh *m, Point* loc ) {
    if( m != NULL && loc != NULL )
        m->location = loc;
}


void mesh_set_texture( Mesh *m, tex *nt ) {

    // a mesh's texture *can* be set to NULL...

    if( m != NULL )
        m->texture = nt;
}


void mesh_set_texrep( Mesh *m, float x, float y ) {
    // i guess they should actually be called "u" and "v",
    // or "s" and "t"... whatever.
    
    if( m != NULL ) {
        m->texrep_x = x;
        m->texrep_y = y;
    }
}


void mesh_set_name( Mesh *m, char* name ) {
    
    if( m != NULL && name != NULL ) {
        if( m->name != NULL )
            free( m->name );
        m->name = strdup( name );
    }
    
}





