/* 
 * File:         mesh.h
 * 
 * Description:  header for mesh.c, defines Mesh struct
 * 
 * 
 * This source code is part of kludge3d, and is released under the 
 * GNU General Public License.
 * 
 * 
 */
 

#ifndef MESH_H
#define MESH_H

#include <glib.h>

#include "geo.h"
#include "transform.h"
#include "texture.h"

typedef struct _Mesh Mesh;

struct _Mesh {
    Trans *trans;     /* rotation only, translation is handled by next elem */
    Point *location;
    GList *points;
    GList *triangles;
    tex *texture;
    float texrep_x, texrep_y;
//    GList *lines;
    GLuint list, dirty;
    char* name;
};

Mesh *mesh_new( void );
void mesh_delete( Mesh * m );

void mesh_set_transform( Mesh *m, Trans *newt );
void mesh_set_location( Mesh *m, Point *loc );
void mesh_set_texture( Mesh *m, tex *nt );
void mesh_set_texrep( Mesh *m, float x, float y );
void mesh_set_name( Mesh *m, char* name );


#endif
