/* 
 * File:         model.c
 * 
 * Description:  handles model-related stuff
 * 
 * This source code is part of kludge3d, and is released under the 
 * GNU General Public License.
 * 
 * 
 */
 
/*
 * Portions of this file were ripped from:
 *    - the original model.c by BEH (very little is left)
 * 
 */



#include <stdlib.h>
#include <string.h>
#include <stdio.h>
#include <math.h>

#include "transform.h"
#include "globals.h"
#include "mesh.h"
#include "model.h"
#include "group_browser.h"
#include "tri.h"
#include "points.h"
#include "texture.h"
#include "tex_man.h"

#include "memwatch.h"


// GLOBALS ////////////////

GNode *model_root = NULL;
GNode *current_node = NULL;
GList *model_textures = NULL;



// MODEL STUFF /////////////////////////////////////////////////////////////


GNode *model_new( void ) {
    GNode * node;
    Mesh * m = mesh_new();

    node = g_node_new( m );
    
    return node;
}

void model_delete( void ) {

    // should clipboard_points be cleared?
    

    current_mesh = NULL;
    current_node = NULL;
    
    point_unselect_all();
    tris_unselect_all();
    
    model_remove_all_textures();
    model_textures = NULL; // just in case
    
    model_delete_node( model_root );
    model_root = NULL;


    docgroup_rebuild_tree();
    texman_rebuild_list();

}


gboolean model_delete_tree( GNode *node, gpointer data ) {
    // recurse through tree
    
    Mesh* m = (Mesh*)node->data;

    mesh_delete( m );
    node->data = NULL;
    
    return FALSE;  /* we want it to continue... */
}


void model_delete_node( GNode *node ) {

    if( node != NULL ) {
        g_node_traverse( node, G_POST_ORDER, G_TRAVERSE_ALL, -1,
                         model_delete_tree, (gpointer)NULL );
        g_node_destroy( node );
    }


}


void model_reparent_node( GNode *node, GNode *new_parent ) {

    g_return_if_fail( node != NULL );
    g_return_if_fail( new_parent != NULL );
    g_return_if_fail( node->parent != NULL );
    g_return_if_fail( node != new_parent );
    
    g_node_unlink( node );
    g_node_append( new_parent, node );

}


gboolean model_remove_texture_tree( GNode *node, gpointer data ) {
    // recurse through tree
    
    Mesh* m = (Mesh*)node->data;

    if( m && ( m->texture == (tex*)data  ||  data == NULL ) )
        m->texture = NULL;
    
    return FALSE;  /* we want it to continue... */
}

void model_remove_all_textures(  ) {

    GList * tl;
    
    if( model_root != NULL ) {
        g_node_traverse( model_root, G_POST_ORDER, G_TRAVERSE_ALL, -1,
                         model_remove_texture_tree, (gpointer)NULL );
    }

    for( tl = model_textures; tl != NULL; tl = tl->next ) {
        tex_delete( (tex*) tl->data );
    }
    g_list_free( model_textures );
    
}

void model_add_texture( tex *nt ) {
    
    if( nt != NULL )
        model_textures = g_list_append( model_textures, nt );
}


void model_remove_texture( tex *tt ) {

    if( tt == NULL ) return;

    if( model_root != NULL ) {
        g_node_traverse( model_root, G_POST_ORDER, G_TRAVERSE_ALL, -1,
                         model_remove_texture_tree, (gpointer)tt );
    }

    model_textures = g_list_remove_link( model_textures,
                                         g_list_find( model_textures,
                                         ( gpointer ) tt ) );
    tex_delete( tt );
}


tex * model_get_texture_by_filename( char* filename ) {
    
    GList * tl = NULL;
    
    if( filename == NULL ) return NULL;
    
    for( tl = model_textures; tl != NULL; tl = tl->next ) {
        tex * t = (tex*) tl->data;
        if( strcmp( t->filename, filename ) == 0 )
            return t;
    }
    
    
    return NULL;
}





