/* 
 * File:         notebook.c
 * 
 * Description:  creates the tabbed notebook containing options, etc
 * 
 * 
 * This source code is part of kludge3d, and is released under the 
 * GNU General Public License.
 * 
 * 
 */
 



#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>


#include "notebook.h"
#include "view.h"
#include "frame.h"
#include "tex_man.h"
#include "group_browser.h"


#include "memwatch.h"

GtkWidget * nb_grid_spacing_label;

void toggle_button_callback( GtkWidget *widget, gpointer data ) {

    if( GTK_TOGGLE_BUTTON (widget)->active ) {
        /* If control reaches here, the toggle button is down */

        *((int *)data) = TRUE;
    } else {
        /* If control reaches here, the toggle button is up */

        *((int *)data) = FALSE;
    }
    draw_all_frames();
}

void change_grid_spacing_callback( GtkWidget *widget, int up_down ) {

    char new_label_string[20];
    
    if( up_down == 0 ) {
        // down
        view_config.grid_spacing[0] *= 0.10;
        view_config.grid_spacing[1] *= 0.10;
        view_config.grid_spacing[2] *= 0.10;

    } else if( up_down == 1 ) {
        // up
        view_config.grid_spacing[0] *= 10.0;
        view_config.grid_spacing[1] *= 10.0;
        view_config.grid_spacing[2] *= 10.0;
    }
    
    if( view_config.grid_spacing[0] < 0.01 )
        view_config.grid_spacing[0] = 
        view_config.grid_spacing[1] = 
        view_config.grid_spacing[2] = 0.01;
    if( view_config.grid_spacing[0] > 100.0 )
        view_config.grid_spacing[0] = 
        view_config.grid_spacing[1] = 
        view_config.grid_spacing[2] = 100.0;

    snprintf( new_label_string, 20, "%.3f", view_config.grid_spacing[0] );
    gtk_label_set_text( GTK_LABEL(nb_grid_spacing_label), new_label_string );
    
    draw_all_frames();
}


void rendermode_wire_cb( GtkWidget *widget, gpointer data ) {
    view_set_rendermode( MP_SHOW_WIREFRAME );
    draw_all_frames();
}
void rendermode_flat_cb( GtkWidget *widget, gpointer data ) {
    view_set_rendermode( MP_SHOW_FLAT );
    draw_all_frames();
}
void rendermode_textured_cb( GtkWidget *widget, gpointer data ) {
    view_set_rendermode( MP_SHOW_TEXTURED );
    draw_all_frames();
}


void triangleorder_ccw_cb( GtkWidget *widget, gpointer data ) {
    view_set_triangleorder( MP_TRI_ORDER_CCW );
    draw_all_frames();
}
void triangleorder_cw_cb( GtkWidget *widget, gpointer data ) {
    view_set_triangleorder( MP_TRI_ORDER_CW );
    draw_all_frames();
}
void triangleorder_all_cb( GtkWidget *widget, gpointer data ) {
    view_set_triangleorder( MP_TRI_ORDER_ALL );
    draw_all_frames();
}

#if 0
void vertexsize_off_cb( GtkWidget *widget, gpointer data ) {
    view_set_vertexsize( MP_VERT_SIZE_OFF );
    draw_all_frames();
}
void vertexsize_normal_cb( GtkWidget *widget, gpointer data ) {
    view_set_vertexsize( MP_VERT_SIZE_NORMAL );
    draw_all_frames();
}
void vertexsize_enlarged_cb( GtkWidget *widget, gpointer data ) {
    view_set_vertexsize( MP_VERT_SIZE_BIG );
    draw_all_frames();
}
#endif


void nb_vertexsize_adj_cb( GtkWidget *widget, gpointer data ) {

    if( widget != NULL )
        view_set_vertexsize( GTK_ADJUSTMENT( widget )->value );
    draw_all_frames();
}

void nb_fov_adj_cb( GtkWidget *widget, gpointer data ) {

    if( widget != NULL )
        view_set_fov( GTK_ADJUSTMENT( widget )->value );
    draw_all_frames();
}

GtkWidget *create_options( void ) {

    GtkWidget * options;
    GtkWidget * button;
    GtkWidget * hbox;
    GtkWidget * vbox;
//    GtkWidget * spinb;
//    GtkAdjustment *adj;
    GtkWidget * frame;
    GtkWidget * gsframe;
//    GtkWidget * hsep;
    GtkAdjustment * hadjustment;
    GtkWidget * hscale;

    options = gtk_vbox_new( FALSE, 5 );
    gtk_container_set_border_width( GTK_CONTAINER(options), 5 );
    
    // GRID STUFF ////////////////////////////////////////////////
    frame = gtk_frame_new( "Grid options" );
    vbox = gtk_vbox_new( FALSE, 0 );

    button = gtk_check_button_new_with_label( "show grid" );
    gtk_signal_connect( GTK_OBJECT( button ), "clicked",
                        GTK_SIGNAL_FUNC( toggle_button_callback ), (gpointer)(&(view_config.grid_visible)) );
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( button ), TRUE );
    gtk_box_pack_start( GTK_BOX( vbox ), button, FALSE, FALSE, 0 );
    gtk_widget_show( button );

    button = gtk_check_button_new_with_label( "snap to grid" );
    gtk_signal_connect (GTK_OBJECT( button ), "clicked",
                        GTK_SIGNAL_FUNC( toggle_button_callback ), (gpointer)(&grid_snap) );
    gtk_box_pack_start( GTK_BOX( vbox ), button, FALSE, FALSE, 0 );
    gtk_widget_show( button );

    // grid spacing ///////////////////////////
    gsframe = gtk_frame_new( "grid spacing" );
    gtk_frame_set_shadow_type( GTK_FRAME( gsframe ), GTK_SHADOW_NONE );
    hbox = gtk_hbox_new( FALSE, 0 );

    button = gtk_button_new_with_label( "-" );
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC( change_grid_spacing_callback ), 0 );
    gtk_widget_show( button );
    gtk_box_pack_start( GTK_BOX( hbox ), button, TRUE, FALSE, 0 );

    nb_grid_spacing_label = gtk_label_new( "1" );
    gtk_widget_show( nb_grid_spacing_label );
    gtk_box_pack_start( GTK_BOX( hbox ), nb_grid_spacing_label, TRUE, FALSE, 0 );
    
    button = gtk_button_new_with_label( "+" );
    gtk_signal_connect (GTK_OBJECT (button), "clicked",
                        GTK_SIGNAL_FUNC( change_grid_spacing_callback ), 1 );
    gtk_widget_show( button );
    gtk_box_pack_start( GTK_BOX( hbox ), button, TRUE, FALSE, 0 );

    gtk_widget_show( hbox );
    gtk_container_add( GTK_CONTAINER( gsframe ), hbox );
    gtk_box_pack_start( GTK_BOX( vbox ), gsframe, FALSE, FALSE, 0 );
    gtk_widget_show( gsframe );
    
    gtk_widget_show( vbox );
    gtk_container_add( GTK_CONTAINER( frame ), vbox );
    gtk_box_pack_start( GTK_BOX( options ), frame, FALSE, FALSE, 0 );
    gtk_widget_show( frame );

//    hsep = gtk_hseparator_new();
//    gtk_box_pack_start( GTK_BOX( options ), hsep, FALSE, FALSE, 0 );
//    gtk_widget_show( hsep );

    // 3D PREVIEW STUFF //////////////////////////////////////////

    // RENDERING MODE RADIO BUTTONS
    frame = gtk_frame_new( "Rendering mode" );
    vbox = gtk_vbox_new( FALSE, 0 );

    button = gtk_radio_button_new_with_label( NULL, "wireframe" );
    gtk_signal_connect (GTK_OBJECT( button ), "clicked",
                        GTK_SIGNAL_FUNC( rendermode_wire_cb ), NULL );
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( button ), TRUE );
    gtk_box_pack_start( GTK_BOX( vbox ), button, FALSE, FALSE, 0 );
    gtk_widget_show( button );

    button = gtk_radio_button_new_with_label( 
                 gtk_radio_button_group( GTK_RADIO_BUTTON( button ) ), 
                 "flat shaded" );
    gtk_signal_connect (GTK_OBJECT( button ), "clicked",
                        GTK_SIGNAL_FUNC( rendermode_flat_cb ), NULL );
    gtk_box_pack_start( GTK_BOX( vbox ), button, FALSE, FALSE, 0 );
    gtk_widget_show( button );

    button = gtk_radio_button_new_with_label( 
                 gtk_radio_button_group( GTK_RADIO_BUTTON( button ) ), 
                 "textured" );
    gtk_signal_connect (GTK_OBJECT( button ), "clicked",
                        GTK_SIGNAL_FUNC( rendermode_textured_cb ), NULL );
    gtk_box_pack_start( GTK_BOX( vbox ), button, FALSE, FALSE, 0 );
    gtk_widget_show( button );

    gtk_widget_show( vbox );
    gtk_container_add( GTK_CONTAINER( frame ), vbox );
    gtk_box_pack_start( GTK_BOX( options ), frame, FALSE, FALSE, 0 );
    gtk_widget_show( frame );


    // TRIANGLE CULLING RADIO BUTTONS
    frame = gtk_frame_new( "Triangle direction" );
    vbox = gtk_vbox_new( FALSE, 0 );

    button = gtk_radio_button_new_with_label( NULL, "ccw" );
    gtk_signal_connect (GTK_OBJECT( button ), "clicked",
                        GTK_SIGNAL_FUNC( triangleorder_ccw_cb ), NULL );
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( button ), TRUE );
    gtk_box_pack_start( GTK_BOX( vbox ), button, FALSE, FALSE, 0 );
    gtk_widget_show( button );

    button = gtk_radio_button_new_with_label( 
                 gtk_radio_button_group( GTK_RADIO_BUTTON( button ) ), 
                 "cw" );
    gtk_signal_connect (GTK_OBJECT( button ), "clicked",
                        GTK_SIGNAL_FUNC( triangleorder_cw_cb ), NULL );
    gtk_box_pack_start( GTK_BOX( vbox ), button, FALSE, FALSE, 0 );
    gtk_widget_show( button );

    button = gtk_radio_button_new_with_label( 
                 gtk_radio_button_group( GTK_RADIO_BUTTON( button ) ), 
                 "all" );
    gtk_signal_connect (GTK_OBJECT( button ), "clicked",
                        GTK_SIGNAL_FUNC( triangleorder_all_cb ), NULL );
    gtk_box_pack_start( GTK_BOX( vbox ), button, FALSE, FALSE, 0 );
    gtk_widget_show( button );

    gtk_widget_show( vbox );
    gtk_container_add( GTK_CONTAINER( frame ), vbox );
    gtk_box_pack_start( GTK_BOX( options ), frame, FALSE, FALSE, 0 );
    gtk_widget_show( frame );



    // VERTEX SIZE STUFF
    frame = gtk_frame_new( "Vertex size" );
    vbox = gtk_vbox_new( FALSE, 0 );

#if 0
    button = gtk_radio_button_new_with_label( NULL, "off" );
    gtk_signal_connect (GTK_OBJECT( button ), "clicked",
                        GTK_SIGNAL_FUNC( vertexsize_off_cb ), NULL );
    gtk_box_pack_start( GTK_BOX( vbox ), button, FALSE, FALSE, 0 );
    gtk_widget_show( button );

    button = gtk_radio_button_new_with_label( 
                 gtk_radio_button_group( GTK_RADIO_BUTTON( button ) ), 
                 "normal" );
    gtk_signal_connect (GTK_OBJECT( button ), "clicked",
                        GTK_SIGNAL_FUNC( vertexsize_normal_cb ), NULL );
    gtk_toggle_button_set_active( GTK_TOGGLE_BUTTON( button ), TRUE );
    gtk_box_pack_start( GTK_BOX( vbox ), button, FALSE, FALSE, 0 );
    gtk_widget_show( button );

    button = gtk_radio_button_new_with_label( 
                 gtk_radio_button_group( GTK_RADIO_BUTTON( button ) ), 
                 "enlarged" );
    gtk_signal_connect (GTK_OBJECT( button ), "clicked",
                        GTK_SIGNAL_FUNC( vertexsize_enlarged_cb ), NULL );
    gtk_box_pack_start( GTK_BOX( vbox ), button, FALSE, FALSE, 0 );
    gtk_widget_show( button );
#endif

    hadjustment = GTK_ADJUSTMENT( gtk_adjustment_new( view_config.vertex_rad, 
                                                      0.01, 1, 0.01, 0.01, 0.01 ) );
    gtk_signal_connect ( GTK_OBJECT ( hadjustment ), "value_changed",
                         GTK_SIGNAL_FUNC ( nb_vertexsize_adj_cb ), NULL );
    hscale = gtk_hscale_new( hadjustment );
    gtk_range_set_update_policy( GTK_RANGE( hscale ), GTK_UPDATE_DISCONTINUOUS );
    gtk_scale_set_digits( GTK_SCALE( hscale ), 2 );  // gtk developers: why does *this* affect the granularity of the scale widget?  are you all on crack?
    gtk_widget_show( hscale );
    gtk_box_pack_start( GTK_BOX( vbox ), hscale, FALSE, FALSE, 0 );

    gtk_widget_show( vbox );
    gtk_container_add( GTK_CONTAINER( frame ), vbox );
    gtk_box_pack_start( GTK_BOX( options ), frame, FALSE, FALSE, 0 );
    gtk_widget_show( frame );



    // FOV ADJUSTMENT
    frame = gtk_frame_new( "3d FOV" );
    vbox = gtk_vbox_new( FALSE, 0 );

    hadjustment = GTK_ADJUSTMENT( gtk_adjustment_new( view_config.fov, 
                                                      10, 140, 0.01, 0.01, 0.01 ) );
    gtk_signal_connect ( GTK_OBJECT ( hadjustment ), "value_changed",
                         GTK_SIGNAL_FUNC ( nb_fov_adj_cb ), NULL );
    hscale = gtk_hscale_new( hadjustment );
    gtk_range_set_update_policy( GTK_RANGE( hscale ), GTK_UPDATE_DISCONTINUOUS );
    gtk_scale_set_digits( GTK_SCALE( hscale ), 2 );  // gtk developers: why does *this* affect the granularity of the scale widget?  are you all on crack?
    gtk_widget_show( hscale );
    gtk_box_pack_start( GTK_BOX( vbox ), hscale, FALSE, FALSE, 0 );

    gtk_widget_show( vbox );
    gtk_container_add( GTK_CONTAINER( frame ), vbox );
    gtk_box_pack_start( GTK_BOX( options ), frame, FALSE, FALSE, 0 );
    gtk_widget_show( frame );



    gtk_widget_show( options );
    
    return options;
}





GtkWidget *create_notebook( void ) {
    GtkWidget * notebook;
    GtkWidget * options;
    GtkWidget * textures;
    GtkWidget * groups;
    
    GtkWidget * label;
    
    
    notebook = gtk_notebook_new(  );

    groups = create_group_browser();
    textures = create_tex_manager();
    options = create_options();
    
    
    label = gtk_label_new( "Groups" );
    gtk_notebook_append_page( GTK_NOTEBOOK( notebook ), groups, label );
    gtk_widget_show( label );

    label = gtk_label_new( "Textures" );
    gtk_notebook_append_page( GTK_NOTEBOOK( notebook ), textures, label );
    gtk_widget_show( label );

    label = gtk_label_new( "Options" );
    gtk_notebook_append_page( GTK_NOTEBOOK( notebook ), options, label );
    gtk_widget_show( label );


    gtk_widget_show( notebook );

    return notebook;
}


