/* 
 * File:         points.c
 * 
 * Description:  funcs for creating, adding, and moving vertices.
 * 		 Used to also do drawing, but not any more!
 * 
 * This source code is part of kludge3d, and is released under the 
 * GNU General Public License.
 * 
 * 
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <gtk/gtk.h>

#include "points.h"
#include "tri.h"
#include "frame.h"

#include "memwatch.h"


int xyz_state = -1;
GList *selected_points = NULL;
GList *clipboard_points = NULL;



Point* point_new( void ) {
    Point* p = (Point*)malloc( sizeof(Point) );
    
    p->v[0] = p->v[1] = p->v[2] = 0.0f;
    
    p->order = 0;
    p->selected = FALSE;
    
    return p;
}




void point_add_v( Mesh* m, Point *np ) {

    if( m != NULL && np != NULL )
        m->points = g_list_append( m->points, ( gpointer ) np );
}




void point_del_v( Mesh* m, Point *np ) {

    GList * cl;

    if( m == NULL || m->points == NULL ) 
        return;
    if( np == NULL ) 
        return;
    
    cl = g_list_find( m->points, ( gpointer ) np );
    if( cl != NULL ) {
        m->points = g_list_remove( m->points, cl->data );
    }
    free( np );
    

}



void point_select_v( Point* p ) {

    if( p != NULL && !g_list_find( selected_points, p ) ) {
            selected_points = g_list_append( selected_points, p );
            p->selected = TRUE;
        
    } else {
        point_unselect_all();
        // unselect everything if user clicked in an empty area
    }
}

/*
void point_move_selected_snap( float deltaX, float deltaY, float deltaZ, 
                               float spacing ) {

    GList *cl;
    Point * p;
    float x, y, z, mult;
    
    if( grid_snap && selected_points ) {
        // vertex snap stuff; this probably shouldn't be here...
        cl = selected_points;
        p = ( Point * ) cl->data;
        
        x = p->v[0] + deltaX;
        y = p->v[1] + deltaY;
        z = p->v[2] + deltaZ;
        
        mult = x / spacing;
        x = spacing * rint( mult );
        mult = y / spacing;
        y = spacing * rint( mult );
        mult = z / spacing;
        z = spacing * rint( mult );
        
        deltaX = x - p->v[0];
        deltaY = y - p->v[1];
        deltaZ = z - p->v[2];
    }
    
    
    
    for ( cl = selected_points; cl != NULL; cl = cl->next ) {
        // go through all the selected vetices and move them

        p = ( Point * ) cl->data;

        p->v[0] += deltaX;
        p->v[1] += deltaY;
        p->v[2] += deltaZ;
    }


}
*/

void point_move_selected( float deltaX, float deltaY, float deltaZ ) {

    GList *cl;
    Point * p;
    
    for ( cl = selected_points; cl != NULL; cl = cl->next ) {
        // go through all the selected vetices and move them

        p = ( Point * ) cl->data;

        p->v[0] += deltaX;
        p->v[1] += deltaY;
        p->v[2] += deltaZ;
    }


}


void point_unselect_v( Point *p ) {

    if( p != NULL ) {
        p->selected = FALSE;
        selected_points = g_list_remove( selected_points, p );
    }
}

void point_unselect_all( void ) {

    GList * l = selected_points;
    
    for( ; l != NULL; l = l->next ) {
        ((Point*)l->data)->selected = FALSE;
    }
    
    g_list_free( selected_points );
    selected_points = NULL;
}


void points_copy( void ) {
    
    GList* pl = selected_points;
    Point * p = NULL;
    
    g_list_free( clipboard_points );
    clipboard_points = NULL;
//FIXME - leak
    
    
    while( pl != NULL ) {
        Point* source = (Point*)pl->data;
        p = point_new();
        p->v[0] = source->v[0];
        p->v[1] = source->v[1];
        p->v[2] = source->v[2];
        p->order = source->order;  // what is "order" for?
        
        clipboard_points = g_list_append( clipboard_points, (gpointer)p );
        pl = pl->next;
    }
    
}

void points_cut( void ) {

    GList *spl = selected_points;
    Point *p;

    points_copy();

    while( spl != NULL ) {
        p = (Point*)(spl->data);
        
        point_del_v( current_mesh, p );
        /* delete any lines or triangles that use this point */
//        line_del_point( p );
        tris_del_point( p );

        spl = spl->next;
    }
    
    point_unselect_all();
}

void points_paste( void ) {

    GList *pl = clipboard_points;
    Point * p = NULL;
    
    if( current_mesh == NULL )
        return;

    point_unselect_all();
    
    while( pl != NULL ) {
        Point* source = (Point*)pl->data;
        p = point_new();
        p->v[0] = source->v[0];
        p->v[1] = source->v[1];
        p->v[2] = source->v[2];
        p->order = source->order;  // what is "order" for?
        
        point_add_v( current_mesh, p );
        point_select_v( p );
        
        pl = pl->next;
    }

}











///////////////////////////////////////////////////////
//
// dead code...
//
///////////////////////////////////////////////////////



