/* 
 * File:         tex_app.c
 * 
 * Description:  dialog for editing the u-v coords for a triangle
 * 
 * 
 * This source code is part of kludge3d, and is released under the 
 * GNU General Public License.
 * 
 * 
 */
 


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "tex_app.h"
#include "geo.h"
#include "frame.h"
#include "texture.h"
#include "tri.h"
#include "globals.h"

#include "memwatch.h"




struct _tex_app {
    GtkWidget *dialog, *vbox, *hbox, *frame;
    GtkWidget *button, *drawing_area;
    GtkWidget *tex_repeat_entry_x;
    GtkWidget *tex_repeat_entry_y;
}
tex_app_dialog;


int current_tv = -1;  /* index of the current texture-vertex, ranges from 0-2 */



//////////////////////////////////////////////////////////////////////

void tv_to_widget( float tv_x, float tv_y, int* w_x, int* w_y, int w_width, int w_height ) {

    *w_x = (int)( fabs(tv_x) * (float)w_width );
    *w_y = (int)( fabs(tv_y) * (float)w_height );
    
    // opengl coords -> X11 coords
    *w_y = w_height - *w_y;
}
void widget_to_tv( int w_x, int w_y, float* tv_x, float* tv_y, int w_width, int w_height ) {

    // X11 coords -> opengl coords
    w_y = w_height - w_y;
    
    *tv_x = (float)w_x / (float)w_width;
    *tv_y = (float)w_y / (float)w_height;
}


void texapp_draw_triangle_overlay( GtkWidget * widget, Tri * t ) {
    GdkGC * gc;
    GdkColor color;
    GdkColormap *colormap;
    int w_x0, w_y0;
    int w_x1, w_y1;
    int w_x2, w_y2;
    
    gc = gdk_gc_new( widget->window );
    colormap = gdk_window_get_colormap( widget->window );
    gdk_color_white( colormap, &color );
    gdk_gc_set_foreground( gc, &color );
    gdk_color_black( colormap, &color );
    gdk_gc_set_background( gc, &color );
    gdk_gc_set_line_attributes( gc, 3, GDK_LINE_DOUBLE_DASH, GDK_CAP_PROJECTING, GDK_JOIN_MITER );

/*
printf( "tv's : %f %f - %f %f - %f %f\n",
t->tv[0].x, t->tv[0].y,
t->tv[1].x, t->tv[1].y,
t->tv[2].x, t->tv[2].y );
*/

    tv_to_widget( t->tv[0].x, t->tv[0].y, &w_x0, &w_y0, widget->allocation.width, widget->allocation.height );
    tv_to_widget( t->tv[1].x, t->tv[1].y, &w_x1, &w_y1, widget->allocation.width, widget->allocation.height );
    tv_to_widget( t->tv[2].x, t->tv[2].y, &w_x2, &w_y2, widget->allocation.width, widget->allocation.height );

    gdk_draw_line( widget->window, gc, w_x0, w_y0, w_x1, w_y1 );
    gdk_draw_line( widget->window, gc, w_x1, w_y1, w_x2, w_y2 );
    gdk_draw_line( widget->window, gc, w_x0, w_y0, w_x2, w_y2 );
        
        
    gdk_gc_unref( gc );
}



void texapp_draw_texture( GtkWidget * widget ) {
    
    tex* ctex = NULL;
//    int tex_psx;
//    int tex_psy;
    char tmp[25];
    
//FIXME - let it be known that this can only handle fixed-size textures
// (which is ok, for now)

    if( current_mesh == NULL ) return;
    
    ctex = current_mesh->texture;
    
    if ( ctex ) {
//        tex_psx = ( widget->allocation.width - ctex->width ) >> 1;
//        tex_psy = ( widget->allocation.height - ctex->height ) >> 1;
        gdk_draw_rgb_image( widget->window,
                            widget->style->fg_gc[ GTK_WIDGET_STATE ( widget ) ],
//                            tex_psx, tex_psy,
                            0, 0,
                            ctex->thumbnail_width,
                            ctex->thumbnail_height,
                            GDK_RGB_DITHER_NORMAL, ctex->thumbnail_data,
                            ctex->thumbnail_width * ctex->depth );
    } else {
        gdk_draw_rectangle( widget->window,
                            widget->style->fg_gc[ GTK_WIDGET_STATE ( widget ) ],
                            TRUE, 0, 0,
                            widget->allocation.width,
                            widget->allocation.height );
    }
    
    if( selected_tri != NULL ) {
        texapp_draw_triangle_overlay( widget, selected_tri );
    }
    
    snprintf( tmp, 24, "%f", current_mesh->texrep_x );
    gtk_entry_set_text( GTK_ENTRY( tex_app_dialog.tex_repeat_entry_x ), tmp );
    snprintf( tmp, 24, "%f", current_mesh->texrep_y );
    gtk_entry_set_text( GTK_ENTRY( tex_app_dialog.tex_repeat_entry_y ), tmp );

}


gint texapp_expose_event( GtkWidget *widget, GdkEventExpose *event, gpointer data ) {

    texapp_draw_texture( widget );
    
    return 0;
}

gint texapp_configure_event( GtkWidget *widget, GdkEvent *event, gpointer data ) {

    if( current_mesh != NULL && current_mesh->texture != NULL ) {

        gtk_widget_draw( tex_app_dialog.drawing_area, NULL );
        gdk_flush();
    }

    return TRUE;
}


//////////////////////////////////////////////////////////////////////


void texapp_change_repeat_cb( GtkWidget *widget, gpointer data ) {

    float x, y;
    char * tmp;
    
    tmp = gtk_entry_get_text( GTK_ENTRY( tex_app_dialog.tex_repeat_entry_x ) );
    x = strtod( tmp, (char **)NULL );

    tmp = gtk_entry_get_text( GTK_ENTRY( tex_app_dialog.tex_repeat_entry_y ) );
    y = strtod( tmp, (char **)NULL );

    mesh_set_texrep( current_mesh, x, y );
}



//////////////////////////////////////////////////////////////////////


gboolean texapp_mouse_down( GtkWidget *widget, GdkEventButton *event,
                            gpointer data ) {

    float x, y;
    int i;

    
    if( selected_tri == NULL ) return TRUE;
    
    widget_to_tv( event->x, event->y, &x, &y, widget->allocation.width, widget->allocation.height );
    
    // find the closest tv to the mouse cursor, and set current_tv to it
    for( i = 0; i < 3; i++ ) {
        if( current_tv == -1 )
            current_tv = i;
        if( !( fabs( selected_tri->tv[i].x - x ) > fabs( selected_tri->tv[current_tv].x - x ) ) &&
            !( fabs( selected_tri->tv[i].y - y ) > fabs( selected_tri->tv[current_tv].y - y ) ) ) {
            
            current_tv = i;
//printf( "current_tv changed to %i \n", current_tv );
        }
    }
    
//printf( "current_tv is %i \n", current_tv );

    return TRUE;
}






gboolean texapp_mouse_motion( GtkWidget *widget, GdkEventMotion *event,
                              gpointer data ) {

    int x, y;
    GdkModifierType state;
    float current_x = 0.0;
    float current_y = 0.0;


    if ( event->is_hint )
        gdk_window_get_pointer ( event->window, &x, &y, &state );
    else {
        x = event->x;
        y = event->y;
        state = event->state;
    }
    
    if( x < 0 || x > widget->allocation.width ||
        y < 0 || y > widget->allocation.height )
        return TRUE;

    widget_to_tv( x, y, &current_x, &current_y, widget->allocation.width, widget->allocation.height );

    if( current_tv < 0 || current_tv > 2 ) {
//        printf( "invalid current_tv in texapp_mouse_motion\n" );
        return TRUE;
    }
    
    if( selected_tri != NULL ) {
        if( (state & GDK_BUTTON1_MASK) || (state & GDK_BUTTON2_MASK) || (state & GDK_BUTTON3_MASK) ) {
            selected_tri->tv[ current_tv ].x = current_x;
            selected_tri->tv[ current_tv ].y = current_y;
            
            texapp_draw_texture( widget );
        }
    }



    return TRUE;
}





gboolean texapp_mouse_up( GtkWidget *widget, GdkEventButton *event,
                          gpointer data ) {

    current_tv = -1;
    
    texapp_draw_texture( widget );
    draw_all_frames();
    return TRUE;
}




#if 0
/////////////////////////////////////////////////////////////////////

gint texapp_uv_adj_cb( GtkWidget *widget, int whichcoord ) {

//printf( "uvcoord: %f -> %f\n", *uv_val, ( double ) GTK_ADJUSTMENT( widget ) ->value );
printf( "tv's : %f %f - %f %f - %f %f\n",
selected_tri->tv[0]->x, selected_tri->tv[0]->y,
selected_tri->tv[1]->x, selected_tri->tv[1]->y,
selected_tri->tv[2]->x, selected_tri->tv[2]->y );

    switch( whichcoord ) {
        case 0:
            selected_tri->tv[0]->x = ( double ) GTK_ADJUSTMENT( widget ) ->value;
            break;
        case 1:
            selected_tri->tv[0]->y = ( double ) GTK_ADJUSTMENT( widget ) ->value;
            break;
        case 2:
            selected_tri->tv[1]->x = ( double ) GTK_ADJUSTMENT( widget ) ->value;
            break;
        case 3:
            selected_tri->tv[1]->y = ( double ) GTK_ADJUSTMENT( widget ) ->value;
            break;
        case 4:
            selected_tri->tv[2]->x = ( double ) GTK_ADJUSTMENT( widget ) ->value;
            break;
        case 5:
            selected_tri->tv[2]->y = ( double ) GTK_ADJUSTMENT( widget ) ->value;
            break;
        default:
            break;
    }

    texapp_draw_texture( tex_app_dialog.drawing_area );
    draw_all_frames();

    return TRUE;
}
#endif


/////////////////////////////////////////////////////////////////////


void texapp_hide( GtkWidget *widget, gpointer data ) {
    gtk_widget_hide( tex_app_dialog.dialog );
}



void show_tex_app( void ) {

#if 0
    GtkAdjustment * hadjustment;
    GtkWidget * hscale;
    int i;
#endif
    GtkWidget * label;

    static int created = FALSE;
    
    if( !created ) {
    
        tex_app_dialog.dialog = gtk_window_new(GTK_WINDOW_TOPLEVEL);
        gtk_container_set_border_width(GTK_CONTAINER(tex_app_dialog.dialog), 5);
        gtk_window_set_title(GTK_WINDOW(tex_app_dialog.dialog), "Edit Texture Coordinates");
    
        tex_app_dialog.hbox = gtk_hbox_new( FALSE, 0 );
        gtk_container_add(GTK_CONTAINER(tex_app_dialog.dialog), tex_app_dialog.hbox);
    
    

#if 0
        for( i = 0; i < 6; i++ ) {
            hadjustment = GTK_ADJUSTMENT( gtk_adjustment_new( 0, 0, 1, 0.01, 0.01, 0.02 ) );
            gtk_signal_connect ( GTK_OBJECT ( hadjustment ), "value_changed",
                                 GTK_SIGNAL_FUNC ( texapp_uv_adj_cb ), i );
            hscale = gtk_hscale_new( hadjustment );
            gtk_widget_show( hscale );
            gtk_box_pack_start( GTK_BOX( tex_app_dialog.vbox ), hscale, TRUE, FALSE, 0 );
        }
#endif


        tex_app_dialog.vbox = gtk_vbox_new( FALSE, 0 );

        label = gtk_label_new( "Texture repeat values\n(for this mesh)" );
        gtk_box_pack_start( GTK_BOX( tex_app_dialog.vbox ), label, TRUE, FALSE, 0 );
        gtk_widget_show( label );

        tex_app_dialog.tex_repeat_entry_x = gtk_entry_new();
        gtk_entry_set_max_length( GTK_ENTRY( tex_app_dialog.tex_repeat_entry_x ), 10 );
        gtk_signal_connect( GTK_OBJECT( tex_app_dialog.tex_repeat_entry_x ), "activate",
                            GTK_SIGNAL_FUNC( texapp_change_repeat_cb ), NULL );
        gtk_box_pack_start( GTK_BOX( tex_app_dialog.vbox ), tex_app_dialog.tex_repeat_entry_x, TRUE, FALSE, 0 );
        gtk_widget_show( tex_app_dialog.tex_repeat_entry_x );

        tex_app_dialog.tex_repeat_entry_y = gtk_entry_new();
        gtk_entry_set_max_length( GTK_ENTRY( tex_app_dialog.tex_repeat_entry_y ), 10 );
        gtk_signal_connect( GTK_OBJECT( tex_app_dialog.tex_repeat_entry_y ), "activate",
                            GTK_SIGNAL_FUNC( texapp_change_repeat_cb ), NULL );
        gtk_box_pack_start( GTK_BOX( tex_app_dialog.vbox ), tex_app_dialog.tex_repeat_entry_y, TRUE, FALSE, 0 );
        gtk_widget_show( tex_app_dialog.tex_repeat_entry_y );

        tex_app_dialog.button = gtk_button_new_with_label( "Close" );
        gtk_signal_connect( GTK_OBJECT( tex_app_dialog.button ), "clicked",
                            GTK_SIGNAL_FUNC( texapp_hide ), NULL );
        gtk_box_pack_start( GTK_BOX( tex_app_dialog.vbox ), tex_app_dialog.button, TRUE, FALSE, 0 );
        gtk_widget_show( tex_app_dialog.button );



        tex_app_dialog.frame = gtk_frame_new( "Texture Preview" );
//        gtk_widget_set_usize( tex_app_dialog.frame, TEXTURE_SIZE + 10, TEXTURE_SIZE + 10 );
    
        tex_app_dialog.drawing_area = gtk_drawing_area_new();
        gtk_widget_set_events( GTK_WIDGET( tex_app_dialog.drawing_area ), 
                               GDK_EXPOSURE_MASK|
                               GDK_BUTTON_PRESS_MASK|
                               GDK_BUTTON_RELEASE_MASK|
                               GDK_POINTER_MOTION_MASK|
                               GDK_POINTER_MOTION_HINT_MASK );
        gtk_widget_set_usize( tex_app_dialog.drawing_area, TEXTURE_SIZE, TEXTURE_SIZE );
        gtk_container_add( GTK_CONTAINER( tex_app_dialog.frame ),
                           tex_app_dialog.drawing_area );
        gtk_signal_connect ( GTK_OBJECT ( tex_app_dialog.drawing_area ),
                             "expose_event",
                             GTK_SIGNAL_FUNC( texapp_expose_event ), NULL );
        gtk_signal_connect ( GTK_OBJECT ( tex_app_dialog.drawing_area ),
                             "configure_event",
                             GTK_SIGNAL_FUNC( texapp_configure_event ), NULL );
        gtk_signal_connect( GTK_OBJECT( tex_app_dialog.drawing_area ), "button_press_event",
                            GTK_SIGNAL_FUNC( texapp_mouse_down ), NULL );
        gtk_signal_connect( GTK_OBJECT( tex_app_dialog.drawing_area ), "motion_notify_event",
                            GTK_SIGNAL_FUNC( texapp_mouse_motion ), NULL );
        gtk_signal_connect( GTK_OBJECT( tex_app_dialog.drawing_area ), "button_release_event",
                            GTK_SIGNAL_FUNC( texapp_mouse_up ), NULL );

        gtk_widget_show( tex_app_dialog.frame );
        gtk_widget_show( tex_app_dialog.drawing_area );
    
        gtk_widget_show( tex_app_dialog.vbox );
    
    
        gtk_box_pack_start( GTK_BOX( tex_app_dialog.hbox ), tex_app_dialog.frame, FALSE, FALSE, 0 );
        gtk_box_pack_start( GTK_BOX( tex_app_dialog.hbox ), tex_app_dialog.vbox, FALSE, FALSE, 0 );

        gtk_widget_show( tex_app_dialog.hbox );

        
        created = TRUE;
    }
    
    gtk_widget_show( tex_app_dialog.dialog );
    texapp_draw_texture( tex_app_dialog.drawing_area );

//    return tex_app_dialog.vbox;
}


////////////////////////////////////////////////////////////////////////////
//
// DEAD CODE
//
////////////////////////////////////////////////////////////////////////////




