/* 
 * File:         tex_man.c
 * 
 * Description:  funcs for creating the "textures" notebook tab.  the texture
 * 		 load dialog stuff was moved into here too.
 * 
 * This source code is part of kludge3d, and is released under the 
 * GNU General Public License.
 * 
 * 
 */
 


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "tex_man.h"
#include "fileops.h"
#include "frame.h"
#include "model.h"
#include "texture.h"

#include "memwatch.h"



int tex_last_row = -1;

struct tex_man {
    GtkWidget *dialog, *vbox, *frame, *table;
    GtkWidget *clist, *button, *entry, *drawing_area, *rename, *delete;// *select;
    GtkWidget *set2d, *apply;
}
tex_man_dialog;

tex *current_tex = NULL;

GtkWidget *tdia, *t_filesel;
unsigned char *texture_image_filename = NULL;





void texman_add_tex( tex *nt ) {
    gchar * tmp_char[ 2 ];
    int row;
    
    if( nt == NULL ) return;
    
    tmp_char[ 0 ] = nt->filename;
    tmp_char[ 1 ] = nt->name;

    if ( GTK_IS_WIDGET( tex_man_dialog.clist ) ) {
        row = gtk_clist_append( GTK_CLIST( tex_man_dialog.clist ), tmp_char );
        gtk_clist_set_row_data( GTK_CLIST( tex_man_dialog.clist ), row, ( gpointer ) nt );
    }
    
}


//////////////////////////////////////////////////////////////////////
//
// texture-load dialog stuff
//
//////////////////////////////////////////////////////////////////////


gint tex_load_ok_cb( GtkWidget *widget, gpointer data ) {
    char * filename;
    tex *new_tex;

    filename = gtk_file_selection_get_filename( GTK_FILE_SELECTION( t_filesel ) );

// FIXME - better checking
    if ( strncmp( filename + strlen( filename ) - 4, ".sgi", 4 ) ) {
        if ( strncmp( filename + strlen( filename ) - 4, ".rgb", 4 ) ) {
            printf( "Not sgi libimage file.\n" );
            return TRUE;
        }
    }

    if ( texture_image_filename )
        free( texture_image_filename );
    texture_image_filename = ( char * ) malloc( strlen( filename ) + 1 );
    strcpy( texture_image_filename, filename );

    gtk_widget_destroy( t_filesel );

    new_tex = tex_new();
    new_tex->filename = texture_image_filename;
    tex_load( new_tex );


    model_add_texture( new_tex );

    texman_add_tex( new_tex );

    return TRUE;
}

// show file selector

gint tex_load_show_dlg( GtkWidget *widget, gpointer data ) {
    GtkWidget * file;

    file = gtk_file_selection_new( "Load texture..." );
    t_filesel = file;
    if ( texture_image_filename )
        gtk_file_selection_set_filename( GTK_FILE_SELECTION( file ),
                                         texture_image_filename );

    gtk_signal_connect( GTK_OBJECT( GTK_FILE_SELECTION( file ) ->
                                    cancel_button ), "clicked",
                        GTK_SIGNAL_FUNC( destroy_dialog ), file );
    gtk_signal_connect( GTK_OBJECT( GTK_FILE_SELECTION( file ) ->
                                    ok_button ), "clicked",
                        GTK_SIGNAL_FUNC( tex_load_ok_cb ), file );

    gtk_widget_show( file );

    return TRUE;
}



/////////////////////////////////////////////////////////////////////
//
// texture manager stuff
//
/////////////////////////////////////////////////////////////////////


void texman_draw_texture( GtkWidget * widget ) {
    
    int tex_psx, tex_psy;

    if ( current_tex ) {
        tex_psx = ( widget->allocation.width - current_tex->thumbnail_width ) >> 1;
        tex_psy = ( widget->allocation.height - current_tex->thumbnail_height ) >> 1;

        gdk_draw_rgb_image( widget->window,
                            widget->style->fg_gc[ GTK_WIDGET_STATE ( widget ) ],
                            tex_psx, tex_psy,
                            current_tex->thumbnail_width,
                            current_tex->thumbnail_height,
                            GDK_RGB_DITHER_NORMAL, current_tex->thumbnail_data,
                            current_tex->thumbnail_width * current_tex->depth );
    } else {
        gdk_draw_rectangle( widget->window,
                            widget->style->fg_gc[ GTK_WIDGET_STATE ( widget ) ],
                            TRUE, 0, 0,
                            widget->allocation.width,
                            widget->allocation.height );
    }
    
}


void texman_dialog_sensitive( int truth ) {
    gtk_widget_set_sensitive( tex_man_dialog.entry, truth );
    gtk_widget_set_sensitive( tex_man_dialog.rename, truth );
    gtk_widget_set_sensitive( tex_man_dialog.delete, truth );
//    gtk_widget_set_sensitive( tex_man_dialog.select, truth );
    gtk_widget_set_sensitive( tex_man_dialog.set2d, truth );
    gtk_widget_set_sensitive( tex_man_dialog.apply, truth );
}


void texman_selection_cb( GtkWidget *clist,
                     gint row,
                     gint column,
                     GdkEventButton *event,
                     gpointer data ) {
    tex_last_row = row;
    current_tex = gtk_clist_get_row_data( GTK_CLIST( tex_man_dialog.clist ),
                                          row );

//FIXME - not necessary, since we force textures to a specific size...
//    if ( current_tex )
//        gtk_widget_set_usize( tex_man_dialog.frame, current_tex->width + 20,
//                              current_tex->height + 20 );

    gtk_widget_draw( tex_man_dialog.drawing_area, NULL );

    if ( current_tex ) {
        if ( current_tex->name ) {
            gtk_entry_set_text( GTK_ENTRY( tex_man_dialog.entry ), current_tex->name );
        } else {
            gtk_entry_set_text( GTK_ENTRY( tex_man_dialog.entry ), "\0" );
        }
    }
    texman_dialog_sensitive( TRUE );
}

gint texman_expose_event( GtkWidget *widget, GdkEventExpose *event, gpointer data ) {

    texman_draw_texture( widget );
    
    return 0;
}

gint texman_configure_event( GtkWidget *widget, GdkEvent *event, gpointer data ) {

    gtk_widget_draw( tex_man_dialog.drawing_area, NULL );
    gdk_flush();

    return TRUE;
}


void texman_rebuild_list(  ) {
    
    int row;
    char *tmp[ 2 ];
    GList *ch;
    tex *nt;

    // used when deleting the model / freeing all the textures
    // or when loading a model that already has a list of textures
    
    g_return_if_fail( tex_man_dialog.clist != NULL );
    g_return_if_fail( GTK_WIDGET_VISIBLE( tex_man_dialog.clist ) );
    
    gtk_clist_clear( GTK_CLIST( tex_man_dialog.clist ) );

    for ( ch = model_textures; ch != NULL; ch = ch->next ) {
        nt = ( tex * ) ch->data;
        tmp[ 0 ] = nt->filename;
        tmp[ 1 ] = nt->name;
        row = gtk_clist_append( GTK_CLIST( tex_man_dialog.clist ), tmp );
        gtk_clist_set_row_data( GTK_CLIST( tex_man_dialog.clist ), 
                                row, ( gpointer ) nt );
    }
    current_tex = NULL;

}


void texman_delete_cb( GtkWidget *widget, gpointer data ) {
    tex * tt;


    tt = ( tex * ) gtk_clist_get_row_data( GTK_CLIST( tex_man_dialog.clist ),
                                           tex_last_row );

    model_remove_texture( tt );

    gtk_clist_remove( GTK_CLIST( tex_man_dialog.clist ), tex_last_row );
    if ( g_list_length( model_textures ) < 1 )
        texman_dialog_sensitive( FALSE );
    gtk_widget_draw( tex_man_dialog.drawing_area, NULL );
}

void texman_rename_cb( GtkWidget *widget, gpointer data ) {
    char * tmp = gtk_entry_get_text( GTK_ENTRY( tex_man_dialog.entry ) );

    if ( current_tex->name )
        free( current_tex->name );
    current_tex->name = malloc( strlen( tmp ) + 1 );
    strcpy( current_tex->name, tmp );

    gtk_clist_set_text( GTK_CLIST( tex_man_dialog.clist ),
                        tex_last_row, 1, current_tex->name );
}


gint texman_apply_tex_cb( GtkWidget *widget, gpointer data ) {

    if( current_mesh == NULL || current_tex == NULL )
        return FALSE;
    
    mesh_set_texture( current_mesh, current_tex );
    
    draw_all_frames();

    return TRUE;
}


gint texman_set2d_cb( GtkWidget *widget, gpointer data ) {
    GList * cl;
    Tri *t;

    if( current_mesh == NULL || current_tex == NULL )
        return FALSE;
    
    for ( cl = current_mesh->triangles; cl != NULL; cl = cl->next ) {
        t = ( Tri * ) cl->data;
#if 0
        t->tv[ 0 ].x = t->vertices[ 0 ]->v[ 0 ];    // / 100;
        t->tv[ 0 ].y = t->vertices[ 0 ]->v[ 1 ];    // / 100;
        t->tv[ 1 ].x = t->vertices[ 1 ]->v[ 0 ];    // / 100;
        t->tv[ 1 ].y = t->vertices[ 1 ]->v[ 1 ];    // / 100;
        t->tv[ 2 ].x = t->vertices[ 2 ]->v[ 0 ];    // / 100;
        t->tv[ 2 ].y = t->vertices[ 2 ]->v[ 1 ];    // / 100;
#else
        // hack...
        t->tv[ 0 ].x = 0.0;
        t->tv[ 0 ].y = 0.0;
        t->tv[ 1 ].x = 1.0;
        t->tv[ 1 ].y = 0.0;
        t->tv[ 2 ].x = 0.0;
        t->tv[ 2 ].y = 1.0;
#endif
        
    }

    draw_all_frames();

    return TRUE;
}




GtkWidget * create_tex_manager( void ) {

    GtkWidget * button;
    char *col_names[] = {"Filename", "Description"};

    tex_man_dialog.vbox = gtk_vbox_new( FALSE, 0 );

    tex_man_dialog.frame = gtk_frame_new( "Texture Preview" );
//    gtk_widget_set_usize( tex_man_dialog.frame, TEXTURE_SIZE + 10, TEXTURE_SIZE + 10 );
    gtk_widget_show( tex_man_dialog.frame );
    gtk_box_pack_start( GTK_BOX( tex_man_dialog.vbox ), tex_man_dialog.frame, FALSE, FALSE, 0 );
    gtk_widget_show( tex_man_dialog.vbox );

    tex_man_dialog.drawing_area = gtk_drawing_area_new();
    gtk_widget_set_usize( tex_man_dialog.drawing_area, TEXTURE_SIZE, TEXTURE_SIZE );
    gtk_container_add( GTK_CONTAINER( tex_man_dialog.frame ),
                       tex_man_dialog.drawing_area );
    gtk_signal_connect ( GTK_OBJECT ( tex_man_dialog.drawing_area ),
                         "expose_event",
                         GTK_SIGNAL_FUNC( texman_expose_event ), NULL );

    gtk_signal_connect ( GTK_OBJECT ( tex_man_dialog.drawing_area ),
                         "configure_event",
                         GTK_SIGNAL_FUNC( texman_configure_event ), NULL );
    gtk_widget_show( tex_man_dialog.drawing_area );

    tex_man_dialog.clist = gtk_clist_new_with_titles( 2, col_names );
    gtk_signal_connect( GTK_OBJECT( tex_man_dialog.clist ), "select_row",
                        GTK_SIGNAL_FUNC( texman_selection_cb ),
                        NULL );
    gtk_clist_column_titles_show( GTK_CLIST( tex_man_dialog.clist ) );
    gtk_box_pack_start( GTK_BOX( tex_man_dialog.vbox ), tex_man_dialog.clist,
                        TRUE, TRUE, 0 );

    texman_rebuild_list();
    gtk_widget_show( tex_man_dialog.clist );

    tex_man_dialog.table = gtk_table_new( 3, 3, FALSE );
    gtk_box_pack_start( GTK_BOX( tex_man_dialog.vbox ), tex_man_dialog.table, FALSE, FALSE, 5 );
    gtk_table_set_row_spacings( GTK_TABLE( tex_man_dialog.table ), 5 );
    gtk_table_set_col_spacings( GTK_TABLE( tex_man_dialog.table ), 5 );
    gtk_widget_show( tex_man_dialog.table );

    button = gtk_button_new_with_label( "Load..." );
    gtk_table_attach_defaults( GTK_TABLE( tex_man_dialog.table ),
                               button,
                               0, 1, 0, 1 );
    gtk_signal_connect( GTK_OBJECT( button ), "clicked",
                        GTK_SIGNAL_FUNC( tex_load_show_dlg ), NULL );
    gtk_widget_show( button );

//    button = gtk_button_new_with_label( "Select" );
//    gtk_widget_set_sensitive( button, FALSE );
//    tex_man_dialog.select = button;
//    gtk_table_attach_defaults( GTK_TABLE( tex_man_dialog.table ), button,
//                               1, 2, 0, 1 );
//    gtk_widget_show( button );

    button = gtk_button_new_with_label( "Delete" );
    tex_man_dialog.delete = button;
    gtk_table_attach_defaults( GTK_TABLE( tex_man_dialog.table ), button,
                               2, 3, 0, 1 );
    gtk_signal_connect( GTK_OBJECT( button ), "clicked",
                        GTK_SIGNAL_FUNC( texman_delete_cb ), NULL );
    gtk_widget_show( button );

    button = gtk_button_new_with_label( "Rename" );
    tex_man_dialog.rename = button;
    gtk_signal_connect( GTK_OBJECT( button ), "clicked",
                        GTK_SIGNAL_FUNC( texman_rename_cb ), NULL );
    gtk_table_attach_defaults( GTK_TABLE( tex_man_dialog.table ),
                               button,
                               0, 1, 1, 2 );
    gtk_widget_show( button );

    tex_man_dialog.entry = gtk_entry_new();
    gtk_signal_connect( GTK_OBJECT( tex_man_dialog.entry ), "activate",
                        GTK_SIGNAL_FUNC( texman_rename_cb ), NULL );
    gtk_table_attach_defaults( GTK_TABLE( tex_man_dialog.table ),
                               tex_man_dialog.entry,
                               1, 3, 1, 2 );
    gtk_widget_show( tex_man_dialog.entry );

    button = gtk_button_new_with_label( "Apply Texture" );
    gtk_widget_set_sensitive( button, FALSE );
    gtk_signal_connect( GTK_OBJECT( button ), "clicked",
                        GTK_SIGNAL_FUNC( texman_apply_tex_cb ), NULL );
    tex_man_dialog.apply = button;
    gtk_table_attach_defaults( GTK_TABLE( tex_man_dialog.table ), button,
                               0, 2, 2, 3 );
    gtk_widget_show( button );

    button = gtk_button_new_with_label( "Remap UV's" );
    gtk_widget_set_sensitive( button, FALSE );
    gtk_signal_connect( GTK_OBJECT( button ), "clicked",
                        GTK_SIGNAL_FUNC( texman_set2d_cb ), NULL );
    tex_man_dialog.set2d = button;
    gtk_table_attach_defaults( GTK_TABLE( tex_man_dialog.table ), button,
                               2, 3, 2, 3 );
    gtk_widget_show( button );


    texman_dialog_sensitive( FALSE );


    return tex_man_dialog.vbox;
}


////////////////////////////////////////////////////////////////////////////
//
// DEAD CODE
//
////////////////////////////////////////////////////////////////////////////

#if 0
gint destroy_tm_dialog( GtkWidget *widget, gpointer data ) {
    current_tex = NULL;
    destroy_dialog( widget, data );

    return TRUE;
}

#endif



