/* 
 * File:         texture.c
 * 
 * Description:  funcs for creating and adding textures.  Essentially 
 * 		 just a wrapper around sgi.c
 * 
 * This source code is part of kludge3d, and is released under the 
 * GNU General Public License.
 * 
 * 
 */
 
#include <stdio.h>
#include <stdlib.h>

#include "texture.h"
#include "globals.h"
#include "sgi.h"

#include "memwatch.h"


unsigned char *vertical_flip( unsigned char *img,
                              int width, int height, int depth );


tex *tex_new( void ) {
    tex * newt;

    newt = ( tex * ) malloc( sizeof( tex ) );
    newt->data = NULL;
    newt->filename = NULL;
    newt->width = newt->height = newt->depth = 0;

    return newt;
}

void tex_delete( tex* tt ) {

    if( tt == NULL ) return;

    if ( tt->name )
        free( tt->name );
    if ( tt->filename )
        free( tt->filename );
    if ( tt->data )
        free( tt->data );

    free( tt );

}



unsigned char *vertical_flip( unsigned char *img,
                              int width, int height, int depth ) {
    unsigned char * tmp_buff;
    int c, d = width * depth, off, off2;

    tmp_buff = ( unsigned char * ) malloc( width * height * depth );

    for ( c = height - 1, off = 0, off2 = width * ( height - 1 ) * depth;
            c >= 0; c--, off += d, off2 -= d )
        memcpy( tmp_buff + off, img + off2, d );
    free( img );
    return tmp_buff;
}


void tex_scale( tex *ntex, int width, int height ) {
    unsigned char * tmp_buff;
    int x, y;
    int depth;
    int src_current_row;
    int src_current_col;
    int dest_offset = 0;
    
    if( ntex == NULL || ntex->data == NULL )
        return;
    
    depth = ntex->depth;
    
    tmp_buff = ( unsigned char * ) malloc( width * height * depth );

    for( y = 0; y < height; y++ ) {

        src_current_row = (int)( (float)ntex->height * ( (float)y / (float)height ) ) * ( ntex->width * depth );
        // array index of the first element of the current row

        for( x = 0; x < width; x++ ) {
            
            src_current_col = (int)( (float)ntex->width * ( (float)x / (float)width ) ) * depth;

            tmp_buff[ dest_offset    ] = ntex->data[ src_current_row + src_current_col    ];
            tmp_buff[ dest_offset +1 ] = ntex->data[ src_current_row + src_current_col +1 ];
            tmp_buff[ dest_offset +2 ] = ntex->data[ src_current_row + src_current_col +2 ];
            
            dest_offset += depth;
            
        }

    }
    

    free( ntex->data );
    ntex->data = tmp_buff;
    ntex->width = width;
    ntex->height = height;
}




void tex_make_thumbnail( tex* t ) {

    // thumbnail stuff
    
    int size;
    
    g_return_if_fail( t != NULL );
    g_return_if_fail( t->data != NULL );

    size = t->width * t->height * t->depth;
    t->thumbnail_data = ( unsigned char * ) malloc( size );
    memcpy( t->thumbnail_data, t->data, size );
    
    t->thumbnail_data = vertical_flip( t->thumbnail_data, t->width, t->height, t->depth );
    t->thumbnail_width = t->width;
    t->thumbnail_height = t->height;

}



int tex_load( tex *ntex ) {
    unsigned char * image;
    int c;

    g_return_val_if_fail( ntex != NULL, 1 );
    
    image = read_sgi( ntex->filename, &ntex->width, &ntex->height, &ntex->depth );
    if ( image == NULL ) return 1;

    ntex->data = image;


    // the equiv. of "basename"
    c = strlen( ntex->filename ) - 1;
    while ( ntex->filename[ c ] != '/' && c > 0 )
        c--;

    ntex->name = ( char * ) malloc( strlen( ntex->filename + c ) + 1 );
    strcpy( ntex->name, ntex->filename + c + 1 );

tex_scale( ntex, TEXTURE_SIZE, TEXTURE_SIZE );

    tex_make_thumbnail( ntex );
    
    return 0;
}



