/* 
 * File:         toolbar.c
 * 
 * Description:  funcs for creating the toolbar, and handling button-press
 * 		 events, etc...
 * 
 * This source code is part of kludge3d, and is released under the 
 * GNU General Public License.
 * 
 * 
 */
 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <gtk/gtk.h>


#include "toolbar.h"
#include "fileops.h"
#include "view.h"    /* for click_mode */

#include "memwatch.h"

//#include "pixmaps/exit.xpm"
#include "pixmaps/file.xpm"
//#include "pixmaps/lineadd.xpm"
//#include "pixmaps/linesel.xpm"
//#include "pixmaps/linesub.xpm"
#include "pixmaps/new.xpm"
#include "pixmaps/pointadd.xpm"
#include "pixmaps/pointmove.xpm"
#include "pixmaps/pointsel.xpm"
#include "pixmaps/pointsub.xpm"
//#include "pixmaps/right_arrow.xpm"
#include "pixmaps/save.xpm"
#include "pixmaps/triadd.xpm"
#include "pixmaps/trisel.xpm"
#include "pixmaps/trisub.xpm"
#include "pixmaps/tritex.xpm"

#include "pixmaps/zoom.xpm"
#include "pixmaps/pan.xpm"


GtkWidget *popup;
GdkPixmap *pixmap_ar[ 12 ];   // poor hack
GdkBitmap *bitmap_ar[ 12 ];
GtkWidget *modal_pixmap[ 4 ];

int menu_callback( GtkWidget *widget, gpointer data ) {
    printf( "Menu callback: %s\n", ( char * ) data );
    return 0;
}

void my_quit( GtkWidget *widget, gpointer data ) {
    gtk_exit( 0 );
}

void toolbar_callback( GtkWidget *widget, GdkEventButton *event, gpointer data ) {
    click_mode[ event->button ] = ( int ) data;

    gtk_pixmap_set( GTK_PIXMAP( modal_pixmap[ event->button ] ),
                    pixmap_ar[ ( int ) data ], bitmap_ar[ ( int ) data ] );
}

GtkWidget *tb_pixmap_new( char **xpm, GtkWidget *window, int cm ) {
    GdkPixmap * gdk_pixmap;
    GdkBitmap *mask;
    GtkWidget *pixmap;

    gdk_pixmap = gdk_pixmap_create_from_xpm_d( window->window, &mask,
                 &window->style->bg[ GTK_STATE_NORMAL ],
                 xpm );
    pixmap_ar[ cm ] = gdk_pixmap;
    bitmap_ar[ cm ] = mask;
    pixmap = gtk_pixmap_new( gdk_pixmap, mask );
    gtk_widget_show( pixmap );

    return pixmap;
}

GtkWidget *pixmap_new( char **xpm, GtkWidget *window ) {
    GdkPixmap * gdk_pixmap;
    GdkBitmap *mask;
    GtkWidget *pixmap;

    gdk_pixmap = gdk_pixmap_create_from_xpm_d( window->window, &mask,
                 &window->style->bg[ GTK_STATE_NORMAL ],
                 xpm );
    pixmap = gtk_pixmap_new( gdk_pixmap, mask );
    gtk_widget_show( pixmap );

    return pixmap;
}

GtkWidget *tb_get_right_half( void ) {
    GtkWidget * hbox, *frame;

    hbox = gtk_hbox_new( FALSE, 0 );
    gtk_widget_show( hbox );

    frame = gtk_frame_new( NULL );
    gtk_frame_set_shadow_type( GTK_FRAME( frame ), GTK_SHADOW_OUT );
    gtk_widget_show( frame );
    gtk_container_add( GTK_CONTAINER( frame ), hbox );

    modal_pixmap[ 1 ] = gtk_pixmap_new( pixmap_ar[ click_mode[ 1 ] ],
                                        bitmap_ar[ click_mode[ 1 ] ] );
    gtk_widget_show( modal_pixmap[ 1 ] );
    gtk_box_pack_start( GTK_BOX( hbox ), modal_pixmap[ 1 ], FALSE, FALSE, 2 );
    modal_pixmap[ 2 ] = gtk_pixmap_new( pixmap_ar[ click_mode[ 2 ] ],
                                        bitmap_ar[ click_mode[ 2 ] ] );
    gtk_widget_show( modal_pixmap[ 2 ] );
    gtk_box_pack_start( GTK_BOX( hbox ), modal_pixmap[ 2 ], FALSE, FALSE, 2 );
    modal_pixmap[ 3 ] = gtk_pixmap_new( pixmap_ar[ click_mode[ 3 ] ],
                                        bitmap_ar[ click_mode[ 3 ] ] );
    gtk_widget_show( modal_pixmap[ 3 ] );
    gtk_box_pack_start( GTK_BOX( hbox ), modal_pixmap[ 3 ], FALSE, FALSE, 2 );

    return frame;
}

GtkWidget *create_toolbar( void ) {
    GtkWidget * toolbar, *frame, *hbox;
    GtkWidget *pixmap;
    GtkWidget *w;

    toolbar = gtk_toolbar_new( GTK_ORIENTATION_HORIZONTAL, GTK_TOOLBAR_BOTH );
    gtk_toolbar_set_style( GTK_TOOLBAR( toolbar ), GTK_TOOLBAR_TEXT );
    gtk_toolbar_set_button_relief( GTK_TOOLBAR( toolbar ), GTK_RELIEF_NONE );
    gtk_toolbar_set_space_style( GTK_TOOLBAR( toolbar ), GTK_TOOLBAR_SPACE_LINE );

//    gtk_toolbar_append_item( GTK_TOOLBAR( toolbar ), NULL, "Quit", NULL,
//                             pixmap_new( exit_xpm, window ),
//                             GTK_SIGNAL_FUNC( my_quit ),
//                             ( gpointer ) popup );
    gtk_toolbar_append_item( GTK_TOOLBAR( toolbar ), NULL, "New", NULL,
                             pixmap_new( new_xpm, window ),
                             GTK_SIGNAL_FUNC( show_new_dialog ),
                             ( gpointer ) popup );
    gtk_toolbar_append_item( GTK_TOOLBAR( toolbar ), NULL, "Open", NULL,
                             pixmap_new( file_xpm, window ),
                             GTK_SIGNAL_FUNC( show_load_dialog ),
                             ( gpointer ) popup );
    gtk_toolbar_append_item( GTK_TOOLBAR( toolbar ), NULL, "Save", NULL,
                             pixmap_new( save_xpm, window ),
                             GTK_SIGNAL_FUNC( save_menu_cb ),
                             ( gpointer ) popup );
    gtk_toolbar_append_space( GTK_TOOLBAR( toolbar ) );

    pixmap = tb_pixmap_new( pointsel_xpm, window, CM_POINT_SEL );
    w = gtk_toolbar_append_item( GTK_TOOLBAR( toolbar ), NULL, "Point Select", NULL,
                                 pixmap,
                                 NULL, NULL );
    gtk_signal_connect( GTK_OBJECT( w ), "button_press_event",
                        GTK_SIGNAL_FUNC( toolbar_callback ),
                        ( gpointer ) CM_POINT_SEL );
    pixmap = tb_pixmap_new( pointmove_xpm, window, CM_POINT_MOVE );
    w = gtk_toolbar_append_item( GTK_TOOLBAR( toolbar ), NULL, "Point Move", NULL,
                                 pixmap,
                                 NULL, NULL );
    gtk_signal_connect( GTK_OBJECT( w ), "button_press_event",
                        GTK_SIGNAL_FUNC( toolbar_callback ),
                        ( gpointer ) CM_POINT_MOVE );
    pixmap = tb_pixmap_new( pointadd_xpm, window, CM_POINT_ADD );
    w = gtk_toolbar_append_item( GTK_TOOLBAR( toolbar ), NULL, "Point Add", NULL,
                                 pixmap,
                                 NULL, NULL );
    gtk_signal_connect( GTK_OBJECT( w ), "button_press_event",
                        GTK_SIGNAL_FUNC( toolbar_callback ),
                        ( gpointer ) CM_POINT_ADD );
    pixmap = tb_pixmap_new( pointsub_xpm, window, CM_POINT_DEL );
    w = gtk_toolbar_append_item( GTK_TOOLBAR( toolbar ), NULL, "Point Subtract", NULL,
                                 pixmap,
                                 NULL, NULL );
    gtk_signal_connect( GTK_OBJECT( w ), "button_press_event",
                        GTK_SIGNAL_FUNC( toolbar_callback ),
                        ( gpointer ) CM_POINT_DEL );

    gtk_toolbar_append_space( GTK_TOOLBAR( toolbar ) );

#if 0
    pixmap = tb_pixmap_new( linesel_xpm, window, CM_LINE_SEL );
    w = gtk_toolbar_append_item( GTK_TOOLBAR( toolbar ), NULL, "Line Select", NULL,
                                 pixmap,
                                 NULL, NULL );
    gtk_signal_connect( GTK_OBJECT( w ), "button_press_event",
                        GTK_SIGNAL_FUNC( toolbar_callback ),
                        ( gpointer ) CM_LINE_SEL );
    pixmap = tb_pixmap_new( lineadd_xpm, window, CM_LINE_ADD );
    w = gtk_toolbar_append_item( GTK_TOOLBAR( toolbar ), NULL, "Line Add", NULL,
                                 pixmap,
                                 NULL, NULL );
    gtk_signal_connect( GTK_OBJECT( w ), "button_press_event",
                        GTK_SIGNAL_FUNC( toolbar_callback ),
                        ( gpointer ) CM_LINE_ADD );
    pixmap = tb_pixmap_new( linesub_xpm, window, CM_LINE_DEL );
    w = gtk_toolbar_append_item( GTK_TOOLBAR( toolbar ), NULL, "Line Subtract", NULL,
                                 pixmap,
                                 NULL, NULL );
    gtk_signal_connect( GTK_OBJECT( w ), "button_press_event",
                        GTK_SIGNAL_FUNC( toolbar_callback ),
                        ( gpointer ) CM_LINE_DEL );

    gtk_toolbar_append_space( GTK_TOOLBAR( toolbar ) );
#endif

    pixmap = tb_pixmap_new( trisel_xpm, window, CM_TRI_SEL );
    w = gtk_toolbar_append_item( GTK_TOOLBAR( toolbar ), NULL, "Triangle Select",
                                 NULL, pixmap,
                                 NULL, NULL );
    gtk_signal_connect( GTK_OBJECT( w ), "button_press_event",
                        GTK_SIGNAL_FUNC( toolbar_callback ),
                        ( gpointer ) CM_TRI_SEL );
    pixmap = tb_pixmap_new( tritex_xpm, window, CM_TRI_TEX );
    w = gtk_toolbar_append_item( GTK_TOOLBAR( toolbar ), NULL, "Triangle Texture",
                                 NULL, pixmap,
                                 NULL, NULL );
    gtk_signal_connect( GTK_OBJECT( w ), "button_press_event",
                        GTK_SIGNAL_FUNC( toolbar_callback ),
                        ( gpointer ) CM_TRI_TEX );
    pixmap = tb_pixmap_new( triadd_xpm, window, CM_TRI_ADD );
    w = gtk_toolbar_append_item( GTK_TOOLBAR( toolbar ), NULL, "Triangle Add", NULL,
                                 pixmap,
                                 NULL, NULL );
    gtk_signal_connect( GTK_OBJECT( w ), "button_press_event",
                        GTK_SIGNAL_FUNC( toolbar_callback ),
                        ( gpointer ) CM_TRI_ADD );
    pixmap = tb_pixmap_new( trisub_xpm, window, CM_TRI_DEL );
    w = gtk_toolbar_append_item( GTK_TOOLBAR( toolbar ), NULL, "Triangle Subtract",
                                 NULL, pixmap,
                                 NULL, NULL );
    gtk_signal_connect( GTK_OBJECT( w ), "button_press_event",
                        GTK_SIGNAL_FUNC( toolbar_callback ),
                        ( gpointer ) CM_TRI_DEL );


    gtk_toolbar_append_space( GTK_TOOLBAR( toolbar ) );

    pixmap = tb_pixmap_new( zoom_xpm, window, CM_ZOOM );
    w = gtk_toolbar_append_item( GTK_TOOLBAR( toolbar ), NULL, "Zoom View",
                                 NULL, pixmap,
                                 NULL, NULL );
    gtk_signal_connect( GTK_OBJECT( w ), "button_press_event",
                        GTK_SIGNAL_FUNC( toolbar_callback ),
                        ( gpointer ) CM_ZOOM );

    pixmap = tb_pixmap_new( pan_xpm, window, CM_PAN );
    w = gtk_toolbar_append_item( GTK_TOOLBAR( toolbar ), NULL, "Pan View",
                                 NULL, pixmap,
                                 NULL, NULL );
    gtk_signal_connect( GTK_OBJECT( w ), "button_press_event",
                        GTK_SIGNAL_FUNC( toolbar_callback ),
                        ( gpointer ) CM_PAN );


    /*
       gtk_toolbar_append_space(GTK_TOOLBAR(toolbar));
       gtk_toolbar_append_widget(GTK_TOOLBAR(toolbar), hbox,
       "Mode indicators", NULL);
    */

    gtk_widget_show( toolbar );

    frame = gtk_frame_new( NULL );
    gtk_frame_set_shadow_type( GTK_FRAME( frame ), GTK_SHADOW_OUT );
    gtk_widget_show( frame );
    gtk_container_add( GTK_CONTAINER( frame ), toolbar );
    hbox = gtk_hbox_new( FALSE, 0 );
    gtk_box_pack_start( GTK_BOX( hbox ), frame, TRUE, TRUE, 0 );

    gtk_box_pack_start( GTK_BOX( hbox ), tb_get_right_half(),
                        FALSE, FALSE, 0 );
    gtk_widget_show( hbox );

    return hbox;
}


