/* 
 * File:         tri.c
 * 
 * Description:  functions to handle things like triangle creation
 * 		 and selection
 * 
 * This source code is part of kludge3d, and is released under the 
 * GNU General Public License.
 * 
 * 
 */
 
#include <stdlib.h>
#include <string.h>  /* why the hell is memset in string.h ? */


#include "tri.h"

#include "memwatch.h"



Tri *selected_tri = NULL;


Tri * tri_new( void ) {
    Tri * result = (Tri*) malloc( sizeof( Tri ) );
    memset( result, 0, sizeof( Tri ) );

    return result;
}


void tris_unselect_all( void ) {
    selected_tri = NULL;
}



void tris_add_v( Mesh* m, Tri * t ) {

    if( m != NULL && t != NULL )
        m->triangles = g_list_append( m->triangles, ( gpointer ) t );


}



void tris_del_v( Mesh* m, Tri * t ) {

    GList * cl;

    if( m == NULL || m->triangles == NULL )
        return ;
    if ( t == NULL )
        return ;

    cl = g_list_find( m->triangles, ( gpointer ) t );
    if( cl != NULL ) {
        m->triangles = g_list_remove( m->triangles, cl->data );
    }
    
    t->vertices[0] = t->vertices[1] = t->vertices[2] = NULL;
    
    free( t );
}


void tris_del_point( Point *p ) {
    GList * cl;
    Tri *tn;

    if( current_mesh == NULL || current_mesh->triangles == NULL )
        return ;

    for ( cl = current_mesh->triangles; cl; ) {
        tn = ( Tri * ) cl->data;
        if ( ( tn->vertices[ 0 ] == p ) || ( tn->vertices[ 1 ] == p ) ||
                ( tn->vertices[ 2 ] == p ) ) {
            current_mesh->triangles = g_list_remove_link( current_mesh->triangles,
                                    g_list_find( current_mesh->triangles,
                                                 ( gpointer ) tn ) );
            free( tn );
            cl = current_mesh->triangles;
            if ( cl == NULL ) return ;
        } else
            cl = cl->next;
    }
}





/////////////////////////////////////////////////////////////////////////
//
// DEAD CODE
//
/////////////////////////////////////////////////////////////////////////



#if 0
void tris_add( Point *p1, Point *p2, Point *p3 ) {
    Tri * tn;
    tn = ( Tri * ) malloc( sizeof( Tri ) );

//    tn->texture = NULL;
    tn->vertices[ 0 ] = p1;
    tn->vertices[ 1 ] = p2;
    tn->vertices[ 2 ] = p3;

    tn->tv[ 0 ] = NULL;
    tn->tv[ 1 ] = NULL;
    tn->tv[ 2 ] = NULL;

    if( current_mesh != NULL )
        current_mesh->triangles = g_list_append( current_mesh->triangles, ( gpointer ) tn );
}

void tris_sel( Point *p1, Point *p2, Point *p3 ) {
    Tri * tn;
    GList *cl;
    tn = ( Tri * ) malloc( sizeof( Tri ) );

    if ( current_mesh->triangles == NULL )
        return ;

    for ( cl = current_mesh->triangles; cl; cl = cl->next ) {
        tn = ( Tri * ) cl->data;
        if ( ( ( tn->vertices[ 0 ] == p1 ) && ( tn->vertices[ 1 ] == p2 ) &&
                ( tn->vertices[ 2 ] == p3 ) ) ||
                ( ( tn->vertices[ 0 ] == p1 ) && ( tn->vertices[ 2 ] == p2 ) &&
                  ( tn->vertices[ 1 ] == p3 ) ) ||
                ( ( tn->vertices[ 1 ] == p1 ) && ( tn->vertices[ 0 ] == p2 ) &&
                  ( tn->vertices[ 2 ] == p3 ) ) ||
                ( ( tn->vertices[ 1 ] == p1 ) && ( tn->vertices[ 2 ] == p2 ) &&
                  ( tn->vertices[ 0 ] == p3 ) ) ||
                ( ( tn->vertices[ 2 ] == p1 ) && ( tn->vertices[ 1 ] == p1 ) &&
                  ( tn->vertices[ 1 ] == p3 ) ) ||
                ( ( tn->vertices[ 2 ] == p1 ) && ( tn->vertices[ 1 ] == p2 ) &&
                  ( tn->vertices[ 0 ] == p3 ) ) ) {
            selected_tri = tn;
            break;
        }
    }
}

void tris_del( Point *p1, Point *p2, Point *p3 ) {
    Tri * tn;
    GList *cl;
    tn = ( Tri * ) malloc( sizeof( Tri ) );

    if( current_mesh == NULL || current_mesh->triangles == NULL )
        return ;

    for ( cl = current_mesh->triangles; cl; cl = cl->next ) {
        tn = ( Tri * ) cl->data;
        if ( ( ( tn->vertices[ 0 ] == p1 ) && ( tn->vertices[ 1 ] == p2 ) &&
                ( tn->vertices[ 2 ] == p3 ) ) ||
                ( ( tn->vertices[ 0 ] == p1 ) && ( tn->vertices[ 2 ] == p2 ) &&
                  ( tn->vertices[ 1 ] == p3 ) ) ||
                ( ( tn->vertices[ 1 ] == p1 ) && ( tn->vertices[ 0 ] == p2 ) &&
                  ( tn->vertices[ 2 ] == p3 ) ) ||
                ( ( tn->vertices[ 1 ] == p1 ) && ( tn->vertices[ 2 ] == p2 ) &&
                  ( tn->vertices[ 0 ] == p3 ) ) ||
                ( ( tn->vertices[ 2 ] == p1 ) && ( tn->vertices[ 1 ] == p1 ) &&
                  ( tn->vertices[ 1 ] == p3 ) ) ||
                ( ( tn->vertices[ 2 ] == p1 ) && ( tn->vertices[ 1 ] == p2 ) &&
                  ( tn->vertices[ 0 ] == p3 ) ) ) {
            current_mesh->triangles = g_list_remove_link( current_mesh->triangles,
                                    g_list_find( current_mesh->triangles,
                                                 ( gpointer ) tn ) );
            free( tn );
            break;
        }
    }
}


typedef struct _Pair_drawable_adj Pair_drawable_adj;


struct _Pair_drawable_adj {
    
    GdkDrawable *drawable;
    gadj *adj;
    
};



void draw_tri( Pair_drawable_adj * pr, Point* p1, Point* p2, Point* p3, int r, int g, int b ) {
    
    GdkDrawable *drawable = pr->drawable;
    int x1, y1, x2, y2, x3, y3;

    x1 = point_get_lx( p1, pr->adj );
    x2 = point_get_lx( p2, pr->adj );
    x3 = point_get_lx( p3, pr->adj );

    y1 = point_get_ly( p1, pr->adj );
    y2 = point_get_ly( p2, pr->adj );
    y3 = point_get_ly( p3, pr->adj );
    
    draw_line( drawable, x1, y1, x2, y2, r, g, b );
    draw_line( drawable, x1, y1, x3, y3, r, g, b );
    draw_line( drawable, x3, y3, x2, y2, r, g, b );
}






gboolean tris_draw_tree( GNode *node, gpointer data ) {
    // recurse through tree
    
    GList * cl;
    Tri *tn;
    Mesh* m = (Mesh*)node->data;
    Pair_drawable_adj *pr;
    float loc_x, loc_y, loc_z;
    Point *p1, *p2, *p3;
    
    pr = (Pair_drawable_adj *) data;

    if( m == NULL || pr == NULL )
        return FALSE;

    loc_x = loc_y = loc_z = 0.0;
    if( m->location != NULL ) {
        loc_x = m->location->v[0];
        loc_y = m->location->v[1];
        loc_z = m->location->v[2];
    }
    
    for ( cl = m->triangles; cl != NULL; cl = cl->next ) {
        
        
        tn = ( Tri * ) cl->data;
        
        
        p1 = point_new();
        p2 = point_new();
        p3 = point_new();
        
        p1->v[0] = tn->vertices[0]->v[0] + loc_x;
        p1->v[1] = tn->vertices[0]->v[1] + loc_y;
        p1->v[2] = tn->vertices[0]->v[2] + loc_z;
        p2->v[0] = tn->vertices[1]->v[0] + loc_x;
        p2->v[1] = tn->vertices[1]->v[1] + loc_y;
        p2->v[2] = tn->vertices[1]->v[2] + loc_z;
        p3->v[0] = tn->vertices[2]->v[0] + loc_x;
        p3->v[1] = tn->vertices[2]->v[1] + loc_y;
        p3->v[2] = tn->vertices[2]->v[2] + loc_z;

        draw_tri( pr, p1, p2, p3, 65535, 65535, 65535 );

        free( p1 );
        free( p2 );
        free( p3 );
    }

    
    return FALSE;  /* we want it to continue... */
}



void tris_draw( GdkDrawable *drawable, gadj *adj ) {

    Tri *tn;
    Pair_drawable_adj pr;
    
    pr.drawable = drawable;
    pr.adj = adj;

    //render the surfaces for all meshes under current_node
    if( current_node != NULL )
        g_node_traverse( current_node, G_IN_ORDER, G_TRAVERSE_ALL, -1,
                         tris_draw_tree, (gpointer)&pr );

    if( current_mesh != NULL && selected_tri != NULL ) {
        float loc_x, loc_y, loc_z;
        Point *p1, *p2, *p3;
        Mesh* m = current_mesh;

        loc_x = loc_y = loc_z = 0.0;
        tn = selected_tri;

        if( m->location != NULL ) {
            loc_x = m->location->v[0];
            loc_y = m->location->v[1];
            loc_z = m->location->v[2];
        }
        p1 = point_new();
        p2 = point_new();
        p3 = point_new();
        
        p1->v[0] = tn->vertices[0]->v[0] + loc_x;
        p1->v[1] = tn->vertices[0]->v[1] + loc_y;
        p1->v[2] = tn->vertices[0]->v[2] + loc_z;
        p2->v[0] = tn->vertices[1]->v[0] + loc_x;
        p2->v[1] = tn->vertices[1]->v[1] + loc_y;
        p2->v[2] = tn->vertices[1]->v[2] + loc_z;
        p3->v[0] = tn->vertices[2]->v[0] + loc_x;
        p3->v[1] = tn->vertices[2]->v[1] + loc_y;
        p3->v[2] = tn->vertices[2]->v[2] + loc_z;

        draw_tri( &pr, p1, p2, p3, 52428, 52428, 0 );

        free( p1 );
        free( p2 );
        free( p3 );
    }
}



#endif

