/* 
 * File:         view.h
 * 
 * Description:  header for view.c
 * 
 * 
 * This source code is part of kludge3d, and is released under the 
 * GNU General Public License.
 * 
 * 
 */
 
#ifndef VIEW_H
#define VIEW_H

#include <gtk/gtk.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <gtkgl/gtkglarea.h>

#include "geo.h"

#define MP_SHOW_WIREFRAME 0
#define MP_SHOW_FLAT      1
#define MP_SHOW_TEXTURED  2

#define MP_TRI_ORDER_CCW  0
#define MP_TRI_ORDER_CW   1
#define MP_TRI_ORDER_ALL  2

#define MP_VERT_SIZE_OFF     0
#define MP_VERT_SIZE_NORMAL  1
#define MP_VERT_SIZE_BIG     2

typedef struct _model_view_config model_view_config;
typedef struct _model_view model_view;

/* view configuration */
struct _model_view_config {
    GLfloat bkgnd_color[ 3 ];

    GLfloat cursor_rad;
    GLfloat vertex_rad;
    GLfloat girder_rad;

    GLfloat fov, near, far;

    GLfloat grid_color[ 3 ];
    GLfloat grid_spacing[ 3 ];

    guint vertices_visible;
    guint grid_visible;
    guint normals_visible;
//    guint enlarge_vertices;
    
    guint cursor_visible;

    int polygon_mode;
    int triangle_order;
};

struct _model_view {

    GtkWidget *gltable;
    GtkWidget *ghs;
    GtkWidget *gvs;
    GtkWidget *glarea;
    

//    model_view_config * config;


    GLfloat grid_mins[ 3 ];
    GLfloat grid_maxes[ 3 ];

    GLfloat heading, pitch, roll; 		  /* viewport orientation */
    GLfloat dist;
    GLfloat camX, camY, camZ; 	                  /* Camera location. */
    GLfloat minX, minY, minZ, maxX, maxY, maxZ;   /* used to set up orthographic views */
    
    GLdouble cursX, cursY, cursZ, cursA;


    int type;
};



extern model_view *views[4];
extern model_view_config view_config;
extern int click_mode[ 4 ];
extern Tri* partially_completed_tri;     /* if there is a triangle in the process of being constructed, it is stored here in the meantime */

extern GLfloat *cursor_glarray; 	       	/* GL arrays describing */
extern GLfloat *vertex_glarray; 		/* helper objects */
extern GLfloat *girder_glarray;



model_view* view_new( int type );
void view_kill( GtkWidget *widget, GdkEvent *event, model_view *view );

GtkWidget *create_view( int type );

void view_glarea_realize( GtkWidget *widget, gpointer user_data );
gboolean view_glarea_configure( GtkWidget *widget, GdkEventConfigure *event,
                                gpointer user_data );
gboolean view_glarea_expose( GtkWidget *widget, GdkEventExpose *event,
                             gpointer user_data );

void view_initialize_config( model_view * view );

void view_rebuild_glarrays(  );

void view_set_rendermode( int mode );
void view_set_triangleorder( int order );
void view_set_vertexsize( float size );
void view_set_fov( float fov );

#endif






